
C--   This is the file srcEvolNN.f containing the nxn evolution routines

C--   subroutine sqcEvFillA(ww,id,fun)
C--   double precision function sqcEvGetAA(ww,id,it,nf,ithrs)
C--   subroutine sqcNNallg(
C--        ww,iww,ec,iec,ff,iff,start,idim,nopt,nf,iz1,iz2,nnn,eps)
C--   subroutine sqcNNupdn(
C--      ww,iw,ec,iec,idim,ff,iff,f1,f2,ig,nopt,nf,iz1,iz2,nnn,eps)
C--   subroutine sqcNNsubg(
C--      ww,iw,ec,iec,idim,ff,iff,f1,f2,ig,nopt,nf,iz1,iz2,nnn,eps)
C--   subroutine sqcNNFtoArray(ff,iff,iz,array,idim,nnn)
C--   subroutine sqcNNsube(
C--        ww,iw,ec,iec,idim,ff,iff,ig,nopt,nf,nyg,iz1,iz2,nnn,eps)
C--   double precision function dqcNNGetEps(w,id,ny,iz)
C--   subroutine sqcNNmult(w,iw,a,ia,b,ib,n,m,nbnd,idim)
C--   subroutine sqcNNeqs(w,iw,a,ia,b,ib,n,m,idim,ierr)
C--   subroutine sqcNNSetStart(ww,start,jdim,iff,ig,nyg,iz,n)
C--   subroutine sqcNNgetVj(ww,id,iz,ig,nyg,buf)
C--   subroutine sqcNNputVj(ww,id,iz,ig,nyg,buf)
C--   subroutine sqcNNFjtoAj(ffin,aout,ny)
C--   subroutine sqcNNAjtoFj(aain,fout,ny)
C--   subroutine sqcNNFtoA(ww,id,iz1,iz2,ig,nyg)
C--   subroutine sqcNNAtoF(ww,id,iz1,iz2,ig,nyg)

C==   ==================================================================
C==   Perturbative coefficients ========================================
C==   ==================================================================

C     ================================
      subroutine sqcEvFillA(ww,id,fun)
C     ================================

C--   ww      (in)  store with type-6 tables
C--   id      (in)  global identifier of a type-6 table
C--   fun     (in)  function returning the coefficient value
C--
C--   The syntax of fun is:
C--
C--   double precision function fun(iq,nf,ithresh)
C--
C--   iq      (in)  mu2 gridpoint index (not t!)
C--   nf      (in)  number of lavours at iq
C--   ithresh (in)  0 = not at thr, +-1 at thr with larger/smaller nf

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qmaps8.inc'

      external fun

      dimension ww(*)

C--   Just to make sure
      if(.not.Lnfmap8) call sqcNfTab(0)

      nflast = nffiz2(1)
      ia     = iqcG6ij(ww,1,id)-1
      do iz = 1,nzz2
        it      = itfiz2(iz)
        iq      = it
        nf      = nffiz2(iz)
        ithresh = 0
        if(it.eq.itchm2 .or. it.eq.itbot2 .or. it.eq.ittop2) then
          if(nf.eq.nflast) then
            ithresh = -1
          elseif(nf.eq.nflast+1) then
            ithresh =  1
          else
            stop 'sqcEVFILLA inconsistent nf --> STOP'
          endif
        endif
        nflast    = nf
        ww(ia+iz) = fun(iq,nf,ithresh)
      enddo
C--   Flag table as filled
      call sqcValidate(ww,id)

      return
      end

C     =======================================================
      double precision function sqcEvGetAA(ww,id,it,nf,ithrs)
C     =======================================================

C--   ww      (in)  store with type-6 tables
C--   id      (in)  global identifier of a type-6 table
C--   it      (in)  t-grid index (same as iq)
C--   nf     (out)  number of flavours
C--   ithrs  (out)  threshold indicator
C--
C--   If it < 0 use nf = (3,4,5) instead of (4,5,6) at iqc,b,t

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qmaps8.inc'

      dimension ww(*)

C--   Just to make sure
      if(.not.Lnfmap8) call sqcNfTab(0)

C--   Find iz index
      ithrs = 0
      if(it.gt.0) then
        iz = izfitU2(it)
        if( it.eq.itchm2 .or. it.eq.itbot2 .or. it.eq.ittop2) ithrs =  1
      elseif(it.lt.0) then
        iz = izfitL2(-it)
        if(-it.eq.itchm2 .or.-it.eq.itbot2 .or.-it.eq.ittop2) ithrs = -1
      else
        stop 'sqcEvGetAA encounter it = 0'
      endif
C--   Return nf and alfa
      nf         = nffiz2(iz)
      ia         = iqcG6ij(ww,iz,id)
      sqcEvGetAA = ww(ia)

      return
      end

C==   ==================================================================
C==   Evolution routines ===============================================
C==   ==================================================================

C     =============================================================
      subroutine sqcNNallg(
     +     ww,iww,ec,iec,ff,iff,start,idim,nopt,nf,iz1,iz2,nnn,eps)
C     =============================================================

C--   Evolution from iz1 to iz2 with fixed nf over all subgrids
C--
C--   ww         (in)    weight table store
C--   iww(i,j,k) (in)    weight table identifiers of Pij for each order k
C--   ec         (in)    expansion coefficient store
C--   iec(i,j,k) (in)    coefficient table id for corresponding Pij
C--   ff         (in)    pdf store
C--   iff(i)     (in)    pdf table identifiers for i = 1,nnn
C--   start(i,j) (inout) pdf_i(jy,iz1) on entry, evolved at iz2 on exit
C--   idim       (in)    first 2 dims of iww and iec and first dim of start
C--   nopt       (in)    number of perturbative terms
C--   nf         (in)    number of flavours
C--   iz1        (in)    start of evolution
C--   iz2        (in)    end of evolution
C--   nnn        (in)    number of coupled equations
C--   eps        (out)   max |lin-quad| mid-between grid points
C--
C--   Output:  Pdf tables with values F in range iz1 to iz2
C--
C--   The array start should contain the input pdfs at iz1 on entry but
C--   will contain the evolved pdfs at iz2 on exit. In this way, the
C--   start array can serve as an input in a chain of evolutions
C--
C--   start -> sqcNNallg(iz1,iz2) -> start -> sqcNNallg(iz2,iz3), etc.

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'
      include 'qmaps8.inc'

      dimension ww(*), ff(*), ec(*)
      dimension iww(idim,idim,*), iec(idim,idim,*), iff(*)
      dimension start(mce0,0:mxg0,*),f1(idim,mxx0),f2(idim,mxx0)

C--   Just to make sure
      if(.not.Lnfmap8) call sqcNfTab(0)

      eps = 0.D0

C--   Loop over subgrids (from coarse to dense)
      do ig = nyg2,1,-1

C--     Set start values for this subgrid
        do i = 1,nnn
          do iy = 1,nyy2(0)
            f1(i,iy) = start(i,ig,iy)
          enddo
        enddo

C--     Evolve
        call sqcNNupdn(
     +  ww,iww,ec,iec,idim,ff,iff,f1,f2,ig,nopt,nf,iz1,iz2,nnn,epsi)

C--     Copy the pdfs at iz2 to the start array
        do i = 1,nnn
          do iy = 1,nyy2(0)
            start(i,ig,iy) = f2(i,iy)
            start(i, 0,iy) = f2(i,iy)
          enddo
        enddo

C--     Max deviation
        eps = max(eps,epsi)

      enddo

C--   Validate the pdfs and store the cuts
      do i = 1,nnn
        call sqcValidate(ff,iff(i))
        call sqcSetCut5(ff,iff(i),ymaxc2,tminc2,tmaxc2)
      enddo

      return
      end

C     ==============================================================
      subroutine sqcNNupdn(
     +   ww,iw,ec,iec,idim,ff,iff,f1,f2,ig,nopt,nf,iz1,iz2,nnn,eps)
C     ==============================================================

C--   Evolution from iz1 to iz2 with fixed nf in a single subgrid
C--   Downward evolution is iterative
C--
C--   ww         (in)  weight table store
C--   iw(i,j,k)  (in)  weight table identifiers of Pij for each order k
C--   ec         (in)  expansion coefficient store
C--   iec(i,j,k) (in)  coefficient table id corresponding to Pij
C--   idim       (in)  first 2 dims of iw and iec
C--   ff         (in)  pdf store
C--   iff(i)     (in)  pdf table identifiers for i = 1,nnn
C--   f1(i,j)    (in)  start pdf_i(jy) at iz1
C--   f2(i,j)    (out) end   pdf_i(jy) at iz2
C--   ig         (in)  subgrid index
C--   nopt       (in)  number of perturbative terms
C--   nf         (in)  number of flavours
C--   nyg        (in)  upper index y-subgrid after cuts
C--   iz1        (in)  start of evolution
C--   iz2        (in)  end of evolution
C--   nnn        (in)  number of coupled equations
C--   eps        (out) max |lin-quad| mid-between grid points
C--
C--   Output:  Spline coefficients A in subgrid points of all pdfs

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'

      dimension ww(*),ff(*),ec(*)
      dimension iw(idim,idim,*),iec(idim,idim,*),iff(*)
      dimension f1(idim,*),f2(idim,*)

      if(iz2.ge.iz1 .or.    !upward evolution  .or.
     +   ioy2.eq.2  .or.    !lin interpolation .or.
     +   niter6.lt.0) then  !always use current interpolation scheme
C--     Then just evolve
        call sqcNNsubg(
     +  ww,iw,ec,iec,idim,ff,iff,f1,f2,ig,nopt,nf,iz1,iz2,nnn,eps)
      else
C--     Proceed with iterative downward evolution
        if0rem = -1
        islast = -2
        ioyrem = ioy2
C--     Linear downward evolution
        ioy2 = 2
        call sqcNNsubg(
     +  ww,iw,ec,iec,idim,ff,iff,f1,f2,ig,nopt,nf,iz1,iz2,nnn,eps)
C--     Iterate?
        if(niter6.eq.0) then
C--       We do want linear downward evolution w/o iteration
           ioy2 = ioyrem
          return
        endif
C--     Now proceed with iteration
C--     Copy startvalues from iz1 to the bins if0rem and islast
        do i = 1,nnn
          call sqcTcopyType5(ff,iff(i),iz1,if0rem)
          call sqcTcopyType5(ff,iff(i),iz1,islast)
        enddo
C--     Quad upward evolution
        ioy2 = 3
        call sqcNNsubg(
     +  ww,iw,ec,iec,idim,ff,iff,f2,f1,ig,nopt,nf,iz2,iz1,nnn,eps)
        niter = 0
C--     Now iterate
        do while(niter.lt.niter6)
C--       Set new startvalue
          do i = 1,nnn
            if0 = iqcG5ijk(ff,1,if0rem,iff(i))-1
            isl = iqcG5ijk(ff,1,islast,iff(i))-1
            do iy = 1,nyy2(0)
              f1(i,iy) = ff(if0+iy) + ff(isl+iy)-f1(i,iy)
            enddo
          enddo
C--       Linear downward evolution
          ioy2 = 2
          call sqcNNsubg(
     +    ww,iw,ec,iec,idim,ff,iff,f1,f2,ig,nopt,nf,iz1,iz2,nnn,eps)
C--       Remember start value
          do i = 1,nnn
            call sqcTcopyType5(ff,iff(i),iz1,islast)
          enddo
C--       Quad upward evolution
          ioy2 = 3
          call sqcNNsubg(
     +    ww,iw,ec,iec,idim,ff,iff,f2,f1,ig,nopt,nf,iz2,iz1,nnn,eps)
          niter = niter+1
        enddo
C--     End of iteration
C--     Restore interpolation degree
        ioy2 = ioyrem
C--     Put endpoint of the evolution into the start array
        call sqcNNFtoArray(ff,iff,iz2,f2,idim,nnn)
C--     Restore the original starting pdf
        do i = 1,nnn
          call sqcTcopyType5(ff,iff(i),if0rem,iz1)
        enddo
      endif

      return
      end

C     =============================================================
      subroutine sqcNNsubg(
     +   ww,iw,ec,iec,idim,ff,iff,f1,f2,ig,nopt,nf,iz1,iz2,nnn,eps)
C     =============================================================

C--   Evolution from iz1 to iz2 with fixed nf in a single subgrid
C--
C--   ww         (in)  weight table store
C--   iw(i,j,k)  (in)  weight table identifiers of Pij for each order k
C--   ec         (in)  expansion coefficient store
C--   iec(i,j,k) (in)  coefficient table id corresponding to Pij
C--   idim       (in)  first 2 dims of iw and iec
C--   ff         (in)  pdf store
C--   iff(i)     (in)  pdf table identifiers for i = 1,nnn
C--   f1(i,j)    (in)  start pdf_i(jy) at iz1
C--   f2(i,j)    (out) end   pdf_i(jy) at iz2
C--   ig         (in)  subgrid index
C--   nopt       (in)  number of perturbative terms
C--   nf         (in)  number of flavours
C--   nyg        (in)  upper index y-subgrid after cuts
C--   iz1        (in)  start of evolution
C--   iz2        (in)  end of evolution
C--   nnn        (in)  number of coupled equations
C--   eps        (out) max |lin-quad| mid-between grid points
C--
C--   Output:  Spline coefficients A in subgrid points of all pdfs

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'

      dimension ww(*),ff(*),ec(*)
      dimension iw(idim,idim,*),iec(idim,idim,*),iff(*)
      dimension startg(mxx0,mce0)
      dimension f1(idim,*),f2(idim,*)

C--   Number of subgrid points after cuts
      nyg = iqcIyMaxG(iymac2,ig)

C--   Pick the subgrid points of f1 and put these into startg
      do i = 1,nnn
        do iyg = 1,nyg
          iy0 = iy0fiyg2(iyg,ig)
          startg(iyg,i) = f1(i,iy0)
        enddo
      enddo

C--   Convert startvalues from F to A and put these into the subgrid ig
      call sqcNNSetStart(ff,startg,mxx0,iff,ig,nyg,iz1,nnn)

C--   Evolve subgrid and fill with A values for all pdfs
      call sqcNNsube(
     +      ww,iw,ec,iec,idim,ff,iff,ig,nopt,nf,nyg,iz1,iz2,nnn,eps)

C--   Convert the subgrid of the pdf tables from A to F
      do i = 1,nnn
        call sqcNNAtoF(ff,iff(i),iz1,iz2,ig,nyg)
      enddo

C--   Copy the pdfs at iz2 to the f2 array
      call sqcNNFtoArray(ff,iff,iz2,f2,idim,nnn)

      return
      end

C     ==================================================
      subroutine sqcNNFtoArray(ff,iff,iz,array,idim,nnn)
C     ==================================================

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'

C--   Copy bin iz of nnn pdfs in the store ff to an array

C--   ff         (in)    pdf store
C--   iff(i)     (in)    pdf table identifiers for i = 1,nnn
C--   iz         (in)    z-bin to copy
C--   array(i,j) (out)   pdf_i(jy) at iz on exit
C--   idim       (in)    first dim of array
C--   nnn        (in)    number of pdfs to copy

      dimension ff(*)
      dimension iff(*)
      dimension array(idim,*)

      do i = 1,nnn
        ia0 = iqcG5ijk(ff,1,iz,iff(i))-1
        do iy = 1,nyy2(0)
          array(i,iy) = ff(ia0+iy)
        enddo
      enddo

      return
      end

C     ==============================================================
      subroutine sqcNNsube(
     +      ww,iw,ec,iec,idim,ff,iff,ig,nopt,nf,nyg,iz1,iz2,nnn,eps)
C     ==============================================================

C--   Evolution from iz1 to iz2 with fixed nf in a single subgrid
C--
C--   ww         (in)  weight table store
C--   iw(i,j,k)  (in)  weight table identifiers of Pij for each order k
C--   ec         (in)  expansion coefficient store
C--   iec(i,j,k) (in)  coefficient table id corresponding to Pij
C--   idim       (in)  first 2 dims of iw and iec
C--   ff         (in)  pdf store
C--   iff(i)     (in)  pdf table identifiers for i = 1,nnn
C--   ig         (in)  subgrid index
C--   nopt       (in)  number of perturbative terms
C--   nf         (in)  number of flavours
C--   nyg        (in)  upper index y-subgrid after cuts
C--   iz1        (in)  start of evolution
C--   iz2        (in)  end of evolution
C--   nnn        (in)  number of coupled equations
C--   eps        (out) max |lin-quad| mid-between grid points
C--
C--   Output:  Spline coefficients A in subgrid points of all pdfs

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'

      dimension ww(*),ff(*),ec(*)
      dimension iw(idim,idim,*),iec(idim,idim,*),iff(*)

      dimension vvv(mce0*mce0*mxx0),aaa(mce0*mxx0),bbb(mce0*mxx0)
      dimension ivv(mce0,mce0),iaa(mce0),ibb(mce0)
      dimension sss(mce0*mce0*mxx0),hhh(mce0*mxx0)
      dimension iss(mce0,mce0),ihh(mce0)
      dimension umat(mxx0),usum(mxx0)

C--   Initialization
C--   --------------
      iy1 = 1
      do i = 1,nyg
        umat(i) = 0.D0
        usum(i) = 0.D0
      enddo
      do i = 1,nnn*nnn*nyg
        sss(i) = 0.D0
      enddo
      do i = 1,nnn*nyg
        hhh(i) = 0.D0
      enddo
C--   Adresses
      ih0 = 0
      is0 = 0
      do i = 1,nnn
        ibb(i) = ih0+1
        ih0    = ih0+nyg
        do j = 1,nnn
          iss(i,j) = is0+1
          is0      = is0+nyg
       enddo
      enddo
C--   Direction of evolution (isign) and first point after it1 (next)
      isign = 1
      next  = iz1+1
      if(iz2.lt.iz1) then 
        isign = -1
        next  = iz1-1
      endif

C--   Calculate vector b at input scale iz1
C--   -------------------------------------
C--   Grid spacing delta = z(next)-z (divided by 2)
      delt = 0.5*abs(zgrid2(next)-zgrid2(iz1))
C--   Transformation matrix divided by delta (U)        
      do i = 1,nmaty2(ioy2)
        umat(i)   = smaty2(i,ioy2)/delt
      enddo
C--   Find t-index
      it1 = itfiz2(iz1)
C--   Weight matrix and  V = U+W at t1
      ityp = 0
      iw0  = 0
      if0  = 0
      do i = 1,nnn
        do j = 1,nnn
          ityp = ityp+1
          do iy = 1,nyg
            if(i.eq.j) then 
              vvv(iw0+iy) = umat(iy)
            else
              vvv(iw0+iy) = 0.D0
            endif
          enddo
          do k = 1,nopt
            if(iw(i,j,k).ne.0) then
              as = ec(iqcG6ij(ec,iz1,iec(i,j,k)))
              ia = iqcGaddr(ww,1,it1,nf,ig,iw(i,j,k))-1
              do iy = 1,nyg
                ia = ia+1
                vvv(iw0+iy) = vvv(iw0+iy) + isign*as*ww(ia)
              enddo
            endif
          enddo
          ivv(i,j) = iw0+1
          iw0      = iw0+nyg
        enddo
C--     Pick the A values of the subgrid
        ia0 = iqcG5ijk(ff,1,iz1,iff(i))-1
        do iy = 1,nyg
          iy0         = iy0fiyg2(iy,ig)
          aaa(if0+iy) = ff(ia0+iy0)
        enddo
        iaa(i) = if0+1
        if0    = if0+nyg
      enddo
C--   Now make the matrix multiplication Va = b
      call sqcNNmult(vvv,ivv,aaa,iaa,bbb,ibb,nnn,nyg,nyg,mce0)

C--   Evolution loop over t
C--   ---------------------
      do iz = next,iz2,isign
C--     Find t-index
        it = itfiz2(iz)
C--     Weight matrix and V matrix at t
        iw0  = 0
        if0  = 0
        ityp = 0
        do i = 1,nnn
          do j = 1,nnn
            ityp = ityp+1
            do iy = 1,nyg
              if(i.eq.j) then 
                vvv(iw0+iy) = umat(iy)
              else
                vvv(iw0+iy) = 0.D0
              endif
            enddo
            do k = 1,nopt
              if(iw(i,j,k).ne.0) then
                as = ec(iqcG6ij(ec,iz,iec(i,j,k)))
                ia = iqcGaddr(ww,1,it,nf,ig,iw(i,j,k))-1
                do iy = 1,nyg
                  ia = ia+1
                  vvv(iw0+iy) = vvv(iw0+iy) - isign*as*ww(ia)
                enddo
              endif
            enddo
            ivv(i,j) = iw0+1
            iw0      = iw0+nyg
          enddo
        enddo
C--     Solve V a = b
        call sqcNNeqs(vvv,ivv,aaa,iaa,bbb,ibb,nnn,nyg,mce0,ierr)
C--     Store subgrid A coefficients in the pdf grid
        do i = 1,nnn
          ia0 = iqcG5ijk(ff,1,iz,iff(i))-1
          do iy = 1,nyg
            iy0         = iy0fiyg2(iy,ig)
            ff(ia0+iy0) = aaa(iaa(i)+iy-1)
          enddo
        enddo
C--     Update b for the next iteration: not at last iteration
        if(iz.ne.iz2) then
C--       Grid spacing delta = z(next)-z (divided by 2)
          delt = 0.5*abs(zgrid2(iz+isign)-zgrid2(iz))
C--       Calculate U(next) and S = U(next) + U(now); band only       
          do i = 1,nmaty2(ioy2)
            sbnext    = smaty2(i,ioy2)/delt
            usum(i)   = umat(i)+sbnext
            umat(i)   = sbnext
          enddo
C--       Make nnn*nnn copies of S
          jss = 0
          do i = 1,nnn
            ihh(i) = (i-1)*nyg+1
            do j = 1,nnn
              iss(i,j) = jss+1
              do k = 1,nmaty2(ioy2)
                jss = jss+1
                if(i.eq.j) then
                  sss(jss) = usum(k)
                else
                  sss(jss) = 0.D0
                endif
              enddo
            enddo
          enddo
C--       Now make matrix multiplication h = Sa 
          call sqcNNmult(sss,iss,aaa,iaa,hhh,ihh,nnn,nyg,nmaty2(ioy2),
     +                   mce0)
C--       Update b(next) = h(now)-b(now)
          do i = 1,nnn
            ia = ihh(i)-1
            do iy = 1,nyg
              ia      = ia+1
              bbb(ia) = hhh(ia)-bbb(ia)
            enddo
          enddo
        endif

C--   End of loop over t
      enddo

C--   Max deviation quad from lin spline
      eps = 0.D0
      do i = 1,nnn
        eps = max(eps,dqcNNGetEps(ff,iff(i),nyg,iz1))
        eps = max(eps,dqcNNGetEps(ff,iff(i),nyg,iz2))
      enddo

C--   Thats it...

      return
      end

C     =================================================
      double precision function dqcNNGetEps(w,id,ny,iz)
C     =================================================

C--   Get max deviation quad-lin at midpoints
C--   This works on a subgrid with A coefficients
C--
C--   w       (in)  : Local store with pdf tables
C--   id      (in)  : Pdf identifier in global format
C--   ny      (in)  : upper loop index in y
C--   iz      (in)  : z-grid index

      implicit double precision (a-h,o-z)
      
      include 'qcdnum.inc'
      include 'qgrid2.inc'

      dimension w(*)
      dimension epsi(mxx0)
      
      dqcNNGetEps = 0.D0
      if(ioy2.ne.3) return                

C--   Base address
      ia = iqcG5ijk(w,1,iz,id)
C--   Now get vector of deviations
      call sqcDHalf(ioy2,w(ia),epsi,ny)
C--   Max deviation
      do iy = 1,ny
        dqcNNGetEps = max(dqcNNGetEps,abs(epsi(iy)))
      enddo

      return
      end


C--   ==================================================
      subroutine sqcNNmult(w,iw,a,ia,b,ib,n,m,nbnd,idim)
C--   ==================================================

C--   Multiply  W a = b
C--
C--   where W is an n*n array of lower band Toeplitz matrices
C--   of dimension m, and bandwith nbnd, a is a vector of n arrays of
C--   dimension m and b is a result vector of n arrays of dimension m.
C--   A Toeplitz is stored as a vector of dimension nbnd
C--
C--   |w1             |
C--   |w2 w1          |
C--   |w3 w2 w1       |  with, in this example, m = 5 and nbnd = 3 
C--   |   w3 w2 w1    |
C--   |      w3 w2 w1 |
C--
C--   (in)  w          Linear array with Toeplitz matrices W_alfbet           
C--   (in)  iw(n,n)    Adresses of first element of each W_alfbet
C--   (out) a          Linear array with input vectors a
C--   (in)  ia(n)      Adresses of first element of each a_alf
C--   (in)  b          Linear array with result vectors b
C--   (in)  ib(n)      Adresses of first element of each b_alf 
C--   (in)  n          Number matrices  alf,bet = 1,...,n
C--   (in)  m          Dimension of each Toeplitz matrix ix = 1,...,m
C--   (in)  nbnd       Matrix bandwidth
C--   (in)  idim       First dimension of iw declared in the calling routine

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'

      dimension w(*),a(*),b(*)
      dimension iw(idim,*),ia(*),ib(*)

C--   Loop over all elements ix of the pdfs
      do ix = 1,m
C--     Loop over all pdfs ialf
        do ialf = 1,n
          balfix = 0.D0
          j1 = max(ix+1-nbnd,1)
C--       Loop over the elements jx
          do jx = j1,ix
C--         Loop over all pdfs ibet
            do ibet = 1,n
              walfbetixjx = w(iw(ialf,ibet)+ix-jx)
              abetjx      = a(ia(ibet)+jx-1)
              balfix      = balfix + walfbetixjx*abetjx
            enddo
          enddo
          b(ib(ialf)+ix-1) = balfix
        enddo
      enddo

      return
      end

C     =================================================
      subroutine sqcNNeqs(w,iw,a,ia,b,ib,n,m,idim,ierr)
C     =================================================

C--   Solves coupled triangular equations
C--
C--            W a = b
C--
C--   where W is an n*n array of lower triangular Toeplitz matrices
C--   of dimension m, a is a solution vector of n arrays of dimension m
C--   and b is a right-hand side vector of n arrays of dimension m.
C--   A Toeplitz matrix T is stored as a vector of dimension m: 
C--
C--                                          | t1  0  0  0  0 |
C--                                          | t2 t1  0  0  0 |
C--   T(i,j<=i) = t(i-j+1)  e.g. for 5 x 5:  | t3 t2 t1  0  0 |
C--                                          | t4 t3 t2 t1  0 |
C--                                          | t5 t4 t3 t2 t1 |
C--
C--   (in)  w          Linear array with Toeplitz matrices W_alfbet           
C--   (in)  iw(n,n)    Adresses of first element of each W_alfbet
C--   (out) a          Linear array with solution vectors a
C--   (in)  ia(n)      Adresses of first element of each a_alf
C--   (in)  b          Linear array with right-hand vectors b
C--   (in)  ib(n)      Adresses of first element of each b_alf 
C--   (in)  n          Number of coupled equations  alf,bet = 1,...,n
C--   (in)  m          Dimension of each Toeplitz matrix ix = 1,...,m
C--   (in)  idim       First dimension of iw declared in the calling routine
C--   (out) ierr       0 = OK, #0 = Singularity encountered

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'

      dimension w(*), a(*), b(*)
      dimension iw(idim,*),ia(*),ib(*)
      dimension vv(mce0,mce0),bb(mce0)
      dimension iwork(mce0)

C--   Loop over all elements ix of the pdfs
      do ix = 1,m
C--     Loop over all pdfs ialf
        do ialf = 1,n
          sumalfix = 0.D0
C--       Loop over the elements jx
          do jx = 1,ix-1
C--         Loop over all pdfs ibet
            do ibet = 1,n
              valfbetixjx = w(iw(ialf,ibet)+ix-jx)
              abetjx      = a(ia(ibet)+jx-1)
              sumalfix    = sumalfix + valfbetixjx*abetjx
C--         End of loop over all pdfs ibet
            enddo
C--       End of loop over elements jx
          enddo
          balfix   = b(ib(ialf)+ix-1)
          bb(ialf) = balfix-sumalfix
          do ibet = 1,n
            vv(ialf,ibet) = w(iw(ialf,ibet))
          enddo
C--     End of loop over all pdfs ialf
        enddo
C--     Now solve the nxn matrix equation for each ix
        call smb_dmeqn(n,vv,mce0,iwork,ierr,1,bb)
        if(ierr.ne.0) stop 'sqcNNeqs singular matrix encountered'
C--     Store the result
        do ialf = 1,n
          a(ia(ialf)+ix-1) = bb(ialf)
        enddo 
C--   End of loop over all elements ix of the pdfs
      enddo

      return
      end

C     =======================================================
      subroutine sqcNNSetStart(ww,start,jdim,iff,ig,nyg,iz,n)
C     =======================================================

C--   Convert array with starting F values to A values and disperse
C--   them on the subgrid of a set of pdf tables
C--
C--   ww           (in)   Pdf store
C--   start(iy,i)  (in)   Array with subgrid start values of pdf i
C--   jdim         (in)   First dim of start declared in calling routine
C--   iff(i)       (in)   Table identifier of pdf i
C--   ig           (in)   Subgrid index
C--   nyg          (in)   Number of subgrid points after cuts
C--   iz           (in)   Z-point where to put the A values
C--   n            (in)   Number of pdfs

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'

      dimension ww(*), start(jdim,*)
      dimension iff(*)
      dimension acoef(mxx0)

C--   Loop over the pdfs
      do i = 1,n
C--     Convert F values to A values
        call sqcNNFjtoAj(start(1,i),acoef,nyg)
C--     Copy A values into the pdf table
        call sqcNNputVj(ww,iff(i),iz,ig,nyg,acoef)
      enddo

      return
      end

C     ==========================================
      subroutine sqcNNgetVj(ww,id,iz,ig,nyg,buf)
C     ==========================================

C--   Contract the subgrid points of a pdf table into the array buf 

C--   ww     (in)  Pdf store
C--   id     (in)  Pdf identifier
C--   iz     (in)  Z-bin index
C--   ig     (in)  Subgrid index
C--   nyg    (in)  Number of subgrid points after cuts
C--   buf    (out) Array with values of all subbgrid points

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'

      dimension ww(*),buf(*)

      ia = iqcG5ijk(ww,1,iz,id)-1
      do iyg = 1,nyg
        iy0      = iy0fiyg2(iyg,ig)
        buf(iyg) = ww(ia+iy0)
      enddo

      return
      end

C     ==========================================
      subroutine sqcNNputVj(ww,id,iz,ig,nyg,buf)
C     ==========================================

C--   Disperse the array buf over the subgrid points of a pdf table

C--   ww     (in)  Pdf store
C--   id     (in)  Pdf identifier
C--   iz     (in)  Z-bin index
C--   ig     (in)  Subgrid index
C--   nyg    (in)  Number of subgrid points after cuts
C--   buf    (in)  Array with values of all subbgrid points

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'

      dimension ww(*),buf(*)

      ia = iqcG5ijk(ww,1,iz,id)-1
      do iyg = 1,nyg
        iy0        = iy0fiyg2(iyg,ig)
        ww(ia+iy0) = buf(iyg)
      enddo

      return
      end

C     ====================================
      subroutine sqcNNFjtoAj(ffin,aout,ny)
C     ====================================

C--   Convert an array of F values to an array of A values
C--   The F values must come from an equidistant (sub)grid

C--   ffin   (in)   Array of ny F values
C--   aout   (out)  Array of ny A values
C--   ny     (in)   Number of points in ffin and aout 

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      
      dimension ffin(*), aout(*)

C--   Convert pdf values to spline coefficients
      call sqcNSeqs(smaty2(1,ioy2),nmaty2(ioy2),aout,ffin,ny)

      return
      end

C     ====================================
      subroutine sqcNNAjtoFj(aain,fout,ny)
C     ====================================

C--   Convert an array of A values to an array of F values
C--   The A values must come from an equidistant (sub)grid

C--   aain   (in)   Array of ny A values
C--   fout   (out)  Array of ny F values
C--   ny     (in)   Number of points in aain and fout 

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      
      dimension aain(*), fout(*)
      
C--   Convert spline coefficients to pdf values
      call sqcNSmult(smaty2(1,ioy2),nmaty2(ioy2),aain,fout,ny)

      return
      end

C     ==========================================
      subroutine sqcNNFtoA(ww,id,iz1,iz2,ig,nyg)
C     ==========================================

C--   Convert from F to A all subgrid points between iz1 to iz2
C--
C--   ww     (in)   Pdf store
C--   id     (in)   Pdf identifier
C--   iz1    (in)   Lower limit of z-range
C--   iz2    (in)   Upper limit of z-range
C--   ig     (in)   Subgrid identifier
C--   nyg    (in)   Numbdr of y-points after cuts
C--
C--   Thus the subgrid contains F values on input and A values on exit  

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'

      dimension ww(*)
      dimension buff(mxx0), bufa(mxx0)

      if(iz2.ge.iz1) then
        idz = 1
      else
        idz = -1
      endif
      do iz = iz1,iz2,idz
        call sqcNNgetVj(ww,id,iz,ig,nyg,buff)
        call sqcNNFjtoAj(buff,bufa,nyg)
        call sqcNNputVj(ww,id,iz,ig,nyg,bufa)
      enddo

      return
      end

C     ==========================================
      subroutine sqcNNAtoF(ww,id,iz1,iz2,ig,nyg)
C     ==========================================

C--   Convert from A to F all subgrid points between iz1 to iz2
C--
C--   ww     (in)   Pdf store
C--   id     (in)   Pdf identifier
C--   iz1    (in)   Lower limit of z-range
C--   iz2    (in)   Upper limit of z-range
C--   ig     (in)   Subgrid identifier
C--   nyg    (in)   Numbdr of y-points after cuts
C--
C--   Thus the subgrid contains A values on input and F values on exit

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'

      dimension ww(*)
      dimension bufa(mxx0), buff(mxx0)

      if(iz2.ge.iz1) then
        idz = 1
      else
        idz = -1
      endif
      do iz = iz1,iz2,idz
        call sqcNNgetVj(ww,id,iz,ig,nyg,bufa)
        call sqcNNAjtoFj(bufa,buff,nyg)
        call sqcNNputVj(ww,id,iz,ig,nyg,buff)
      enddo

      return
      end

