
C--   This is the file srcInterpolation.f containing interpolation routines 

C--   subroutine sqcStInterp(stfun,idg,xlist,qlist,flist,n,ierr)
C--   subroutine sqcStIntMpt(stfun,idg,xlist,qlist,flist,n,ierr)
C--   subroutine sqcWeedIt(yin,tin,nin,yout,tout,infout,nout)
C--   subroutine sqcIntLst(w,id,iy1,iy2,iz1,iz2,y,t,f,n)
C--   subroutine sqcIntPol(w,id,iy1,iy2,iz1,iz2,y,t,f)
C--   subroutine sqcZmesh(y,t,margin,iymi,iyma,izmi,izma)
C--   subroutine sqcMarkit(mark,ylst,tlst,margin,iy1,iy2,iz1,iz2,n)
C--   subroutine sqcPolint(xa,ya,n,x,y,dy)
C--   subroutine sqcPolin2(xa,nx,ya,ny,za,x,y,z)
C--   double precision function dqcPolint(x,xi,yi,n)
C--   double precision function dqcPolin2(xa,nx,ya,ny,za,x,y)
C--   subroutine sqcGetABC(u,v,w,delta,a,b,c)


C     ==========================================================
      subroutine sqcStInterp(stfun,idg,xlist,qlist,flist,n,ierr)
C     ==========================================================

C--   Interpolation of stfun(ix,iq) for arbitrary large n
C--   This routine does not suffer from the mpt0 limit (by buffering)
C--
C--   stfun      (in)  :     Function of (ix,iq), declared external
C--   idg        (in)  :     Global address of temporary pdf table
C--   xlist(n)   (in)  :     List of x-values
C--   qlist(n)   (in)  :     List of qmu2 values
C--   flist(n)   (out) :     List of interpolated structure functions
C--   n          (in)  :     Number of items in the list
C--   ierr       (out) :     -1   Count mpt0 exceeded (never occurs)
C--                           0   OK
C--                           1   One or more x,q outside grid 
C--

      implicit double precision(a-h,o-z)

      include 'qcdnum.inc'
      
      external stfun
      
      dimension xx(mpt0),qq(mpt0)
      dimension xlist(*), qlist(*), flist(*)
      
C--   Fill output array flist in batches of mpt0 words
      ipt = 0
      jj  = 0      
      do i = 1,n
        ipt     = ipt+1
        xx(ipt) = xlist(i)
        qq(ipt) = qlist(i)
        if(ipt.eq.mpt0) then
          call sqcStIntMpt(stfun,idg,xx,qq,flist(jj*mpt0+1),mpt0,ierr)
          ipt = 0
          jj  = jj+1
        endif
      enddo
C--   Flush remaining ipt points
      if(ipt.ne.0) then
        call sqcStIntMpt(stfun,idg,xx,qq,flist(jj*mpt0+1),ipt,ierr)
      endif

      return
      end

C     ==========================================================
      subroutine sqcStIntMpt(stfun,idg,xlist,qlist,flist,n,ierr)
C     ==========================================================

C--   Interpolation of stfun(ix,iq) but with n <= mpt0
C--
C--   stfun      (in)  :     Function of (ix,iq), declared external
C--   igt        (in)  :     Global address of temporary pdf table
C--   xlist(n)   (in)  :     List of x-values
C--   qlist(n)   (in)  :     List of qmu2 values
C--   flist(n)   (out) :     List of interpolated structure functions
C--   n          (in)  :     Number of items in the list
C--   ierr       (out) :     -1   Count mpt0 exceeded
C--                           0   OK
C--                           1   One or more x,q outside grid  

      implicit double precision(a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      external stfun

      dimension xlist(*), qlist(*), flist(*)
      dimension yin(mpt0),tin(mpt0),yout(mpt0),tout(mpt0),infout(mpt0)
      dimension iy1(mpt0),iy2(mpt0),iz1(mpt0),iz2(mpt0),fout(mpt0)
      logical   mark(0:mxx0,0:mqq0+7)

C--   Check
      if(n.gt.mpt0) then
         ierr = -1
         return
      endif
C--   Convert to y-t
      do i = 1,n
        yin(i) = -log(xlist(i))
        tin(i) =  log(qlist(i))
      enddo
C--   Weed points outside grid or cuts
      call sqcWeedIt(yin,tin,n,yout,tout,infout,nout)
C--   Not all points inside grid
      ierr = 0
      if(nout.ne.n) ierr = 1                       
C--   Mark grid points
      margin = 1
      call sqcMarkit(mark,yout,tout,margin,iy1,iy2,iz1,iz2,nout)
C--   Store structure function in scratch buffer
      do it = 1,ntt2
        iz = izfit2(it)
        ia = iqcG5ijk(stor7,1,iz,idg)-1
        do iy = 1,nyy2(0)
          ia = ia+1
          if(mark(iy,iz)) then
            ix        = nyy2(0)+1-iy
            stor7(ia) = stfun(ix,it)
          endif
        enddo    
      enddo
C--   Interpolate
      call sqcIntLst(stor7,idg,iy1,iy2,iz1,iz2,yout,tout,fout,nout)
C--   Preset
      do i = 1,n
        flist(i) = qnull6
      enddo
C--   Return interpolated result
      do i = 1,nout
        flist(infout(i)) = fout(i)
      enddo                

      return
      end

C     =======================================================
      subroutine sqcWeedIt(yin,tin,nin,yout,tout,infout,nout)
C     =======================================================

C--   Weed points outside x-mu2 grid, including x = 1
C--
C--   yin(i),tin(i)    (in)  : list of y-t points
C--   nin              (in)  : number of points in yin,tin
C--   yout(j),tout(j)  (out) : list of y-t points inside grid
C--   infout(j)        (out) : input index i from output index j
C--   nout             (out) : number of points in yout,tout

      implicit double precision (a-h,o-z)
      
      logical lqcInside
      
      dimension yin(*),tin(*),yout(*),tout(*),infout(*)
      
      nout  = 0
      do i = 1,nin
        xx = exp(-yin(i))
        qq = exp(tin(i))        
        if(lqcInside(xx,qq)) then
          nout         = nout+1   
          yout(nout)   = yin(i)
          tout(nout)   = tin(i)
          infout(nout) = i
        endif
      enddo

      return
      end

C     ==================================================
      subroutine sqcIntLst(w,id,iy1,iy2,iz1,iz2,y,t,f,n)
C     ==================================================

C--   List of polynomial interpolations on a table 
C--
C--   w          (in)  :  workspace
C--   id         (in)  :  global pdf table identifioer
C--   iy1(i)     (in)  :  lower limit of interpolation mesh in y
C--   iy2(i)     (in)  :  upper limit of interpolation mesh in y
C--   iz1(i)     (in)  :  lower limit of interpolation mesh in z
C--   iz2(i)     (in)  :  upper limit of interpolation mesh in z
C--   y(i),t(i)  (in)  :  interpolation point
C--   f(i)       (out) :  value of the interpolation polynomial P(y,t)
C--   n          (in)  :  number of items in the list

      implicit double precision (a-h,o-z)

      dimension w(*)
      
      dimension iy1(*),iy2(*),iz1(*),iz2(*),y(*),t(*),f(*)
      
      do i = 1,n
        call sqcIntPol(w,id,iy1(i),iy2(i),iz1(i),iz2(i),
     +                 y(i),t(i),f(i))
      enddo
      
      return
      end

C     ================================================
      subroutine sqcIntPol(w,id,iy1,iy2,iz1,iz2,y,t,f)
C     ================================================

C--   Polynomial interpolation on a table 
C--
C--   w    (in)  :  workspace
C--   id   (in)  :  pdf table identifier in global formast
C--   iy1  (in)  :  lower limit of interpolation mesh in y
C--   iy2  (in)  :  upper limit of interpolation mesh in y
C--   iz1  (in)  :  lower limit of interpolation mesh in z
C--   iz2  (in)  :  upper limit of interpolation mesh in z
C--   y,t  (in)  :  interpolation point
C--   f    (out) :  value of the interpolation polynomial P(y,t)
C--
C--   NB: y,t should be within the interpolation mesh, otherwise
C--       we have extrapolation. The routine sqcZmesh figures out
C--       what the interpolation mesh boundaries are for given y,t

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'

      dimension w(*),work(iom0)

      ny = iy2-iy1+1
      nz = iz2-iz1+1
C--   Do nz interpolations in y
      j  = 0
      do iz = iz1,iz2
        j       = j+1
        iadr    = iqcG5ijk(w,iy1,iz,id)
        work(j) = dqcPolint(y,ygrid2(iy1),w(iadr),ny)
      enddo
C--   Do one interpolation in t
      f = dqcPolint(t,zgrid2(iz1),work,nz)
      
      return
      end
      
C     ===================================================
      subroutine sqcZmesh(y,t,margin,iymi,iyma,izmi,izma)
C     ===================================================

C--   Find the boundaries of a iosp*3 interpolation mesh in (iy,iz) 
C--   around y and t. This is not entirely trivial because the mesh
C--   should not  cross the grid boundaries or thresholds. It may
C--   happen that the mesh is too wide to fit between two thresholds or
C--   between a threshold and a boundary in which case we must adjust
C--   the width of the mesh. The interpolation point (y,t) must be
C--   inside the boundaries of the qcdnum y-t grid.
C--   The margin parameter defines the distance between the upper edge
C--   of the mesh and a threshold. Margin = 0 the upper edge can
C--   touch the threshold, margin = 1 the edge stays one point away
C--   from the threshold. Since the thresholds are at least two points
C--   apart, there should be no problem with margin = 1.

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qmaps8.inc'

      if(margin.ne.0 .and. margin.ne.1)
     +   stop  'sqcZmesh: invalid margin'

C--   Get bin in y
      iy = iqcFindIy(y)
      if(iy.eq.-1) stop 'sqcZmesh: y out of range ---> STOP'
C--   Mesh width is equal to the spline order ioy2
      iyma = min(iy+ioy2-1,nyy2(0))
      iymi = max(iyma-ioy2+1,0)

C--   Get bin in t
      it = iqcItfrmt(t)
      if(it.eq.0)  stop 'sqcZmesh: t out of range ---> STOP'
      iz  = izfit2(it)
      nf  = nffiz2(iz)
      iz1 = izminf8(nf)
      iz2 = izmaxf8(nf)
*mb      iz1 = 0
*mb      iz2 = 0
*mbC--   Find upper and lower limit of the nf range
*mb      do i = 1,nlist8
*mb        if(nflist8(i).eq.nf) then
*mb          iz1 = izmin8(i)
*mb          iz2 = izmax8(i)
*mb        endif
*mb      enddo
*mb      if(iz1.eq.0 .or. iz2.eq.0) 
*mb     &   stop 'sqcZmesh: zero nf range  ---> STOP'
C--   Mesh width is always 3 for interpolation in t
      izma = min(iz+2,iz2-margin)
      izmi = max(izma-2,iz1)
      if(izma.le.izmi)
     +  stop 'sqcZmesh: zero or negative mesh width in t ---> STOP'
      return
      end
      
C     =============================================================
      subroutine sqcMarkit(mark,ylst,tlst,margin,iy1,iy2,iz1,iz2,n)
C     =============================================================

C--   Process list of interpolation points and mark gridpoints
C--
C--   mark    (out)  : logical table containing the marks in y and z
C--   ylst(n) (in)   : list of y points
C--   tlst(n) (in)   : list of t points
C--   margin  (in)   : distance away from thresholds [0,1]
C--   iy1(n)  (out)  : list of lower mesh limits in y
C--   iy2(n)  (out)  : list of upper mesh limits in y
C--   iz1(n)  (out)  : list of lower mesh limits in z
C--   iz2(n)  (out)  : list of upper mesh limits in z
C--   n       (in)   : number of points in the list

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      dimension ylst(*),tlst(*),iy1(*),iy2(*),iz1(*),iz2(*)
      logical   mark(0:mxx0,0:mqq0+7)

C--   Initialize
      do j = 0,mqq0+7
        do i = 0,mxx0
          mark(i,j) = .false.
        enddo
      enddo
C--   Loop over interpolation points
      do i = 1,n
        call sqcZmesh(ylst(i),tlst(i),margin,iya,iyb,iza,izb)
        iy1(i) = iya
        iy2(i) = iyb
        iz1(i) = iza
        iz2(i) = izb
        do iz = iza,izb
          do iy = iya,iyb
            mark(iy,iz) = .true.
          enddo
        enddo
      enddo

      return 
      end

C     ====================================
      subroutine sqcPolint(xa,ya,n,x,y,dy)
C     ====================================

C--   Polynomial tru n points using Neville's algorithm
C--   From Numerical Recipes, chapter 3.1
C--
C--   Input:  xa    table of n abscissa
C--           ya    table of n function values
C--           n     number of (x,y) points = order of polynomial
C--           x     interpolation point
C--   Output: y     interpolated polynomial P(x)
C--           dy    estimated error on y 

      implicit double precision(a-h,o-z)

      dimension xa(*),ya(*),c(10),d(10)

      if(n.gt.10) stop 'sqcPolint: degree n too large --> STOP'
      
      if(n.eq.2) then
        t  = (x-xa(1))/(xa(2)-xa(1))
        y  = (1-t)*ya(1) + t*ya(2)
        dy = 0.D0
        return
      endif  

      ns  = 1
      dif = abs(x-xa(1))
      do i = 1,n
        dift = abs(x-xa(i))
        if(dift.lt.dif) then
          ns  = i
          dif = dift
        endif
        c(i) = ya(i)
        d(i) = ya(i)
      enddo
      y = ya(ns)
      ns = ns-1
      do m = 1,n-1
        do i = 1,n-m
          ho   = xa(i)-x
          hp   = xa(i+m)-x
          w    = c(i+1)-d(i)
          den  = ho-hp
          if(den.eq.0) stop 'sqcPolint: equal abscissa --> STOP'
          den  = w/den
          d(i) = hp*den
          c(i) = ho*den
        enddo
        if(2*ns.lt.n-m) then
          dy = c(ns+1)
        else
          dy = d(ns)
          ns = ns-1
        endif
        y = y+dy
      enddo

      return
      end

C     ==========================================
      subroutine sqcPolin2(xa,nx,ya,ny,za,x,y,z)
C     ==========================================

C--   Two-dim polynomial interpolation using sqcPolint
C--
C--   xa, nx  (in)    table of nx x-abscissa
C--   ya, ny  (in)    table of ny y-abscissa  
C--   za      (in)    2-dim array za(nx,ny) with function values
C--   x,y     (in)    interpolation point
C--   z       (out)   interpolated polynomial P(x,y)

      implicit double precision(a-h,o-z)

      dimension xa(*),ya(*),za(nx,ny),work(10)

C--   Do ny interpolations in x
      do i = 1,ny
        call sqcPolint(xa,za(1,i),nx,x,work(i),epsi)
      enddo
C--   Do one interpolation in y
      call sqcPolint(ya,work,ny,y,z,epsi)

      return
      end
      
C     ==============================================      
      double precision function dqcPolint(x,xi,yi,n)
C     ==============================================

C--   Neville's algorithm for lin or quad interpolation
C--
C--   x     (in)   interpolation point
C--   xi(n) (in)   list of abscissa in ascending order
C--   yi(n) (in)   list of ordinates
C--   n     (in)   order of interpolation 2=lin, 3=quad

      implicit double precision (a-h,o-z)
      
      dimension xi(*), yi(*)
      
      flin(x,x1,x2,y1,y2) = ((x-x2)*y1+(x1-x)*y2)/(x1-x2)
      
      dqcPolint = 0.D0  !avoid compiler warning
      
      if(n.eq.2) then
        dqcPolint = flin(x,xi(1),xi(2),yi(1),yi(2))
      elseif(n.eq.3) then
        y12       = flin(x,xi(1),xi(2),yi(1),yi(2))
        y23       = flin(x,xi(2),xi(3),yi(2),yi(3))
        dqcPolint = flin(x,xi(1),xi(3),y12,y23)
      else
        stop 'dqcPolint: invalid order n --> STOP'
      endif
      
      return
      end
      
C     =======================================================
      double precision function dqcPolin2(xa,nx,ya,ny,za,x,y)
C     =======================================================

C--   Two-dim polynomial interpolation using dqcPolint
C--
C--   xa, nx  (in)    table of nx x-abscissa
C--   ya, ny  (in)    table of ny y-abscissa  
C--   za      (in)    2-dim array za(nx,ny) with function values
C--   x,y     (in)    interpolation point

      implicit double precision(a-h,o-z)

      dimension xa(*),ya(*),za(nx,ny),work(10)

C--   Do ny interpolations in x
      do i = 1,ny
        work(i) = dqcPolint(x,xa,za(1,i),nx)
      enddo
C--   Do one interpolation in y
      dqcPolin2 =  dqcPolint(y,ya,work,ny)

      return
      end
            
C     =======================================
      subroutine sqcGetABC(u,v,w,delta,a,b,c)
C     =======================================

C--   Caculate coefficients for quadratic interpolation in a bin of width
C--   delta. Let the interpolation formula be written as
C--
C--                    f(x) = A(x-x0)^2 + B(x-x0) + C
C--
C--   with x0 the lower edge of the bin and x0+delta the upper edge. Then
C--   this routine calculates the coefficients A, B and C, given delta and
C--   three sample points U = f(x0), V = f(x0+delta/2) and W = f(x0+delta).

      implicit double precision (a-h,o-z)

      a = ( 2*u - 4*v + 2*w)/(delta*delta)
      b = (-3*u + 4*v -   w)/ delta
      c =     u

      return
      end

