
C--  This is sqcPdfFill.f  with filling routines other than EVOLFG

C--   subroutine sqcPdfCpy(kset1,kset2)
C--   subroutine sqcPdfExt(subr,ig0,n,del)

C     =================================
      subroutine sqcPdfCpy(kset1,kset2)
C     =================================

C--   Copy jset1 [1-3] to jset2 [5-mset0]

C--   kset1 (in)    : source set id in stor7
C--   kset2 (in)    : target set id in stor7

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      dimension itypes(mtyp0)
      data itypes /mtyp0*0/

C--   Copy parameter list
      call sqcCpyPar(stor7,kset1,stor7,kset2)
C--   Copy type 5 + satellite tables, set iset pointer and validate
      idg1 = 1000*kset1+500
      idg2 = 1000*kset2+500
      ntab = iqcGetNumberOfTables(stor7,kset1,5)
      do i = 1,ntab
        call sqcCopyType5(stor7,idg1+i,stor7,idg2+i)
        call sqcValidate(stor7,idg2+i)
      enddo
      ntb2 = iqcGetNumberOfTables(stor7,kset2,5)
      do i = ntab+1,ntb2
        call sqcInvalidate(stor7,idg2+i)
      enddo
C--   Copy type 6
      idg1 = 1000*kset1+600
      idg2 = 1000*kset2+600
      ntab = iqcGetNumberOfTables(stor7,kset1,6)
      do i = 1,ntab
        call sqcCopyType6(stor7,idg1+i,stor7,idg2+i)
      enddo

      return
      end

C     ====================================
      subroutine sqcPdfExt(subr,ig0,n,del)
C     ====================================

C--   Fill set of pdf tables.

C--   subr    (external): user supplied subroutine
C--   ig0     (in)      : ig0 global id of gluon table in stor7
C--   n       (in)      : number of extra tables beyond gluon, q, qbar
C--   del     (in)      : threshold offset mu2h*(1+-del)
C--
C--   subr(x,mu2,qqbar) should return qqbar(-6:6+n)

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'
      include 'qmaps8.inc'

      dimension itypes(mtyp0)
      data itypes /mtyp0*0/
      
      dimension qqbar(-6:6+n),qpm(12),epm(0:12)
      
      external subr

C--   Global identifiers
      ig1  = ig0+1
      ig2  = ig0+2
      ig12 = ig0+12
C--   Address increment
      inc = iqcG5ijk(stor7,1,1,ig2)-iqcG5ijk(stor7,1,1,ig1)
C--   Loop over iz and it      
      do iz = izmic2,izmac2
        it = itfiz2(iz)
        qi = exp(tgrid2(it))
C--     Detect threshold crossing: isign =  ... 0, 0, 0, -1, +1, 0, 0 ... 
        if(iz.ne.1 .and. iz.ne.nzz2) then
          isign = 2*nffiz2(iz)-nffiz2(iz-1)-nffiz2(iz+1)
        else
          isign = 0
        endif  
        qi = qi * (1.D0+isign*del)        
        do iy = 1,iymac2
          xi = exp(-ygrid2(iy))
C--       Get qbar, gluon, q, extra
          call subr(xi,qi,qqbar)
C--       Convert to |q+->
          do i = 1,6
            qpm(i)   = qqbar(i) + qqbar(-i)
            qpm(6+i) = qqbar(i) - qqbar(-i)
          enddo
C--       Gluon
          epm(0) = qqbar(0)
C--       Convert to |e+->          
          do i = 1,12
            epm(i) = 0.D0
            do j = 1,12
              epm(i) = epm(i) + tmateq(i,j)*qpm(j)
            enddo
          enddo
C--       Fill tables                                               
          ia  = iqcG5ijk(stor7,iy,iz,ig0)-inc
          do i = 0,12
            ia        = ia+inc
            stor7(ia) = epm(i)
          enddo
C--       Fill extra tables
          ia  = iqcG5ijk(stor7,iy,iz,ig12)
          do i = 1,n
            ia        = ia+inc
            stor7(ia) = qqbar(6+i)
          enddo

C--     End of loop over y
        enddo  
C--   End of loop over t
      enddo

      return
      end
