
C--   This is the file srcPdfInOut.f containing pdfs in/out routines

C--   subroutine sqcPdfMat
C--   subroutine sqcPdIdef(tmatpq,ierr)
C--
C--   subroutine sqcAllInp(idg,func)
C--   subroutine sqcEfrmP(pval,eval)
C--   double precision function dqcEifrmP(i,pval)
C--
C--   double precision function dqcXSplne(idg,y,it)
C--   double precision function dqcEpmyt(idg,yy,tt)
C--   double precision function dqcEpmij(idg,iy,it)
C--   subroutine sqcQpQmyt(idg,id,y,t,qplus,qminu)
C--   subroutine sqcQpQmij(idg,id,iy,it,qplus,qminu)
C--   double precision function dqcSumQQByt(idg,def,y,t)
C--   double precision function dqcSumQQBij(idg,def,iy,it)
C--   double precision function dqcSumQQGyt(idg,def,y,t)
C--   subroutine sqcAllQpQmyt(idg,y,t,val)
C--   subroutine sqcAllQpQmij(idg,iy,it,val)
C--   double precision function dqcSplChk(idg,it)

C--   subroutine sqcEfromQQ(qvec,evec,nf)
C--   subroutine sqcEweedQQ(qvec,wt,id,n,nf)
C--   subroutine sqcGetDinv(def,dinv,nf,ierr)

C--   subroutine sqcPdfTab(idgl,def,itmp,xx,nxx,qq,nqq,fff)
C--   subroutine sqcPdfLin(idgl,def,iymi,iyma,izmi,izma,igout)

C===================================================================
C==   Pdf linear combinations ======================================
C===================================================================

C     ====================
      subroutine sqcPdfMat
C     ====================

C--   Setup the matrices which relate the flavour basis |q> to the
C--   to the evolution basis |e>
C-- 
C--   |ei> = tmateq(i,j) |qj>
C--   |qj> = tmatqe(i,j) |ei>    NB: |q> = |q +- qbar>
C--
C--            1  2  3  4  5  6  7  8  9 10 11 12
C--   |ei>     s  2+ 3+ 4+ 5+ 6+ v  2- 3- 4- 5- 6-
C--   |qi>     d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--
C--   Called by sqc_qcinit

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      dimension imateq(6,6), work(6,6)

      data imateq /
C--      d  u  s  c  b  t
C--      1  2  3  4  5  6 
     +   1, 1, 1, 1, 1, 1, !si   =  1
     +  -1, 1, 0, 0, 0, 0, !ns1  =  2
     +   1, 1,-2, 0, 0, 0, !ns2  =  3
     +   1, 1, 1,-3, 0, 0, !ns3  =  4
     +   1, 1, 1, 1,-4, 0, !ns4  =  5
     +   1, 1, 1, 1, 1,-5/ !ns5  =  6

C--   Transform imateq to math notation (swap indices) -> umateq 
      do i = 1,6
        do j = 1,6
          umateq(i,j) = imateq(j,i)
        enddo
      enddo
C--   Loop over 3,..,6 flavors
      do nf = 3,6
C--     Invert the nf*nf submatrix of umateq
        call sqcOrtInv(umateq,work,6,nf)
C--     Store the result in common block 
        do i = 1,6
          do j = 1,6
            umatqe(i,j,nf) = work(i,j)
          enddo
        enddo 
      enddo
C--   Umatqe depends on nf but for tmatqe we can take nf = 6 because
C--   all 12 pdfs are always correctly filled by evolff even when nf < 6.
C--   Fill matrix tmateq and tmatqe
      do i = 1,6
        do j = 1,6
          tmateq(i,j)     = umateq(i,j)
          tmateq(i,j+6)   = 0.D0
          tmateq(i+6,j)   = 0.D0
          tmateq(i+6,j+6) = umateq(i,j)
          tmatqe(i,j)     = umatqe(i,j,6)
          tmatqe(i,j+6)   = 0.D0
          tmatqe(i+6,j)   = 0.D0
          tmatqe(i+6,j+6) = umatqe(i,j,6)
        enddo
      enddo

      return
      end

C     =================================
      subroutine sqcPdIdef(tmatpq,ierr)
C     =================================
  
C--   The set of input distns defined (by the user) on the q+-
C--   basis is stored in the matrix tmatpq(i,j), where i = 1,...,12 is
C--   the user's input pdf index and j = 1,...12 is the q+- index: 
C--            1  2  3  4  5  6  7  8  9 10 11 12
C--           d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--
C--   Here are the various transformations performed in QCDNUM:
C--
C--   |pi> = tmatpq(i,j) |qj>  pdfs written on the q+- basis (input)
C--   |qi> = tmatqp(i,j) |pj>  inverse of the above
C--   |qi> = tmatqe(i,j) |ej>  q->e transformation matrix (from sqcPdfMat)
C--   |ei> = tmateq(i,j) |qj>  inverse of above (from sqcPdfMat)
C--   |pi> = tmatpe(i,j) |ej>  pdfs written on the si/ns basis
C--   |ei> = tmatep(i,j) |pj>  si/ns basis as lin comb of input pdfs 
C--
C--   Given the matrices tmatpq, tmatqe and tmateq the routine calculates
C--   the matrices tmatpe and tmatep

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      dimension tmatpq(12,12),tmatqp(12,12)
      dimension iwork(12)

C--   Invert tmatpq
      do i = 1,12
        do j = 1,12
          tmatqp(i,j)   = tmatpq(i,j)
        enddo
      enddo
      call smb_dminv(12,tmatqp,12,iwork,ierr)
C--   Oh, lala .... that doesnt look good      
      if(ierr.ne.0) return

C--   tmatpe = tmatpq*tmatqe
      do i = 1,12
        do j = 1,12
          sum = 0.D0
          do k = 1,12
            sum = sum + tmatpq(i,k)*tmatqe(k,j)
          enddo
          tmatpe(i,j) = sum
        enddo
      enddo
C--   tmatep = tmateq*tmatqp
      do i = 1,12
        do j = 1,12
          sum = 0.D0
          do k = 1,12
            sum = sum + tmateq(i,k)*tmatqp(k,j)
          enddo
          tmatep(i,j) = sum
        enddo
      enddo

      return
      end

C===================================================================
C==   Pdf input routines ===========================================
C===================================================================

C     ==============================
      subroutine sqcAllInp(idg,func)
C     ==============================

C--   Calculate 2nf+1 pdfs provided by the user in func(j,x) and 
C--   store these in      |p> (j=0,...,12)  
C--   Then transform to   |e> (i=0,...,12)
C--   Put the transformed pdfs in stor7(iy,it=0,i), i = 0,...,12
C--   NB: pdfs stored in bin it = 0 serve as evolution start values
C--
C--   idg   (in)  gluon table identifier in global format

      implicit double precision (a-h,o-z)

      external func

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      dimension pval(0:12), eval(0:12)

C--   Initialize
      nfmax = max(nfix6,3)
      do i = 0,12
        pval(i) = 0.D0
        eval(i) = 0.D0
      enddo

C--   Loop over y gridpoints
      do iy = 1,nyy2(0)
        y  = ygrid2(iy)
        x  = exp(-y)
        do j = 0,2*nfmax
          pval(j) = func(j,x)
        enddo
C--     Transform
        call sqcEfrmP(pval,eval)
C--     Store
        do id = 0,12
          iadr = iqcG5ijk(stor7,iy,0,idg+id)
          stor7(iadr) = eval(id)
        enddo
      enddo

      return
      end

C     ==============================
      subroutine sqcEfrmP(pval,eval)
C     ==============================

C--   Transform 13 parton values |p> (j=0,...,12)
C--   to the evolution basis     |e> (i=0,...,12)
C--
C--   Input:  pval(0:12)  input values  |p>
C--   Output: eval(0:12)  output values |e>

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      dimension pval(0:12), eval(0:12)

      do i = 0,12
        eval(i) = dqcEifrmP(i,pval)
      enddo

      return
      end

C     ===========================================
      double precision function dqcEifrmP(i,pval)
C     ===========================================

C--   Transform 13 parton values            |pj> (j=0,...,12)
C--   to one element of the evolution basis |ei> (i=0,...,12)
C--
C--   Input:  i          = index of |ei> basis element (i = 0 = gluon)
C--           pval(0:12) = input vector |p>  (j = 0 = gluon)
C--
C--   Output: dqcEifrmP  = value of evolution basis element |ei>

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      dimension pval(0:12)

      if(i.eq.0) then
C--     Gluon
        dqcEifrmP = pval(0)
      else
C--     |ei>  = tmatep(i,j) |pj>
        sum = 0.D0
        do j = 1,12
          sum = sum + tmatep(i,j)*pval(j)
        enddo
        dqcEifrmP = sum
      endif

      return
      end

C===================================================================
C==   Pdf output routines ==========================================
C===================================================================

C     =============================================
      double precision function dqcXSplne(idg,y,it)
C     =============================================

C--   Spline interpolation in x

C--   idg    (in)  : Global pdf id in stor7
C--   y      (in)  : value of y
C--   it     (in)  : t grid point

      implicit double precision (a-h,o-z)
      
      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      
      logical lqcAxeqy
      
      dimension acoef(mxx0)

C--   Catch y = 0  (x = 1)
      if(lqcAxeqy(y,0.D0,aepsi6)) then
        dqcXSplne = 0.D0
        return
      endif  
C--   Spline storage index
      idk = ioy2-1      
C--   Find y grid index in main grid G0
      iy = iqcFindIy(y)
C--   z grid index
      iz = izfit2(it) 
C--   Convert to A coefficients
      call sqcGetSplA(stor7,idg,iy,iz,ig,iyg,acoef)
C--   We did hit the end-point: iyg --> iyg-1
      iyg = min(iyg,nyy2(ig)-1)
C--   Spline index range in y
      call sqcByjLim(idk,iyg+1,minby,maxby)  !iy --> iy+1
C--   Loop over spline y
      val = 0.D0
      do jy = minby,maxby
        yjm1  = (jy-1)*dely2(ig)
        spy = dqcBsplyy(idk,1,(y-yjm1)/dely2(ig))
        val = val + acoef(jy)*spy
      enddo

      dqcXSplne = val

      return
      end
      
C     =============================================
      double precision function dqcEpmyt(idg,yy,tt)
C     =============================================

C--   Value of an internal pdf at y and t
C--
C--   id  =  0   1   2   3   4   5   6   7   8   9  10  11  12  13 ...
C--          g  e1+ e2+ e3+ e4+ e5+ e6+ e1- e2- e3- e4- e5- e6- extra
C--
C--   idg   (in) : stor7 pdf identifier in global format
C--   yy,tt (in) : interpolation point
      
      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'
      include 'qpars6.inc'
      
      logical lqcAxeqy

C--   Catch y = 0  (x = 1)      
      if(lqcAxeqy(yy,0.D0,aepsi6)) then
        dqcEpmyt = 0.D0
      else
        call sqcZmesh(yy,tt,0,iy1,iy2,iz1,iz2)
        call sqcIntPol(stor7,idg,iy1,iy2,iz1,iz2,yy,tt,dqcEpmyt)
      endif

      return
      end
      
C     =============================================
      double precision function dqcEpmij(idg,iy,it)
C     =============================================

C--   Value of an internal pdf at iy and it
C--
C--   id  =  0   1   2   3   4   5   6   7   8   9  10  11  12  13 ...
C--          g  e1+ e2+ e3+ e4+ e5+ e6+ e1- e2- e3- e4- e5- e6- extra
C--
C--   idg  (in)  : stor7 pdf identifier in global format
C--   iy,it (in) : grid point
C--
C--   Remark: it > 0 then follow the 4,5,6 convention at iqc,b,t
C--              < 0 then follow the 3,4,5 convention at iqc,b,t
      
      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      if(it.gt.0) then
        iz = izfitU2(it)
      elseif(it.lt.0) then
        iz = izfitL2(-it)
      else
        stop 'dqcEpmij: it = 0 encountered'
      endif
      ia       = iqcG5ijk(stor7,iy,iz,idg)
      dqcEpmij = stor7(ia)

      return
      end      

C     ============================================
      subroutine sqcQpQmyt(idg,id,y,t,qplus,qminu)
C     ============================================

C--   Return gluon or qplus=q+qbar and qminus=q-qbar
C--   id     =  0  1  2  3  4  5  6
C--   val    =  g  d  u  s  c  b  t
C--
C--   idg    (in) : gluon pdf id in stor7 (global format)
C--   id     (in) : local pdf identifier [0,6]
C--   y,t    (in) : interpolation point
C--   qplus (out) : g, q + qbar
C--   qminu (out) : g, q - qbar

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'
      include 'qpars6.inc'
      
      call sqcZmesh(y,t,0,iy1,iy2,iz1,iz2)
      
C--   Output gluon
      if(id.eq.0) then
        call sqcIntPol(stor7,idg,iy1,iy2,iz1,iz2,y,t,qplus)
        qminu = qplus
      else
C--     Figure out number of flavors
        it = iqcItfrmt(t)
        if(it.eq.0)  stop 'sqcQpmyt: t out of range ---> STOP'
        iz  = izfit2(it)
        nf  = nffiz2(iz)
C--     Output |qi> = umatqe(i,j,nf) |ej>
C--     i  =  0  1  2  3  4  5  6  7  8  9 10 11 12 
C--     qi =  g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--     q+qbar and q-qbar
        qplus = 0.D0
        qminu = 0.D0
        do j = 1,nf
          jdg = idg+j
          call sqcIntPol(stor7,jdg  ,iy1,iy2,iz1,iz2,y,t,eplus)
          call sqcIntPol(stor7,jdg+6,iy1,iy2,iz1,iz2,y,t,eminu)
          qplus = qplus + umatqe(id,j,nf)*eplus
          qminu = qminu + umatqe(id,j,nf)*eminu
        enddo
      endif

      return
      end
      
C     ==============================================
      subroutine sqcQpQmij(idg,id,iy,it,qplus,qminu)
C     ==============================================

C--   Return gluon or qplus and qminus
C--   id     =  0  1  2  3  4  5  6
C--   val    =  g  d  u  s  c  b  t
C--
C--   idg    (in) : gluon pdf id in stor7 (global format)
C--   id     (in) : local pdf identifier [0,6]
C--   iy,it  (in) : grid point
C--   qplus (out) : g, q + qbar
C--   qminu (out) : g, q - qbar

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      if(it.gt.0) then
        iz = izfitU2(it)
      elseif(it.lt.0) then
        iz = izfitL2(-it)
      else
        stop 'sqcQpQmij: it = 0 encountered'
      endif

C--   Output gluon
      if(id.eq.0) then
        ia    = iqcG5ijk(stor7,iy,iz,idg)
        qplus = stor7(ia)
        qminu = qplus
      else
C--     Figure out number of flavors
        nf  = nffiz2(iz)
C--     Output |qi> = umatqe(i,j,nf) |ej>
C--     i  =  0  1  2  3  4  5  6  7  8  9 10 11 12 
C--     qi =  g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--     q+qbar and q-qbar
        qplus = 0.D0
        qminu = 0.D0
        do j = 1,nf
          ia    = iqcG5ijk(stor7,iy,iz,idg+j)
          eplus = stor7(ia)
          ia    = iqcG5ijk(stor7,iy,iz,idg+j+6)
          eminu = stor7(ia)
          qplus = qplus + umatqe(id,j,nf)*eplus
          qminu = qminu + umatqe(id,j,nf)*eminu
        enddo
      endif

      return
      end

C     ==================================================
      double precision function dqcSumQQByt(idg,def,y,t)
C     ==================================================

C--   Return linear combination of q and qbar
C--   id     =  -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6
C--   def    =  tb bb cb sb ub db  g  d  u  s  c  b  t
C--
C--   idg       (in) : gluon id in stor7 (global format)
C--   def(-6:6) (in) : coefficients of the linear combination
C--   y,t       (in) : interpolation point

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'
      
      dimension def(-6:6)

      sum = 0.D0
      do id = 1,6
        if(def(id).ne.0.D0 .or. def(-id).ne.0.D0) then
          call sqcQpQmyt(idg,id,y,t,qplus,qminu)
          wpl = 0.5D0 * (def(id)+def(-id))
          wmi = 0.5D0 * (def(id)-def(-id))
          sum = sum + wpl*qplus + wmi*qminu
        endif
      enddo

      dqcSumQQByt = sum

      return
      end

C     ====================================================
      double precision function dqcSumQQBij(idg,def,iy,it)
C     ====================================================

C--   Return linear combination of q and qbar
C--   id     =  -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6
C--   def    =  tb bb cb sb ub db  g  d  u  s  c  b  t
C--
C--   idg       (in) : gluon id in stor7 (global format)
C--   def(-6:6) (in) : coefficients of the linear combination
C--   iy,it     (in) : grid point

      implicit double precision (a-h,o-z)

      dimension def(-6:6)

      sum = 0.D0
      do id = 1,6
        if(def(id).ne.0.D0 .or. def(-id).ne.0.D0) then
          call sqcQpQmij(idg,id,iy,it,qplus,qminu)
          wpl = 0.5D0 * (def(id)+def(-id))
          wmi = 0.5D0 * (def(id)-def(-id))
          sum = sum + wpl*qplus + wmi*qminu
        endif
      enddo

      dqcSumQQBij = sum

      return
      end
      
C     ==================================================
      double precision function dqcSumQQGyt(idg,def,y,t)
C     ==================================================

C--   Return linear combination of q, qbar and gluon
C--   id     =  -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6
C--   def    =  tb bb cb sb ub db  g  d  u  s  c  b  t
C--
C--   idg       (in) : gluon id in stor7 (global format)
C--   def(-6:6) (in) : coefficients of the linear combination
C--   y,t       (in) : interpolation point

      implicit double precision (a-h,o-z)
      
      include 'qcdnum.inc'
      include 'qpars6.inc'

      dimension def(-6:6)

      sum = 0.D0
      do id = 0,6
        if(def(id).ne.0.D0 .or. def(-id).ne.0.D0) then
          call sqcQpQmyt(idg,id,y,t,qplus,qminu)
          wpl = 0.5D0 * (def(id)+def(-id))
          wmi = 0.5D0 * (def(id)-def(-id))
          sum = sum + wpl*qplus + wmi*qminu
        endif
      enddo

      dqcSumQQGyt = sum

      return
      end

C     ====================================
      subroutine sqcAllQpQmyt(idg,y,t,val)
C     ====================================

C--   i      =  0  1  2  3  4  5  6  7  8  9 10 11 12 
C--   val(i) =  g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--
C--   idg       (in)  : gluon id in stor7 (global format)
C--   y,t       (in)  : interpolation point
C--   val(0:12) (out) : all q+- values

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'
      include 'qpars6.inc'

      dimension val(0:12)
      
      call sqcZmesh(y,t,0,iy1,iy2,iz1,iz2)
      
C--   Output gluon
      call sqcIntPol(stor7,idg,iy1,iy2,iz1,iz2,y,t,val(0))
C--   Output |qi> = tmatqe(i,j) |ej>
      do i = 1,12
        val(i) = 0.D0
      enddo
      do j = 1,12
        jdg = idg+j
        call sqcIntPol(stor7,jdg,iy1,iy2,iz1,iz2,y,t,funj)
        do i = 1,12
          val(i) = val(i) + tmatqe(i,j)*funj
        enddo
      enddo

      return
      end
      
C     ======================================
      subroutine sqcAllQpQmij(idg,iy,it,val)
C     ======================================

C--   i      =  0  1  2  3  4  5  6  7  8  9 10 11 12 
C--   val(i) =  g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--
C--   idg       (in)  : gluon id in stor7 (global format)
C--   iy,it     (in)  : grid point
C--   val(0:12) (out) : all q+- values

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      dimension val(0:12)

      if(it.gt.0) then
        iz = izfitU2(it)
      elseif(it.lt.0) then
        iz = izfitL2(-it)
      else
        stop 'sqcAllQpQmij: it = 0 encountered'
      endif

C--   Output gluon
      ia = iqcG5ijk(stor7,iy,iz,idg)
      val(0) = stor7(ia)
C--   Output |qi> = tmatqe(i,j) |ej>
      do i = 1,12
        val(i) = 0.D0
      enddo
      do j = 1,12
        ia   = iqcG5ijk(stor7,iy,iz,idg+j)
        funj = stor7(ia)
        do i = 1,12
          val(i) = val(i) + tmatqe(i,j)*funj
        enddo
      enddo

      return
      end
      
C     ===========================================
      double precision function dqcSplChk(idg,it)
C     ===========================================

C--   Returns epsi = || quad-lin || interpolation at midpoints
C--
C--   idg   (in) : stor7 pdf identifier in global format
C--   it    (in) : mu2 grid point

      implicit double precision (a-h,o-z)
      
      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'
      
      dimension acoef(mxx0), epsi(mxx0)

C--   Initialize
      dqcSplChk = 0.D0
C--   Linear interpolation
      if(ioy2.ne.3) return
C--   Find z-bin 
      if(it.gt.0) then
        iz = izfitU2(it)
      elseif(it.lt.0) then
        iz = izfitL2(-it)
      else
        stop 'dqcSplChk: it = 0 encountered'
      endif

C--   Loop over subgrids
      do jg = 1,nyg2
        iy = iyma2(jg)
        call sqcGetSplA(stor7,idg,iy,iz,ig,ny,acoef)
C--     Debug checks
        if(ig.ne.jg)        stop 'dqcSplChk: ig not jg'
        if(ny.ne.nyy2(jg))  stop 'dqcSplChk: ny not nyy2(jg)'
C--     Now get vector of deviations 
        nyma = iqcIyMaxG(iymac2,ig)       
        call sqcDHalf(ioy2,acoef,epsi,nyma)
C--     Max deviation
        do iy = 1,nyma
          dqcSplChk = max(dqcSplChk,abs(epsi(iy)))
        enddo  
      enddo
      
      return
      end
    
C     ===================================
      subroutine sqcEfromQQ(qvec,evec,nf)
C     ===================================

C--   Transform coefficients from flavor basis to si/ns basis
C--
C--         -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6
C--   qvec  tb bb cb sb ub db  g  d  u  s  c  b  t
C--
C--   |qpm>  0  1  2  3  4  5  6  7  8  9 10 11 12
C--          g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--
C--   |epm>  0  1  2  3  4  5  6  7  8  9 10 11 12
C--          g  si 2+ 3+ 4+ 5+ 6+ v  2- 3- 4- 5- 6-
C--
C--   qvec(-6:6)   (in)  Coefficients in flavour space
C--                        qvec(0) is ignored --> quarks only!
C--   evec(12)     (out) Coefficients in si/ns space
C--   nf           (in)  Number of active flavours [3-6]

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpdfs7.inc'
      
      dimension qvec(-6:6),qpm(12),evec(12)

C--   Get qpm coefficient from q,qbar coefficients (eq 2.20)
      do i = 1,nf
        qpm(i  ) = 0.5*(qvec(i) + qvec(-i))
        qpm(6+i) = 0.5*(qvec(i) - qvec(-i))
      enddo
C--   Transform (eq 2.29)
      do i = 1,12
        evec(i) = 0.D0
      enddo
      do i = 1,nf
        diplu = 0.D0
        dimin = 0.D0
        do j = 1,nf
          diplu = diplu + qpm(j  )*umatqe(j,i,nf)
          dimin = dimin + qpm(6+j)*umatqe(j,i,nf)
        enddo
        evec(i  ) = diplu
        evec(i+6) = dimin
      enddo

      return
      end
      
C     ======================================
      subroutine sqcEweedQQ(qvec,wt,id,n,nf)
C     ======================================

C--   Transform coefficients from flavor basis to si/ns basis
C--   and weed out zero-value coefficients
C--
C--         -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6
C--   qvec  tb bb cb sb ub db  g  d  u  s  c  b  t
C--
C--   qvec(-6:6)   (in)  Coefficients in flavour space
C--                        qvec(0) = nonzero --> pick gluon
C--                        qvec(0) = zero    --> pick quarks
C--   wt(i)        (out) Weight of i-th si/ns pdf
C--   id(i)        (out) Identifier of i-th si/ns pdf
C--   n            (out) Number of terms in weighted sum
C--   nf           (in)  Number of active flavours [3-6]

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'
      
      logical lqcAcomp
      
      dimension qvec(-6:6),wt(12),id(12)
      
      if(.not.lqcAcomp(qvec(0),0.D0,aepsi6)) then
C--     Gluon      
        wt(1) = qvec(0)
        id(1) = 0
        n     = 1
      else
C--     Quarks 
        call sqcEfromQQ(qvec,wt,nf)
        n = 0
        do i = 1,12 
          if(.not.lqcAcomp(wt(i),0.D0,aepsi6)) then
            n     = n + 1
            wt(n) = wt(i)
            id(n) = i
          endif              
        enddo
      endif          

      return
      end

C     ============================================
      subroutine sqcGetDinv( def, dinv, nf, ierr )
C     ============================================

C--   The basis pdfs |e> are given by a linear combination of the 2nf
C--   user input pdfs |f> :
C--
C--               |ei> = Sum_j dinv(i,j) |fj>  j = 1,...2nf
C--
C--   This routine calculates the coefficients dinv(i,j)
C--
C--   def(-6:6,12)   (in) : def(i,j)   contribution of flavour i to |fj>
C--   dinv(12,12)   (out) : dinv(i,j)  contribution of |fj> to |ei>
C--   nf             (in) : number of active flavours [3-6]

      implicit double precision (a-h,o-z)
      dimension def(-6:6,12), dinv(12,12)
      dimension d(12), work(12,12), iw(12)

      do i = 1,2*nf
C--     Get d coefficients
        call sqcEfromQQ( def(-6,i), d, nf )
C--     Copy d to work array
        do j = 1,nf
            work(i,j   ) = d(j  )
            work(i,j+nf) = d(j+6)
        enddo
      enddo
C--   Invert work array
      call smb_dminv( 2*nf, work, 12, iw, ierr )
      if( ierr.ne.0 ) return   !singular matrix
C--   Initialize dinv
      do i = 1,12
        do j = 1,12
          dinv(i,j) = 0.D0
        enddo
      enddo
C--   Copy work array to dinv
      do i = 1,nf
        do j = 1,2*nf
          dinv(i  ,j) = work(i   ,j)
          dinv(i+6,j) = work(i+nf,j)
        enddo
      enddo

      return
      end

C     =====================================================
      subroutine sqcPdfTab(idgl,def,itmp,xx,nxx,qq,nqq,fff)
C     =====================================================

C--   Return pdfs interpolated on a x-mu2 grid 
C--
C--   idgl         (in)   stor7 gluon id in global format
C--   def(-6:6)    (in)   array of pdf coefficients
C--                       def(0) = 0.D0   select quarks
C--                       def(0) # 0.D0   select gluons
C--   itmp         (in)   stor7 temporary pdf id in global format
C--   xx(i)        (in)   table of x-values
C--   nxx          (in)   number of x-values
C--   qq(i)        (in)   table of mu2 values
C--   nqq          (in)   number of mu2 values
C--   fff(nxx*nqq) (out)  interpolated pdfs (linear store)
C--
C--   Remark: xx(i) and qq(i) are all supposed to be within the grid
C--   Remark: result fff(nx*nq) is stored linearly 

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      dimension def(-6:6)
      dimension xx(*), qq(*), fff(*)
      
C--   Find subgrid range in x and q2      
      ymi    = -log(xx(nxx))
      yma    = -log(xx(1))
      tmi    =  log(qq(1))
      tma    =  log(qq(nqq))
      margin = 0
      call sqcZmesh(ymi,tmi,margin,iymi,iydum,izmi,izdum)
      call sqcZmesh(yma,tma,margin,iydum,iyma,izdum,izma)
C--   Calculate lincomb of pdfs inside subgrid and store in itmp
      call sqcPdfLin(idgl,def,iymi,iyma,izmi,izma,itmp)
C--   Loop over x and q and interpolate itmp, store result in fff
      ia = 0
      do iq = 1,nqq
        t = log(qq(iq))
        do ix = 1,nxx
          ia = ia + 1
          y  = -log(xx(ix))
          call sqcZmesh(y,t,margin,iy1,iy2,iz1,iz2)
          call sqcIntPol(stor7,itmp,iy1,iy2,iz1,iz2,y,t,fff(ia))
        enddo         
      enddo
      
      return
      
      end

C     ========================================================      
      subroutine sqcPdfLin(idgl,def,iymi,iyma,izmi,izma,igout)
C     ======================================================== 

C--   Store linear combination of pdfs in igout
C--
C--   idgl         (in)   stor7 gluon id in global format
C--   def(-6:6)    (in)   array of pdf coefficients
C--                       def(0) = 0.D0   select quarks
C--                       def(0) # 0.D0   select gluons 
C--   iymi,iyma    (in)   range in iy
C--   izmi,izma    (in)   range in iz
C--   igout        (in)   stor7 output id in global format

      implicit double precision (a-h,o-z)
      
      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'
      
      dimension def(-6:6), wt(12,3:6), id(12,3:6), nn(3:6)
      
C--   Get coefficients in si/ns space      
      do nf = 3,6
        call sqcEweedQQ(def,wt(1,nf),id(1,nf),nn(nf),nf)
      enddo 
C--   Initialize output table
      call sqcPreset(igout,0.D0)
C--   Sum linear combination
      do iz = izmi,izma 
C--     Output base address      
        jo = iqcG5ijk(stor7,iymi,iz,igout)-1
C--     Figure out number of flavors
        nf  = nffiz2(iz)
C--     Loop over input identifiers        
        do i = 1,nn(nf)
C--       Output base address
          io = jo        
C--       Input base address
          idg = idgl + id(i,nf)
          ia = iqcG5ijk(stor7,iymi,iz,idg)-1
          do iy = iymi,iyma
            ia        = ia + 1
            io        = io + 1
            stor7(io) = stor7(io) + wt(i,nf)*stor7(ia)
          enddo
        enddo
      enddo
          
      return      
      end
