
C--   file usrcvol.f containing convolution engine user interface

C--   subroutine MakeWtA(w,id,afun,achi)
C--   subroutine MakeWtB(w,id,bfun,achi,ndel)
C--   subroutine MakeWRS(w,id,rfun,sfun,achi,ndel)
C--   subroutine MakeWtD(w,id,dfun,achi)
C--   subroutine MakeWtX(w,id)

C--   integer function idSpfun(string,iord,jset)
C--   integer function iPdftab(jset,id)

C--   subroutine ScaleWt(w,c,id)
C--   subroutine CopyWgt(w,jd1,id2,iadd)
C--   subroutine WcrossW(w,jda,jdb,idc,iadd)
C--   subroutine WtimesF(w,fun,jd1,id2,iadd)

C--   double precision function FcrossK(w,idw,jset,idf,ix,iq)
C--   double precision function FcrossF(w,idw,jset,ida,idb,ix,iq)

C--   subroutine PushPar
C--   subroutine PullPar
C--   subroutine EfromQQ(qvec,evec,nf)
C--   subroutine QQfromE(evec,qvec,nf)

C--   subroutine StfunXq(stfun,x,qmu2,stf,n,jchk)

C     ==================================
      subroutine MakeWtA(w,jd,afun,achi)
C     ==================================

C--   Make weight table for regular contribution A
C--
C--   w     (in) :   workspace
C--   jd    (in) :   table identifier in global format
C--   afun  (in) :   function declared external in the calling routine 
C--   achi  (in) :   function declared external in the calling routine 

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'

      external afun, achi

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /    1,     1   / , iflg /   0,    0  /
      data icma  /    1,     4   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam /'MAKEWTA ( W, ID, AFUN, ACHI )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check id
      igl = iqcSjekId(subnam,'ID',w,jd,icmi,icma,iflg,lint)

C--   Do the work
      call sqcUweitA(w,igl,afun,achi,ierr)
      if(ierr.eq.1) then
        call sqcErrMsg(subnam,'Function achi(qmu2) < 1 encountered')
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C     =======================================
      subroutine MakeWtB(w,jd,bfun,achi,ndel)
C     =======================================

C--   Make weight table for singular contribution B
C--
C--   w      (in) :  workspace
C--   jd     (in) :  table identifier in global format
C--   bfun   (in) :  function declared external in the calling routine
C--   achi   (in) :  function declared external in the calling routine
C--   ndel   (in) :  1 = no delta(1-x) contribution; otherwise yes

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'

      external bfun, achi

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign   iset  itype              pzi  cfil
      data icmi  /    1,     1   / , iflg /   0,    0  /
      data icma  /    1,     4   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam  /'MAKEWTB ( W, ID, BFUN, ACHI, NODELTA )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check id
      igl = iqcSjekId(subnam,'ID',w,jd,icmi,icma,iflg,lint)

C--   Do the work (note flip ndel 0 <--> 1)
      call sqcUweitB(w,igl,bfun,achi,1-ndel,ierr)
      if(ierr.eq.1) then
        call sqcErrMsg(subnam,'Function achi(qmu2) < 1 encountered')
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C     ============================================
      subroutine MakeWRS(w,jd,rfun,sfun,achi,ndel)
C     ============================================

C--   Make weight table for contribution RS
C--
C--   w      (in) :  workspace
C--   jd     (in) :  table identifier in global format
C--   rfun   (in) :  function declared external in the calling routine
C--   sfun   (in) :  function declared external in the calling routine 
C--   achi   (in) :  function declared external in the calling routine
C--   ndel   (in) :  1 = no delta(1-x) contribution; otherwise yes

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'

      external rfun, sfun, achi

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /    1,     1   / , iflg /   0,    0  /
      data icma  /    1,     4   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam /'MAKEWRS ( W, ID, RFUN, SFUN, ACHI, NODELTA )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check id
      igl = iqcSjekId(subnam,'ID',w,jd,icmi,icma,iflg,lint)

C--   Do the work (note flip ndel 0 <--> 1)
      call sqcUwgtRS(w,igl,rfun,sfun,achi,1-ndel,ierr)
      if(ierr.eq.1) then
        call sqcErrMsg(subnam,'Function achi(qmu2) < 1 encountered')
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C     ==================================
      subroutine MakeWtD(w,jd,dfun,achi)
C     ==================================

C--   Make weight table for factor*delta(1-x) contribution
C--
C--   w      (in) :  workspace
C--   jd     (in) :  table identifier in global format
C--   dfun   (in) :  function declared external in the calling routine
C--   achi   (in) :  function declared external in the calling routine


      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'

      external dfun,achi

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /    1,     1   / , iflg /   0,    0  /
      data icma  /    1,     4   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam  /'MAKEWTD ( W, ID, DFUN, ACHI )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check id
      igl = iqcSjekId(subnam,'ID',w,jd,icmi,icma,iflg,lint)

C--   Do the work
      call sqcUweitD(w,igl,dfun,achi,ierr)
      if(ierr.eq.1) then
        call sqcErrMsg(subnam,'Function achi(qmu2) < 1 encountered')
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end
      
C     ========================
      subroutine MakeWtX(w,jd)
C     ========================

C--   Make weight table for FxF convolution
C--
C--   w    (in) :    workspace
C--   jd   (in) :    table identifier in global format

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /    1,     1   / , iflg /   0,    0  /
      data icma  /    1,     4   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam /'MAKEWTX ( W, ID )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check id
      igl = iqcSjekId(subnam,'ID',w,jd,icmi,icma,iflg,lint)

C--   Do the work
      call sqcUweitX(w,igl,ierr)
      if(ierr.eq.1) then
        call sqcErrMsg(subnam,'Error condition encountered')
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C     ==========================================
      integer function idSpfun(string,iord,jset)
C     ==========================================

C--   Returns splitting function table index encoded as -(1000*jset+idspl)
C--   Returns -99999 if splitting function table not available
C--   NB: jset 1=unpol, 2=pol, 3=timelike

      implicit double precision (a-h,o-z)      
      
      include 'qcdnum.inc'
      include 'qstor7.inc'
      
      character*(*) string
      character*3   input
      character*3   ptab(12)
C--                 1     2     3     4     5     6     7
      data ptab / 'PQQ','PQG','PGQ','PGG','PMI','PPL','PVA',
C--                 8     9    10    11    12
     +            'AGQ','AGG','AQQ','AHQ','AHG' /
      
      idspfun = -1
      if(jset.lt.1. .or. jset.gt.3) return
      if(mxord7(jset).eq.0)         return

      input        = '   '
      len          = min(imb_lenoc(string),3)
      input(1:len) = string(1:len)
      call smb_cltou(input)
      id = 0
      do i = 1,7
        if(input.eq.ptab(i))   id = idPij7(i,iord,jset)
      enddo
      if(iord.eq.3 .and. jset.eq.1) then
        do i = 8,12
          if(input.eq.ptab(i))   id = idAij7(i-7,jset)
        enddo
      endif

      if(id.eq.0) then
        idspfun = -99999
      else
        idspfun = -id
      endif

      return
      end

C     ==========================
      subroutine ScaleWt(w,c,jd)
C     ==========================

C--   Multiply weight table by a constant
C--
C--   w   (in)     workspace
C--   c   (in)     constant
C--   jd  (in)     weight table identifier in w (global format)

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /    1,     1   / , iflg /   0,    0  /
      data icma  /    1,     4   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam /'SCALEWT ( W, C, ID )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check id
      igl = iqcSjekId(subnam,'ID',w,jd,icmi,icma,iflg,lint)

C--   Do the work
      call sqcScaleWt(w,c,igl)

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C     ==================================
      subroutine CopyWgt(w,jd1,jd2,iadd)
C     ==================================

C--   Add content of jd1 to jd2
C--
C--   w    (in) : workspace
C--   jd1  (in) : input id (global format), < 0 table in internal memory
C--   jd2  (in) : output id (global format) in workspace w
C--   iadd (in) : -1,0,1 = subtract/copy/add id1 to id2

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi1(2), icma1(2), iflg1(2)
C--                isign  itype               pzi  cfil
      data icmi1  /   -1,     1   / , iflg1 /   0,    1  /
      data icma1  /    1,     6   /

      dimension icmi2(2), icma2(2), iflg2(2)
C--                isign  itype               pzi  cfil
      data icmi2  /    1,     1   / , iflg2 /   0,    0  /
      data icma2  /    1,     6   /

      logical lint1, lint2

      dimension w(*)

C--   iotyp(ityp_in,ityp_out) = 0 --> combination not allowed      
      dimension iotyp(6,6)
C--   ityp_in      1  2  3  4  5  6
      data iotyp / 1, 0, 0, 0, 0, 0,   !ityp_out 1
     +             1, 1, 0, 0, 0, 0,   !ityp_out 2
     +             1, 0, 1, 0, 0, 0,   !ityp_out 3
     +             1, 1, 1, 1, 0, 0,   !ityp_out 4
     +             0, 0, 0, 0, 1, 0,   !ityp_out 5
     +             0, 0, 0, 0, 0, 1 /  !ityp_out 6

      character*80 subnam
      data subnam /'COPYWGT ( W, ID1, ID2, IADD )'/

C--   Avoid compiler warning
      ifirst = 0
C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
      
C--   Overwrite, no thank you...
      if(jd2.eq.jd1) then
        call sqcErrMsg(subnam,'ID2 cannot be equal to ID1')
      endif      

C--   Check iadd
      call sqcIlele(subnam,'IADD',-1,iadd,1,' ')

C--   Check id1
      igl1 = iqcSjekId(subnam,'ID1',w,jd1,icmi1,icma1,iflg1,lint1)

C--   Check id2
      igl2 = iqcSjekId(subnam,'ID2',w,jd2,icmi2,icma2,iflg2,lint2)

C--   Check input/output table types
      call sqcChkTyp12(subnam,'ID1','ID2',igl1,igl2,iotyp)

C--   Do the work
      if(lint1) then
        call sqcChkIoy12(subnam,'ID1','ID2',stor7,igl1,w,igl2)
        call sqcCopyWt(stor7,igl1,w,igl2,iadd)
      else
        call sqcChkIoy12(subnam,'ID1','ID2',w,igl1,w,igl2)
        call sqcCopyWt(w,igl1,w,igl2,iadd)
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C     ======================================
      subroutine WcrossW(w,jda,jdb,jdc,iadd)
C     ======================================

C--   Make weight table for convolution C = A cross B

C--   w    (in) : workspace
C--   jda  (in) : input id (global format), < 0 table in internal memory
C--   jdb  (in) : input id (global format), < 0 table in internal memory
C--   jdc  (in) : output id (global format) in workspace w
C--   iadd (in) : -1,0,1 = subtract/copy/add id1 to id2

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmia(2), icmaa(2), iflga(2)
C--                isign  itype               pzi  cfil
      data icmia  /   -1,     1   / , iflga /   0,    1  /
      data icmaa  /    1,     4   /
      dimension icmib(2), icmab(2), iflgb(2)
C--                isign  itype               pzi  cfil
      data icmib  /   -1,     1   / , iflgb /   0,    1  /
      data icmab  /    1,     4   /
      dimension icmic(2), icmac(2), iflgc(2)
C--                isign  itype               pzi  cfil
      data icmic  /    1,     1   / , iflgc /   0,    0  /
      data icmac  /    1,     4   /

      logical linta, lintb, lintc

      dimension w(*)
      
C--   iotyp(ityp_in,ityp_out) = 0 --> combination not allowed      
      dimension iotyp(6,6)
C--   ityp_in      1  2  3  4  5  6
      data iotyp / 1, 0, 0, 0, 0, 0,   !ityp_out 1
     +             1, 1, 0, 0, 0, 0,   !ityp_out 2
     +             1, 0, 1, 0, 0, 0,   !ityp_out 3
     +             1, 1, 1, 1, 0, 0,   !ityp_out 4
     +             0, 0, 0, 0, 0, 0,   !ityp_out 5
     +             0, 0, 0, 0, 0, 0 /  !ityp_out 6

      character*80 subnam
      data subnam /'WCROSSW ( W, IDA, IDB, IDC, IADD )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Overwrite, no thank you...
      if(jdc.eq.jda .or. jdc.eq.jdb) then
        call sqcErrMsg(subnam,'IDC cannot be equal to IDA or IDB')
      endif
      
C--   Check iadd
      call sqcIlele(subnam,'IADD',-1,iadd,1,' ')
      
C--   Check ida
      igla = iqcSjekId(subnam,'IDA',w,jda,icmia,icmaa,iflga,linta)

C--   Check idb
      iglb = iqcSjekId(subnam,'IDB',w,jdb,icmib,icmab,iflgb,lintb)

C--   Check idc
      iglc = iqcSjekId(subnam,'IDC',w,jdc,icmic,icmac,iflgc,lintc)

C--   Check input/output table types
      call sqcChkTyp12(subnam,'IDA','IDC',igla,iglc,iotyp)
      call sqcChkTyp12(subnam,'IDA','IDC',iglb,iglc,iotyp)

C--   Global scratch table identifiers
      idt1 = iqcIdPdfLtoG(-1,1)
      idt2 = iqcIdPdfLtoG(-1,2)
C--   Do the work
      if(linta .and. lintb)     then
        call sqcChkIoy12(subnam,'IDA','IDC',stor7,igla,w,iglc)
        call sqcChkIoy12(subnam,'IDB','IDC',stor7,iglb,w,iglc)
        call sqcWcrossW(stor7,igla,stor7,iglb,w,iglc,idt1,idt2,iadd)
      elseif(linta .and. .not.lintb) then
        call sqcChkIoy12(subnam,'IDA','IDC',stor7,igla,w,iglc)
        call sqcChkIoy12(subnam,'IDB','IDC',w    ,iglb,w,iglc)
        call sqcWcrossW(stor7,igla,w,iglb,w,iglc,idt1,idt2,iadd)
      elseif(.not.linta .and. lintb) then
        call sqcChkIoy12(subnam,'IDA','IDC',w    ,igla,w,iglc)
        call sqcChkIoy12(subnam,'IDB','IDC',stor7,iglb,w,iglc)
        call sqcWcrossW(w,igla,stor7,iglb,w,iglc,idt1,idt2,iadd)
      else
        call sqcChkIoy12(subnam,'IDA','IDC',w    ,igla,w,iglc)
        call sqcChkIoy12(subnam,'IDB','IDC',w    ,iglb,w,iglc)
        call sqcWcrossW(w,igla,w,iglb,w,iglc,idt1,idt2,iadd)
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end
      
C     ======================================
      subroutine WtimesF(w,fun,jd1,jd2,iadd)
C     ======================================

C--   Multiply id1 by fun(iq,nf) and store result in id2
C--
C--   w    (in) : workspace
C--   fun  (in) : user defined function
C--   jd1  (in) : input id (global format), < 0 table in internal memory
C--   jd2  (in) : output id (global format) in workspace w
C--   iadd (in) : -1,0,1  subtract,store,add result to id2

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi1(2), icma1(2), iflg1(2)
C--                isign  itype               pzi  cfil
      data icmi1  /   -1,     1   / , iflg1 /   0,    1  /
      data icma1  /    1,     4   /
      dimension icmi2(2), icma2(2), iflg2(2)
C--                isign  itype               pzi  cfil
      data icmi2  /    1,     1   / , iflg2 /   0,    0  /
      data icma2  /    1,     4   /

      logical lint1, lint2

      dimension w(*)
      
C--   iotyp(ityp_in,ityp_out) = 0 --> combination not allowed
      dimension iotyp0(6,6)
C--   ityp_in      1  2  3  4  5  6    !fun depends on nothing
      data iotyp0/ 1, 0, 0, 0, 0, 0,   !ityp_out 1
     +             1, 1, 0, 0, 0, 0,   !ityp_out 2
     +             1, 0, 1, 0, 0, 0,   !ityp_out 3
     +             1, 1, 1, 1, 0, 0,   !ityp_out 4
     +             0, 0, 0, 0, 0, 0,   !ityp_out 5
     +             0, 0, 0, 0, 0, 0 /  !ityp_out 6
      dimension iotyp1(6,6)
C--   ityp_in      1  2  3  4  5  6    !fun depends on iq only
      data iotyp1/ 0, 0, 0, 0, 0, 0,   !ityp_out 1
     +             0, 1, 0, 0, 0, 0,   !ityp_out 2
     +             1, 0, 1, 0, 0, 0,   !ityp_out 3
     +             1, 1, 1, 1, 0, 0,   !ityp_out 4
     +             0, 0, 0, 0, 0, 0,   !ityp_out 5
     +             0, 0, 0, 0, 0, 0 /  !ityp_out 6
      dimension iotyp2(6,6)
C--   ityp_in      1  2  3  4  5  6    !fun depends on nf only
      data iotyp2/ 0, 0, 0, 0, 0, 0,   !ityp_out 1
     +             1, 1, 0, 0, 0, 0,   !ityp_out 2
     +             0, 0, 0, 0, 0, 0,   !ityp_out 3
     +             1, 1, 1, 1, 0, 0,   !ityp_out 4
     +             0, 0, 0, 0, 0, 0,   !ityp_out 5
     +             0, 0, 0, 0, 0, 0 /  !ityp_out 6
      dimension iotyp3(6,6)
C--   ityp_in      1  2  3  4  5  6    !fun depends on both iq and nf
      data iotyp3/ 0, 0, 0, 0, 0, 0,   !ityp_out 1
     +             0, 0, 0, 0, 0, 0,   !ityp_out 2
     +             0, 0, 0, 0, 0, 0,   !ityp_out 3
     +             1, 1, 1, 1, 0, 0,   !ityp_out 4
     +             0, 0, 0, 0, 0, 0,   !ityp_out 5
     +             0, 0, 0, 0, 0, 0 /  !ityp_out 6

      external fun

      character*80 subnam
      data subnam /'WTIMESF ( W, FUN, ID1, ID2, IADD )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check iadd
      call sqcIlele(subnam,'IADD',-1,iadd,1,' ')

C--   Check id1
      igl1 = iqcSjekId(subnam,'ID1',w,jd1,icmi1,icma1,iflg1,lint1)

C--   Check id2
      igl2 = iqcSjekId(subnam,'ID2',w,jd2,icmi2,icma2,iflg2,lint2)

C--   Check fun dependence
      idep = 0                                    !no dpendence
      if(fun(1,3).ne.fun(2,3)) idep = idep+1      !iq dependent
      if(fun(1,3).ne.fun(1,4)) idep = idep+2      !nf dependent

C--   Check input/output table types
      if(idep.eq.0) then
        call sqcChkTyp12(subnam,'ID1','ID2',igl1,igl2,iotyp0)
      elseif(idep.eq.1) then
        call sqcChkTyp12(subnam,'ID1','ID2',igl1,igl2,iotyp1)
      elseif(idep.eq.2) then
        call sqcChkTyp12(subnam,'ID1','ID2',igl1,igl2,iotyp2)
      elseif(idep.eq.3) then
        call sqcChkTyp12(subnam,'ID1','ID2',igl1,igl2,iotyp3)
      endif

C--   Do the work
      if(lint1) then
        call sqcChkIoy12(subnam,'ID1','ID2',stor7,igl1,w,igl2)
        call sqcWtimesF(fun,stor7,igl1,w,igl2,iadd)
      else
        call sqcChkIoy12(subnam,'ID1','ID2',w    ,igl1,w,igl2)
        call sqcWtimesF(fun,w,igl1,w,igl2,iadd)
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end
      
C==   ===============================================================
C==   Convolution engine ============================================
C==   ===============================================================

C     =======================================================
      double precision function FcrossK(w,idw,jdum,idf,ix,iq)
C     =======================================================

C--   Convolution F cross K
C--
C--   w     (in) :   workspace
C--   idw   (in) :   weight table identifier (global format)
C--   jdum  (in) :   dummy variable
C--   idf   (in) :   identifier of pdf F (global format)
C--   ix    (in) :   x-grid index
C--   iq    (in) :   mu2 grid index

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmiw(2), icmaw(2), iflgw(2)
C--                isign  itype               pzi  cfil
      data icmiw  /   -1,     1   / , iflgw /   0,    1  /
      data icmaw  /    1,     4   /
      dimension icmif(2), icmaf(2), iflgf(2)
C--                isign  itype               pzi  cfil
      data icmif  /   -1,     5   / , iflgf /   0,    1  /
      data icmaf  /    1,     5   /

      logical lintw, lintf

      dimension w(*)

      character*80 subnam
      data subnam /'FCROSSK ( W, IDW, ISET, IDF, IX, IQ )'/

      jset = jdum !avoid compoler warning

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
      
C--   Check idw
      iglw = iqcSjekId(subnam,'IDW',w,idw,icmiw,icmaw,iflgw,lintw)

C--   Check idf
      iglf = iqcSjekId(subnam,'IDF',w,idf,icmif,icmaf,iflgf,lintf)

C--   Check cuts on pdf
      if(lintf) then
        ifail = iqcChkLmij(subnam,stor7,iglf,ix,iq,1)
      else
        ifail = iqcChkLmij(subnam,  w  ,iglf,ix,iq,1)
      endif

C--   Do the work
      iy = nyy2(0) + 1 - ix
      if(lintw .and. lintf)     then
        FcrossK = dqcFcrossK(stor7,iglw,stor7,iglf,iy,iq)
      elseif(lintw .and. .not.lintf) then
        FcrossK = dqcFcrossK(stor7,iglw,w    ,iglf,iy,iq)
      elseif(.not.lintw .and. lintf) then
        FcrossK = dqcFcrossK(w    ,iglw,stor7,iglf,iy,iq)
      else
        FcrossK = dqcFcrossK(w    ,iglw,w    ,iglf,iy,iq)
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end    
      
C     ===========================================================
      double precision function FcrossF(w,idw,jdum,ida,idb,ix,iq)
C     ===========================================================

C--   Convolution Fa cross Fb
C--
C--   w     (in) :   store dimensioned in the calling routine
C--   idw   (in) :   weight table identifier (global format)
C--   jdum  (in) :   dummy variable
C--   ida   (in) :   identifier Fa (global format)
C--   idb   (in) :   identifier Fb (global format)
C--   ix    (in) :   x-grid index
C--   iq    (in) :   mu2 grid index

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmiw(2), icmaw(2), iflgw(2)
C--                isign  itype               pzi  cfil  iset
      data icmiw  /    1,     1   / , iflgw /   0,    1  /
      data icmaw  /    1,     4   /
      dimension icmia(2), icmaa(2), iflga(2)
C--                isign  itype               pzi  cfil  iset
      data icmia  /   -1,     5   / , iflga /   0,    1  /
      data icmaa  /    1,     5   /
      dimension icmib(2), icmab(2), iflgb(2)
C--                isign  itype               pzi  cfil  iset
      data icmib  /   -1,     5   / , iflgb /   0,    1  /
      data icmab  /    1,     5   /

      logical lintw, linta, lintb

      dimension w(*)

      character*80 subnam
      data subnam /'FCROSSF ( W, IDW, ISET, IDA, IDB, IX, IQ )'/

      jset = jdum !avoid compoler warning

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check idw
      iglw = iqcSjekId(subnam,'IDW',w,idw,icmiw,icmaw,iflgw,lintw)

C--   Check ida
      igla = iqcSjekId(subnam,'IDA',w,ida,icmia,icmaa,iflga,linta)

C--   Check idb
      iglb = iqcSjekId(subnam,'IDB',w,idb,icmib,icmab,iflgb,lintb)

C--   Check cuts on pdf a
      if(linta) then
        ifail = iqcChkLmij(subnam,stor7,igla,ix,iq,1)
      else
        ifail = iqcChkLmij(subnam,  w  ,igla,ix,iq,1)
      endif

C--   Check cuts on pdf b
      if(lintb) then
        ifail = iqcChkLmij(subnam,stor7,iglb,ix,iq,1)
      else
        ifail = iqcChkLmij(subnam,  w  ,iglb,ix,iq,1)
      endif

C--   Do the work
      iy = nyy2(0) + 1 - ix
      if    (     lintw .and.      linta .and.      lintb)     then
        FcrossF = dqcFcrossF(stor7,iglw,stor7,igla,stor7,iglb,iy,iq)
      elseif(     lintw .and.      linta .and. .not.lintb)     then
        FcrossF = dqcFcrossF(stor7,iglw,stor7,igla,w    ,iglb,iy,iq)
      elseif(     lintw .and. .not.linta .and.      lintb)     then
        FcrossF = dqcFcrossF(stor7,iglw,w    ,igla,stor7,iglb,iy,iq)
      elseif(     lintw .and. .not.linta .and. .not.lintb)     then
        FcrossF = dqcFcrossF(stor7,iglw,w    ,igla,w    ,iglb,iy,iq)
      elseif(.not.lintw .and.      linta .and.      lintb)     then
        FcrossF = dqcFcrossF(w    ,iglw,stor7,igla,stor7,iglb,iy,iq)
      elseif(.not.lintw .and.      linta .and. .not.lintb)     then
        FcrossF = dqcFcrossF(w    ,iglw,stor7,igla,w    ,iglb,iy,iq)
      elseif(.not.lintw .and. .not.linta .and.      lintb)     then
        FcrossF = dqcFcrossF(w    ,iglw,w    ,igla,stor7,iglb,iy,iq)
      else
        FcrossF = dqcFcrossF(w    ,iglw,w    ,igla,w    ,iglb,iy,iq)
      endif

C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C==   ==================================================================
C==   Utility routines =================================================
C==   ==================================================================

C     ==================
      subroutine PushPar
C     ==================

C--   Save /qpars6/ to LIFO buffer

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'PUSHPAR'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif
C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
C--   Copy /pars6/ to buffer
      call sqcBufPar6(1,ierr)
C--   Error
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'LIFO buffer full: please call PULLPAR first')

      return
      end

C     ==================
      subroutine PullPar
C     ==================

C--   Restore /pars6/ from LIFO buffer

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'PULLPAR'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif
C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
C--   Copy buffer to /pars6/
      call sqcBufPar6(-1,ierr)
C--   Error
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'LIFO buffer empty: please call PUSHPAR first')

      return
      end

C     ================================
      subroutine EfromQQ(qvec,evec,nf)
C     ================================

C--   Transform coefficients from flavor basis to si/ns basis
C--
C--         -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6
C--   qvec  tb bb cb sb ub db  g  d  u  s  c  b  t
C--
C--   |qpm>  0  1  2  3  4  5  6  7  8  9 10 11 12
C--          g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--
C--   |epm>  0  1  2  3  4  5  6  7  8  9 10 11 12
C--          g  si 2+ 3+ 4+ 5+ 6+ v  2- 3- 4- 5- 6-

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qpdfs7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension qvec(-6:6), evec(12)

      character*80 subnam
      data subnam /'EFROMQQ ( QVEC, EVEC, NF )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif
C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
C--   Do the work      
      call sqcEfromQQ(qvec,evec,nf)
C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C     ================================
      subroutine QQfromE(evec,qvec,nf)
C     ================================

C--   Transform si/ns basis to flavor basis
C--
C--   |epm>  0  1  2  3  4  5  6  7  8  9 10 11 12
C--          g  si 2+ 3+ 4+ 5+ 6+ v  2- 3- 4- 5- 6-
C--
C--   |qpm>  0  1  2  3  4  5  6  7  8  9 10 11 12
C--          g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--
C--         -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6
C--   qvec  tb bb cb sb ub db  g  d  u  s  c  b  t

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qpdfs7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension evec(12), qpm(12), qvec(-6:6)

      character*80 subnam
      data subnam /'QQFROME ( EVEC, QVEC, NF )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif
C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Transform  (eq 2.29)
      do i = 1,nf
        biplu = 0.D0
        bimin = 0.D0
        do j = 1,nf
          biplu = biplu + evec(j  )*umateq(j,i)
          bimin = bimin + evec(6+j)*umateq(j,i)
        enddo
        qpm(i  ) = biplu
        qpm(i+6) = bimin
      enddo
C--   Get q,qbar coeff from qpm coeff  (eq 2.20)
      do i = -6,6
        qvec(i) = 0.D0
      enddo
      do i = 1,nf
        qvec( i) = qpm(i) + qpm(6+i)
        qvec(-i) = qpm(i) - qpm(6+i)
      enddo
 
C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

C==   ===============================================================
C==   Interpolation =================================================
C==   ===============================================================

C     ===========================================
      subroutine StfunXq(stfun,x,qmu2,stf,n,jchk)
C     ===========================================

C--   Interpolate stfun(ix,iq)

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'

      external stfun
      dimension x(*),qmu2(*),stf(*)

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'STFUNXQ ( STFUN, X, QMU2, STF, N, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif
C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
C--   Check user input 
      call sqcIlele(subnam,'N',1,n,mpt0,
     +    'You can increase mpt0 in qcdnum.inc (not recommended)')
C--   Global identifier of scratch pdf table
      jset = -1
      idt  =  1
      idg  = iqcIdPdfLtoG(jset,idt)
C--   Do the work
      call sqcStInterp(stfun,idg,x,qmu2,stf,n,ierr)
C--   Again, check on number of interpolations
      if(ierr.eq.-1) then
        call sqcErrMsg(subnam,
     + 'Too many interpolations: increase mpt0 in qcdnum.inc')
      endif
C--   Outside grid
      if(jchk.ne.0 .and. ierr.eq.1) then
        call sqcErrMsg(subnam,'At least one x or mu2 outside grid')
      endif
C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end
