

C--   This is the file usrevnn.f containing the evolution toolbox routines

C--   subroutine EvFillA (w,id,func)
C--   double precision function EvGetAA(w,id,iq,nf,ithresh)
C--   subroutine EvDglap(w,subr,idw,ida,idf,n,iq0,eps)
C--   double precision function EvPdfij(w,id,ix,iq,jchk)
C--   subroutine EvPlist(w,id,x,qmu2,pdf,m,jchk)
C--   subroutine sqcInterpList(subnam,w,igl,x,q,f,n,jchk)
C--   subroutine EvTable(w,id,x,nx,q,nq,table,jchk)
C--   subroutine EvPCopy(w,id,def,n,jset)

C     ==============================
      subroutine EvFillA (w,id,func)
C     ==============================

C--   Fill a type-6 table with expansion coefficients
C--
C--   w     (in)   Store containing type-6 table
C--   id    (in)   Type-6 table identifier in global format
C--   func  (in)   function declared external in the calling routine

      implicit double precision (a-h,o-z)

      external func

      include 'qcdnum.inc'
      include 'qmaps8.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /    1,     6   / , iflg /   0,    0  /
      data icma  /    1,     6   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam /'EVFILLA ( W, ID, FUNC )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
C--   Make sure Nfmap exists 
      if(.not.Lnfmap8) call sqcNfTab(0)
C--   Check global identifier
      igl = iqcSjekId(subnam,'ID',w,id,icmi,icma,iflg,lint)

C--   Do the work
      call sqcEvFillA(w,igl,func)

      return
      end

C     =====================================================
      double precision function EvGetAA(w,id,iq,nf,ithresh)
C     =====================================================

C--   Get expansion coefficient from a type-6 table
C--
C--   w        (in)   Store containing type-6 table
C--   id       (in)   Type-6 table identifier in global format
C--   iq       (in)   Qgrid point (may be < 0)
C--   nf      (out)   Number of flavours at iq
C--   ithresh (out)   Threshold indicator 0,+-1

C--   for iq < 0 nf = 3,4,5 instead of 4,5,6 at iqc,b,t

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /    1,     6   / , iflg /   0,    0  /
      data icma  /    1,     6   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam /'EVGETAA ( W, ID, IQ, NF, ITHRESH )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check global identifier
      igl = iqcSjekId(subnam,'ID',w,id,icmi,icma,iflg,lint)

C--   Check iq
      call sqcIlele(subnam,'IQ',1,abs(iq),ntt2,' ')

C--   Do the work
      EvGetAA = sqcEvGetAA(w,igl,iq,nf,ithresh)

      return
      end

C     ========================================================
      subroutine EvDglap(w,idw,ida,idf,start,m,n,iqlim,nf,eps)
C     ========================================================

C--   n-fold evolution routine
C--
C--   w          (in)    store with weight, coefficient and pdf tables
C--   idw(i,j,k) (in)    Pij weight table identifiers of order k
C--   ida(i,j,k) (in)    aij coefficient table identifiers of order k
C--   idf(i)     (in)    Pdf identifiers
C--   start(i,j) (inout) either starting pdf_i(xj) or discontinuity at iq0
C--   m          (in)    first 2 dims of idw and ida and first dim of start
C--   n          (in)    abs(n) = number of pdfs to evolve simultaneously
C--                      n > 0  = select input mode
C--                      n < 0  = select transfer mode
C--   iqlim(2)   (inout) requested iq0,iq1 on entry, actual iq0,iq1 on exit
C--   nf         (out)   number of flavours; nf < 0 flags end of range
C--   eps        (out)   max deviation at midpoint of xbins
C--
C--   n > 0 input    mode: start = input pdf on entry, evolved pdf on exit
C--   n < 0 transfer mode: start = discontinuity on entry, zero on exit

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qmaps8.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmiw(2), icmaw(2), iflgw(2)
C--                isign  itype              pzi  cfil
      data icmiw  /    1,     1   /, iflgw /   1,    1  /
      data icmaw  /    1,     4   /
      dimension icmia(2), icmaa(2), iflga(2)
C--                isign  itype              pzi  cfil
      data icmia  /    1,     6   /, iflga /   0,    1  /
      data icmaa  /    1,     6   /
      dimension icmif(2), icmaf(2), iflgf(2)
C--                isign  itype              pzi  cfil
      data icmif  /    1,     5   /, iflgf /   0,    0  /
      data icmaf  /    1,     5   /

      logical lint

      dimension w(*),idw(m,m,*),ida(m,m,*),idf(*)
      dimension start(m,*),finp(mce0,0:mxg0,mxx0)
      dimension iqlim(2)

      save finp

      character*80 subnam
      data subnam /
     +'EVDGLAP ( W, IDW, IDA, IDF, START, M, N, IQLIM, NF, EPS )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Setup flavor map
      if(.not.Lnfmap8) call sqcNfTab(0)

C--   Check number of coupled evolutions
      nabs = abs(n)
      call sqcIlele(subnam,'N',1,nabs,mce0,
     + 'To many coupled evolutions, please increase MCE0 in qcdnum.inc')

C--   Check dimension M
      if(m.lt.nabs) call sqcErrMsg(subnam,'Invalid dimension M < N' )

C--   Check order
      call sqcIlele(subnam,'IORD',1,iord6,nnopt6(0),
     + 'Please call SETORD(IORD) with correct value of IORD')

C--   Check starting point
      call sqcIlele(subnam,'IQLIM(1)',1,iqlim(1),ntt2,
     +'Startpoint of evolution out of range')

C--   Check Pij and Alfa table identifiers
      do i = 1,nabs
        do j = 1,nabs
          do k = 1,iord6
            ierr =
     +      iqcSjekId(subnam,'IDW',w,idw(i,j,k),icmiw,icmaw,iflgw,lint)
            ierr =
     +      iqcSjekId(subnam,'IDA',w,ida(i,j,k),icmia,icmaa,iflga,lint)
          enddo
        enddo
      enddo

C--   Check Pdf table identifiers
      do i = 1,nabs
        ierr =
     +      iqcSjekId(subnam,'IDF',w,idf(i),icmif,icmaf,iflgf,lint)
      enddo

C--   Bring evolution in range
      it1 = iqlim(1)
      it2 = min(iqlim(2),ntt2)
      it2 = max(it2,1)
      if(it2.ge.it1) then
C--     Forward evolution it2 >= it1
        iz1 = izfitU2(it1)                        !iz1 with higher nf
        nf  = nffiz2(iz1)
        iz2 = min(izfitU2(it2),izmaxf8(nf))
      else
C--     Backward evolution it2 < it1 
        iz1 = izfitL2(it1)                        !iz1 with lower nf
        nf  = nffiz2(iz1)
        iz2 = max(izfitL2(it2),izminf8(nf))
      endif

C--   Set start values
      if(n.gt.0) then
        fac = 0.D0           !input mode   : copy start array  to finp
      else
        fac = 1.D0           !transfer mode: add discontinuity to finp
      endif
      do iy = 1,nyy2(0)
        ix  = nyy2(0)+1-iy
        do ig = 0,nyg2
          do i = 1,nabs
            finp(i,ig,iy) = fac*finp(i,ig,iy) + start(i,ix)
          enddo
        enddo
      enddo

C--   Number of perturbative terms
      nopt = nnopt6(iord6)

C--   Do the work
      call sqcNNallg(w,idw,w,ida,w,idf,finp,m,nopt,nf,iz1,iz2,nabs,eps)

C--   Pass end values
      if(n.gt.0) then
        fac = 1.D0           !input mode   : copy finp to start array
      else
        fac = 0.D0           !transfer mode: set start array to zero
*        fac = 1.D0           !transfer mode: set start array to zero
      endif
      do iy = 1,nyy2(0)
        ix  = nyy2(0)+1-iy
        do i = 1,nabs
          start(i,ix) = fac*finp(i,0,iy)
        enddo
      enddo

C--   Pass evolution range
      iqlim(1) = itfiz2(iz1)
      iqlim(2) = itfiz2(iz2)

C--   Flag end of range
      if(iz2.eq.1 .or. iz2.eq.nzz2) nf = -nf

C--   Check max deviation
      if(dflim6.gt.0.D0 .and. eps.gt.dflim6) call sqcErrMsg(subnam, 
     +          'Possible spline oscillation detected')

      return
      end

C     ==================================================
      double precision function EvPdfij(w,id,ix,jq,jchk)
C     ==================================================

C--   Get pdf at gridpoint
C--
C--   w      (in)  store (not aligned)
C--   id     (in)  pdf table identifier in global format
C--   ix     (in)  x-grid index
C--   jq     (in)  q-grid index can be < 0

C--   for iq < 0 nf = 3,4,5 instead of 4,5,6 at iqc,b,t

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      include 'qmaps8.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /   -1,     5   / , iflg /   0,    1  /
      data icma  /    1,     5   /

      logical lint

      dimension w(*)

      character*80 subnam
      data subnam /'EVPDFIJ ( W, ID, IX, IQ, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

      iq = abs(jq)

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Setup flavor map
      if(.not.Lnfmap8) call sqcNfTab(0)

C--   Check Pdf identifier
      igl = iqcSjekId(subnam,'ID',w,id,icmi,icma,iflg,lint)

C--   Ranges (taking cuts into account)
      ixmi = nyy2(0)+1-iymac2
      iqmi = itfiz2(izmic2)
      iqma = itfiz2(izmac2)

C--   Check or nocheck thas the question
      if(jchk.ne.0) then
        call sqcIlele(subnam,'IX',ixmi,ix,nyy2(0),' ')
        call sqcIlele(subnam,'IQ',iqmi,iq,iqma   ,' ')
      else
        EvPdfij = qnull6
        if(ix.lt.ixmi .or. ix.gt.nyy2(0))    return
        if(iq.lt.iqmi .or. iq.gt.iqma)       return
      endif

C--   Do the work
      iy      = nyy2(0)+1-ix
      it      = jq
C--   Internal memeory
      if(lint) then
        EvPdfij = dqcEvPdfij(stor7,igl,iy,it)
C--   Local workspace
      else
        EvPdfij = dqcEvPdfij(w,    igl,iy,it)
      endif

      return
      end

C     =====================================
      subroutine EvPlist(w,id,x,q,f,n,jchk)
C     =====================================

C--   Process an arbitrarily long list of interpolation points
C--
C--   w          (in)  Workspace
C--   id         (in)  Global identifier of the pdf to be interpolated
C--   x(n),q(n)  (in)  List of interpolation points
C--   f(n)      (out)  List of interpolated results
C--   n          (in)  Number of interpolation points
C--   jchk       (in)  Yes/no check boundaries

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /   -1,     5   / , iflg /   0,    1  /
      data icma  /    1,     5   /

      logical lint

      dimension w(*), x(*), q(*), f(*)

      character*80 subnam
      data subnam /'EVPLIST ( W, ID, X, QMU2, PDF, N, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check Pdf identifier
      igl = iqcSjekId(subnam,'ID',w,id,icmi,icma,iflg,lint)

C--   Check user input
      if(n.le.0) call sqcErrMsg(subnam,'N should be larger than zero')

      if(lint) then
        call sqcInterpList(subnam,stor7,igl,x,q,f,n,jchk)
      else
        call sqcInterpList(subnam,w    ,igl,x,q,f,n,jchk)
      endif

      return
      end


C     =============================================
      subroutine EvTable(w,id,xx,nx,qq,nq,pdf,jchk)
C     =============================================

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /   -1,     5   / , iflg /   0,    1 /
      data icma  /    1,     5   /

      logical lint

      dimension w(*)
      dimension xx(nx), qq(nq), pdf(nx,nq),fff(nx*nq)

      character*80 subnam
      data subnam /'EVTABLE ( W, ID, X, NX, Q, NQ, TABLE, ICHK)'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
C--   Check Pdf identifier
      igl = iqcSjekId(subnam,'ID',w,id,icmi,icma,iflg,lint)
C--   Check user input
      if(nx.le.0) call sqcErrMsg(subnam,'NX .le. 0 not allowed')
      if(nq.le.0) call sqcErrMsg(subnam,'NQ .le. 0 not allowed')
C--   Findout x-range
      xmi = xminc2
      xma = xmaxc2    !exclude x = 1
      call sqcRange(xx,nx,xmi,xma,aepsi6,ixmi,ixma,ierrx)
      if(ierrx.eq.2) 
     +    call sqcErrMsg(subnam,'X not in strictly ascending order')
      if(jchk.ne.0 .and. (ixmi.ne.1 .or. ixma.ne.nx))
     +    call sqcErrMsg(subnam,'At least one X(i) out of range')
C--   Findout mu2-range
      qmi = qminc2
      qma = qmaxc2
      call sqcRange(qq,nq,qmi,qma,aepsi6,iqmi,iqma,ierrq)
      if(ierrq.eq.2) 
     +    call sqcErrMsg(subnam,'Q not in strictly ascending order')
      if(jchk.ne.0 .and. (iqmi.ne.1 .or. iqma.ne.nq))
     +    call sqcErrMsg(subnam,'At least one Q(i) out of range')
C--   Preset output table
      do iq = 1,nq
        do ix = 1,nx
          pdf(ix,iq) = qnull6
        enddo            
      enddo
C--   x or qmu2 completely out of range
      if(ierrx.ne.0 .or. ierrq.ne.0) return

C--   Now do the work
      nxx = ixma-ixmi+1
      nqq = iqma-iqmi+1
      if(lint) then
        call sqcEvTable(stor7,igl,xx(ixmi),nxx,qq(iqmi),nqq,fff)
      else
        call sqcEvTable(    w,igl,xx(ixmi),nxx,qq(iqmi),nqq,fff)
      endif

C--   Now copy linear store fff(i) to pdf(ix,iq)
      i = 0
      do iq = iqmi,iqma
        do ix = ixmi,ixma
          i = i+1
          pdf(ix,iq) = fff(i)
        enddo
      enddo

      return
      end

C     ===================================
      subroutine EvPCopy(w,id,def,n,jset)
C     ===================================

C--   Copy EVDGLAP pdfs from toobox workspace to internal memory
C--
C--   w            (in) : local workspace
C--   id(0:12+n)   (in) : list of global identifiers to copy (0=gluon)
C--   def(-6:6,12) (in) : def(i,j) is amount of flavour i in pdf j
C--   n            (in) : number of extra tables beyond 13
C--   jset         (in) : pdf set number in internal memory [5,mset0]

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      include 'qmaps8.inc'

      logical first
      save    first
      data    first /.true./

      character*60 emsg
      character*10 etxt

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension icmi(2), icma(2), iflg(2)
C--               isign  itype              pzi  cfil
      data icmi  /    1,     5   / , iflg /   0,    1 /
      data icma  /    1,     5   /


      dimension w(*)
      dimension id(0:12+n), def(-6:6,12)

      character*80 subnam
      data subnam /'EVPCOPY ( W, ID, DEF, N, ISET )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check status bits
      call sqcChkflg(1,ichk,subnam)

C--   Check n is in range
      call sqcIlele(subnam,'N',0,n,mpdf0-13,' ')

C--   Check pdf set in range
      call sqcIlele(subnam,'ISET',5,jset,mset0,' ')

C--   Setup flavor map
      if(.not.Lnfmap8) call sqcNfTab(0)
C--   Make sure astable exists
      if(.not.Lastab8) call sqcAlfTab(iord6)
C--   Initialize store
      if(.not.Lwtini8) call sqcIniWt
C--   Define kinematic plane
      if(.not.Levcut8) call sqcEvCuts(0)

C--   Now book jset if it does not already exist
      ntab  = 13+n
      ifrst = 0
      noalf = 0
      call sqcPdfBook(jset,ntab,ifrst,noalf,nwlast,ierr)

C--   Handle error code from sqcPdfBook
      if(ierr.ge.-3) then
        call sqcMemMsg(subnam,nwlast,ierr)
      elseif(ierr.eq.-4) then
        call sqcNtbMsg(subnam,'ISET',jset)
      else
        stop 'EvPcopy unkown error code from sqcPdfBook'
      endif

C--   Max number of flavours
      nfmax = nffiz2(nzz2)

C--   Check Pdf table identifiers
      do i = 0,2*nfmax
        ierr =
     +      iqcSjekId(subnam,'ID(i)',w,id(i),icmi,icma,iflg,lint)
      enddo
      do i = 13,n
        ierr =
     +      iqcSjekId(subnam,'ID(i)',w,id(i),icmi,icma,iflg,lint)
      enddo

      id0 = iqcIdPdfLtoG(jset,0)        !global gluon id in stor7

C--   Now copy the pdfs from w to stor7
      call sqcEvPCopy(w,id,def,n,id0,nfmax,ierr)

C--   Input pdfs not linearly independent
      if(ierr.gt.0) then
        call smb_itoch(2*ierr,etxt,ltxt)
        write(emsg,'(''First '',A,
     &  '' input pdfs not linearly independent'')') etxt(1:ltxt)
        call sqcErrMsg(subnam,emsg)
      endif
C--   Validate the pdfs and store the cuts
      do i = ifrst7(jset),ilast7(jset)
        idglobal = iqcIdPdfLtoG(jset,i)
        call sqcValidate(stor7,idglobal)
        call sqcSetCut5(stor7,idglobal,ymaxc2,tminc2,tmaxc2)
      enddo

C--   Store evolution parameters and alphas tables in jset
      call sqcCpyParI( 0,jset)  !  0 = copy parms from qpars6
      call sqcCpyAlfI(-1,jset)  ! -1 = copy atabs from set -1
C--   Store the type of evolution
      call sqcSetPar(stor7,isetf7(jset),idievtyp6,5)
C--   Set filled
      Lfill7(jset) = .true.

C--   Update status bits
      call sqcSetflg(iset,idel,jset)

      return
      end






