
C--   This is the file usrevol.f containing the evolution routines

C--   double precision function rfromf(f2)
C--   double precision function ffromr(r2)
C--   function asfunc(r2,nfout,ierr)
C--   double precision function EvolAs(iord,as0,r20,r2,iqcdnum,nfout,ierr)
C--   double precision function AlTabN(jset,jq,n,ierr)
C--   subroutine EvolFG(jset,func,def,iq0,epsi)

C==   ===============================================================
C==   Scale transformations =========================================
C==   ===============================================================
      
C     ====================================
      double precision function rfromf(f2)
C     ====================================

C--   Convert factorization scale f2 to renormalization scale r2.

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'

      logical first
      save    first
      data    first /.true./

      character*80 subnam
      data subnam /'RFROMF ( F2 )'/

C--   Check if QCDNUM is initialized
      if(first) then
        call sqcChkIni(subnam)
        first = .false.
      endif

C--   Transform
      rfromf = aar6*f2 + bbr6

      return
      end
      
C     ====================================
      double precision function ffromr(r2)
C     ====================================

C--   Convert renormalization scale r2 to factorization scale f2.

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'

      logical first
      save    first
      data    first /.true./

      character*80 subnam
      data subnam /'FFROMR ( R2 )'/

C--   Check if QCDNUM is initialized
      if(first) then
        call sqcChkIni(subnam)
        first = .false.
      endif

C--   Transform
      ffromr = (r2-bbr6)/aar6

      return
      end
      
C==   ===============================================================
C==   Alpha-s =======================================================
C==   ===============================================================
            
C     ===============================================
      double precision function asfunc(r2,nfout,ierr)
C     ===============================================

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qpars6.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'ASFUNC ( R2, NFOUT, IERR )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

      jord = iord6
      alf0 = alfq06
      ralf = q0alf6
C--   Update thresholds on the renormalisation scale in the VFNS
      if(nfix6.eq.0) call sqcRmass2(qthrs6,rthrs6)
C--   Evolve alfas
      asfunc = dqcAsEvol(r2,ralf,alf0,rthrs6,jord,nfout,ierr)

      return
      end
            
C     ===========================================================
      double precision function 
     +                 EvolAs(iord,as0,r20,r2,iqcdnum,nfout,ierr)
C     ===========================================================

C--   Evolve alphas(r2) at order iord.
C--   This function is not documented in the write-up but kept for
C--   backward compatibility
C--
C--   Input   iord     1 = LO, 2 = NLO, 3 = NNLO, other = QCDNUM setting
C--           as0      Starting alphas, QCDNUM setting if .le. 0  or .gt. 1
C--           r20      Starting R2,     QCDNUM setting if .le. 0.1
C--           r2       End scale should be .gt. 0.1
C--           iqcdnum  0 take thesholds as set by setthr (or by default)
C--                    1 take internal thresholds (rounded down to grid points)
C--   
C--   Output  nfout    number of flavors at r2
C--           ierr     0 all OK
C--                    1 r2 at or below lambda2
C--                    2 no internal thresholds available

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qpars6.inc'

      dimension rmsq(4:6)
 
      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam 
     +    /'EVOLAS ( IORD, AS0, R20, R2, INTERN, NFOUT, IERR )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Order taken from user input or from current QCDNUM setting
      if(iord.ge.1 .and. iord.le.3) then
        jord = iord
      else
        jord = iord6
      endif
C--   Alpha0 taken from user or from current QCDNUM setting
      if(as0.gt.0.D0 .and. as0.lt.1.D0) then
        alf0 = as0
      else
        alf0 = alfq06
      endif
C--   R20 taken from user or from current QCDNUM setting
      if(abs(r20).gt.0.1) then
        ralf = r20
      else
        ralf = q0alf6
      endif 
C--   Check which thresholds to take
      if(iqcdnum.ne.1) then
C--     Use the thresholds defined by setthrs or by default
C--     Convert these from factorization to renormalization scale
        do i = 4,6
          rmsq(i) = aar6*qthrs6(i) + bbr6
        enddo
      else
C--     Use the internal QCDNUM values
C--     First, check that grid is available (no error msg)
        call sqcChekit(1,ichk,jbit)
C--     error
        if(jbit.ne.0) then
          evolas = qnull6
          ierr   = 2
          return
        endif
C--     OK, now get the pole mass squared from QCDNUM
        do i = 4,6
          rmsq(i) = aar6*qthrs6(i) + bbr6
        enddo
      endif

      evolas = dqcAsEvol(r2,ralf,alf0,rmsq,jord,nfout,ierr)
      
      return
      end

C     ================================================
      double precision function AlTabN(jset,jq,n,ierr)
C     ================================================

C--   Returns (as/2pi)^n on the factorisation scale
C--
C--   jset  (in) : pdf set 0=base,1=unpol,2=pol,etc
C--   jq    (in) : mu2 grid point >0 nf = 4,5,6 at iqc,b,t
C--                               <0 nf = 3,4,5 at iqc,b,t
C--   n     (in) :  1, 2, 3 --> a^n in expansion (a, a^2, a^3)
C--                 0,-1,-2 --> a^n in expansion (1, a  , a^2)
C--   ierr (out) :  1  iq close or below Lambda^2
C--                 2  iq outside grid

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      include 'qmaps8.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'ALTABN ( ISET, IQ, N, IERR )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif
C--   Check status bits
      call sqcChkflg(1,ichk,subnam)
C--   Make sure Nfmap exists
      if(.not.Lnfmap8) call sqcNfTab(0)      
C--   Make sure astable exists
      if(.not.Lastab8) call sqcAlfTab(iord6)
C--   Make sure jset is in range
      call sqcIlele(subnam,'ISET',0,jset,mset0,' ')
C--   Make sure jset is filled
      kset = jset
      if(jset.eq.0) kset = -1
      if(.not.Lfill7(kset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Check range
      call sqcIlele(subnam,'N',-2,n,20,' ')
C--   Check iq inside grid 
      iq = abs(jq)
      if(iq.lt.1 .or. iq.gt.ntt2) then
        altabn = qnull6
        ierr   = 2
        return
      endif
C--   Not below Lambda, thank you      
      if(iq.lt.itmin6) then
        altabn = qnull6
        ierr   = 1
        return
      endif  
C--   Find out which iz index to take
      iz = izfit2(iq)
C--   Take iz-1 at threshold if jq preceeded by minus sign
      if(jq.lt.0 .and. iz.ne.1) then
        if(nffiz2(iz-1).eq.nffiz2(iz)-1) iz=iz-1
      endif
C--   Do the work
      ierr = 0
      if(n.eq.0)                      then
        altabn = 1.D0
      elseif(n.lt.0)                  then
        altabn = antab8(iz,n,kset)
      elseif(n.gt.0 .and. n.le.iord6) then
        altabn = antab8(iz,n,kset)
      else  
        altabn = (antab8(iz,0,kset))**n
      endif
 
C--   Update status bits
      call sqcSetflg(iset,idel,0)
      
      return
      end

          
C==   ===============================================================
C==   PDF evolution =================================================
C==   ===============================================================
            
      
C     ==========================================
      subroutine EvolFG(jset,func,def,iq0,epsi)
C     ==========================================

C--   Evolve all pdfs.
C-- 
C--   jset      (in) :  Type 1=unpol, 2=pol, 3=timelike, 4=custom
C--   func(j,x) (in) :  Function that returns input pdf_j(x) at iq0
C--   def(i,j)  (in) :  Contribution of flavour (i) to input pdf (j)
C--   iq0       (in) :  Starting scale
C--   epsi     (out) :  Largest deviation quad - lin at midpoint
C--   
C--   Flavor indices :  tb bb cb sb ub db  g  d  u  s  c  b  t
C--                     -6 -5 -4 -3 -2 -1  0  1  2  3  4  5  6
C--   q+- indices    :   g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
C--                      0  1  2  3  4  5  6  7  8  9 10 11 12  
C--   Si/ns  indices :   g  si 2+ 3+ 4+ 5+ 6+ va 2- 3- 4- 5- 6-
C--                      0  1  2  3  4  5  6  7  8  9 10 11 12    

      implicit double precision (a-h,o-z)

      external func

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'
      include 'qmaps8.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension def(-6:6,12), pdef(12,12)
      
      character*80 subnam
      data subnam /'EVOLFG ( ISET, FUNC, DEF, IQ0, EPSI )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check evolution type
      call sqcIlele(subnam,'ISET',1,jset,4,' ')

C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)

C--   Book pdf and alfas tables in stor7 (if they do not exist already)
      ntab  = 13
      ifrst = 0
      noalf = 0
      call sqcPdfBook(jset,ntab,ifrst,noalf,nwlast,ierr)

C--   Handle error code from sqcPdfBook
      if(ierr.ge.-3) then
        call sqcMemMsg(subnam,nwlast,ierr)
      elseif(ierr.eq.-4) then
        call sqcNtbMsg(subnam,'ISET',jset)
      else
        stop 'EvolFG: unkown error code from sqcPdfBook'
      endif

C--   Setup flavor map
      if(.not.Lnfmap8) call sqcNfTab(0)
C--   Setup kinematic plane
      if(.not.Levcut8) call sqcEvCuts(0)
      
C--   Check perturbative order
      call sqcIlele(subnam,'IORD',1,iord6,mxord7(jset),
     &              'Too large order, please call SETORD')       

C--   Check user input
      if(nfix6.eq.0) then
C--     VFNS
        call sqcIlele(subnam,'IQ0',1,iq0,itchm2-1,
     &                'IQ0 should be below charm threshold')
      else
C--     FFNS
        call sqcIlele(subnam,'IQ0',1,iq0,ntt2,' ')
      endif

C--   Initialize pdef 
      do i = 1,12
        do j = 1,12
          pdef(i,j) = 0.D0
        enddo
      enddo
C--   Predefine heavy quark qplus  in case nfmax < 4
      pdef( 7, 4) = 1.D0      !cplus
      pdef( 8,10) = 1.D0      !cmin
      pdef( 9, 5) = 1.D0      !bplus
      pdef(10,11) = 1.D0      !bmin
      pdef(11, 6) = 1.D0      !tplus
      pdef(12,12) = 1.D0      !tmin
C--   Build q+- matrix (pdef) from user input (def)
C--   Note index swap def(iflavor,ipdf) --> pdef(ipdf,iflavor)
      nfmax = max(nfix6,3)
      do ipdf = 1,2*nfmax
        do j = 1,6
          pdef(ipdf,j)   = 0.5*(def(j,ipdf)+def(-j,ipdf))
          pdef(ipdf,j+6) = 0.5*(def(j,ipdf)-def(-j,ipdf))
        enddo
      enddo 

C--   Now tell qcdnum about these definitions
      call sqcPdIdef(pdef,ierr)
C--   Well that did not go OK...
      if(ierr.ne.0) call sqcErrMsg(subnam,
     +             'Input quark densities not linearly independent')

C--   Global identifier of gluon
      idg = iqcIdPdfLtoG(jset,0)
C--   Enter input distributions
      call sqcAllInp(idg,func)

C--   Cut range
      iq1 = itfiz2(izmic2)
      iq2 = itfiz2(izmac2)
      call sqcIlele(subnam,'IQ0',iq1,iq0,iq2,
     +              'IQ0 should be inside cuts')

C--   Off we go...
      call sqcEvolve(jset,iord6,iq0,iq1,iq2,epsi,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     +                   'Attempt to evolve with too large alpha-s')

C--   Check max deviation
      if(dflim6.gt.0.D0 .and. epsi.gt.dflim6) call sqcErrMsg(subnam, 
     +          'Possible spline oscillation detected')

C--   Store evolution parameters and alphas tables
      call sqcCpyParI( 0,jset)  !  0 = copy parms from qpars6
      call sqcCpyAlfI(-1,jset)  ! -1 = copy atabs from set -1

C--   Store the type of evolution
      call sqcSetPar(stor7,isetf7(jset),idievtyp6,jset)

C--   Set filled
      Lfill7(jset) = .true.

C--   Update status bits
      call sqcSetflg(iset,idel,jset)
      
      return
      end
      

      
