
C--   This is the file usrglobal with conversions to global addressing

C--   integer function iqcIdAtab(id,iset)
C--   integer function iqcIaAtab(iz,id,iset)
C--   double precision function antab8(iz,i,iset)
C--   subroutine sqcCpyAlfI(iset1,iset2)

C--   subroutine sqcCpyParI(iset1,iset2)

C--   integer function iqcPdfIjkl(iy,it,id,itype)
C--   integer function iqcIdPdfLtoG(isetin,idin)
C--   subroutine sqcIdPdfGtoL(idgin,isetout,idout)
C--   subroutine sqcPdfLims(idg,iy1,iy2,it1,it2,jmax)

C--   integer function iqcIbufGlobal(ibuf)
C--   subroutine sqcInvalidateBuf(ibuf)

C=======================================================================
C==== User callable internal pdf addres function
C=======================================================================

C     =================================
      integer function iPdftab(iset,id)
C     =================================

C--   Returns minus the global pdf identifier in the internal store
C--
C--   iset  (in): pdf set identifier [1,mset0]
C--   id    (in): pdf identifier     [0,mpdf0]
C--
C--   Returns -1990XX if iset not in range   (ierr = 1)
C--           -2XX099 if id   not in range   (ierr = 2)
C--           -3XX0YY if iset does not exist (ierr = 3)
C--           -4XX0YY if id   does not exist (ierr = 4)
C--           -5XX0YY if id   is not filled  (ierr = 5)

      implicit double precision (a-h,o-z)      
      
      include 'qcdnum.inc'
      include 'qstor7.inc'

      logical lqcIsEvolved

      idg(ierr,iset,id) = 100000*ierr+1000*iset+id

      ierr = 0

C--   Bring input iset in range (2-digit poitieve number)
      if(iset.ge.1. .and. iset.le.mset0) then
        jset = iset
      else
        jset = 99
        ierr = 1
      endif
C--   Bring input id in range (2-digit positive number)
      if(id.ge.0 .and. id.le.mpdf0) then
        jd   = id
      else
        jd   = 99
        ierr = 2
      endif

C--   Error: iset and/or id were out of range
      if(ierr.ne.0) then
        iPdftab = -idg(ierr,jset,jd)
        return
      endif

C--   Find out if jset exists
      if(jset.ge.1 .and. jset.le.mset0) then
        kset = isetf7(jset)                  !kset might be 0
      else
        kset = 0
      endif

      if(kset.eq.0) then
C--     Error: jset does not exist
        ierr    =  3
        iPdftab = -idg(ierr,jset,jd)
        return
      elseif(jd.ge.ifrst7(jset) .and. jd.le.ilast7(jset)) then
C--     OK: jd does exist
        kd = jd - ifrst7(jset) + 501
      else
C--     Error: jd does not exist
        ierr    =  4
        iPdftab = -idg(ierr,jset,jd)
        return
      endif

C--   Get global identifier
      kdg = idg(ierr,kset,kd)

      if(.not.lqcIsEvolved(stor7,kdg)) then
C--     Error: pdf table is not evolved with current parameters
        ierr = 5
        iPdftab = -idg(ierr,jset,jd)
      else
C--     OK: pdf table is filled
        iPdftab = -kdg
      endif

      return
      end

C===================================================================
C==== Alpha_s table addressing =====================================
C===================================================================

C     ===================================
      integer function iqcIdAtab(id,iset)
C     ===================================

C--   Get global identifier of alfas table in internal memory
C--
C--   id   (in) :  table index [-mord0,mord0]
C--   iset (in) : -1 base tables
C--                0 not used here (fast tables)
C--               >0 table set id in internal memory [1,mset0]

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      if(iset.lt.-1 .or. iset.gt.mset0) stop 'iqcIdAtab wrong iset'
      if(id.lt.-mord0 .or. id.gt.mord0) stop 'iqcIdAtab wrong id'

      iqcIdatab = 1000*isetf7(iset) + 601 + id + mord0

      return
      end

C     ======================================
      integer function iqcIaAtab(iz,id,iset)
C     ======================================

C--   Get address of entry alfas(iz,id,iset) in internal memeory

C--   iz   (in) :  iz index    [1,nzz2]
C--   id   (in) :  table index [-mord0,mord0]
C--   iset (in) : -1 base tables
C--                0 not used here (fast tables)
C--               >0 table set id in internal memory [1,mset0]

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qstor7.inc'

      if(iz.lt.1 .or. iz.gt.nzz2) stop 'iqcIaAtab wrong iz'

      jd        = iqcIdAtab(id,iset)
      iqcIaAtab = iqcG6ij(stor7,iz,jd)

      return
      end

C     ===========================================
      double precision function antab8(iz,i,iset)
C     ===========================================

C--   Mimics the array antab8(iz,i) for set iset
C--
C--   iz   (in) :  iz index    [1,nzz2]
C--   i    (in) :  table index [-mord0,mord0]
C--   iset (in) : -1 base tables
C--                0 not used here (fast tables)
C--               >0 table set id in internal memory [1,mset0]

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'
      include 'qmaps8.inc'

      ia     = iqcIaAtab(iz,i,iset)
      antab8 = stor7(ia)

      return
      end

C     ==================================
      subroutine sqcCpyAlfI(iset1,iset2)
C     ==================================

C--   Copy alfas tables from iset1 to iset2 in internal memory
C--
C--   iset1,2  (in): -1=base,0=notused(fastbuf),1=unpol,2=pol,etc

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      if(iset1.lt.-1 .or. iset1.gt.mset0) stop 'sqcCpyAlfI wrong iset1'
      if(iset2.lt.-1 .or. iset2.gt.mset0) stop 'sqcCpyAlfI wrong iset2'

      do id = -mord0,mord0
        jd1 = iqcIdAtab(id,iset1)
        jd2 = iqcIdAtab(id,iset2)
        call sqcCopyType6(stor7,jd1,stor7,jd2)
      enddo

      return
      end

C=======================================================================
C==   Copy parameters ==================================================
C=======================================================================

C     ==================================
      subroutine sqcCpyParI(iset1,iset2)
C     ==================================

C--   Copy parameter list from iset1 to iset2 in internal memory
C--
C--   iset1,2  (in) : -1=base,0=seeremark,1=unpol,2=pol,etc
C--
C--   Remark: if iset1 = 0 then copy parameters directly from qpars6

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      kset2 = isetf7(iset2)

      if(iset1.eq.0) then
        call sqcGetPar6(stor7,kset2)
      else
        kset1 = isetf7(iset1)
        call sqcCpyPar(stor7,kset1,stor7,kset2)
      endif

      return
      end

C=======================================================================
C==   Pdf address in the internal store ================================
C=======================================================================

C     ===========================================
      integer function iqcPdfIjkl(iy,it,id,itype)
C     ===========================================

C--   Return address of pdf in internal memory
C--   iy    = y index
C--   it    = t index
C--   id    = pdf identifier [ifrst7,ilast7]
C--   itype = -1=scratch, 0=fastbuf, 1=unp, 2=pol, 3=frag, 5-mset0=ext

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      if(itype.lt.-1 .or. itype.gt.mset0) stop 'iqcPdfIjk wrong itype'

      if(id.ge.0) then
        if(id.lt.ifrst7(itype) .or. id.gt.ilast7(itype)) then
          write(6,*) 'iqcPdfIjk wrong id = ',id
          stop
        endif
        jd = 1000*isetf7(itype) + id - ifrst7(itype) + 501
      else
        if(-id.lt.ifrst7(-1) .or. -id.gt.ilast7(-1)) then
          write(6,*) 'iqcPdfIjk wrong id = ',id
          stop
        endif
        jd = 1000*isetf7(-1) - id - ifrst7(-1) + 501
      endif

      iqcPdfIjkl = iqcG5ijk(stor7,iy,it,jd)

      return
      end

C     ==========================================
      integer function iqcIdPdfLtoG(isetin,idin)
C     ==========================================

C--   Convert iset [0,mset0] and id [ifrst7,ilast7] to global pdf id

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      if(isetin.lt.-1 .or. isetin.gt.mset0) then
        write(6,*) 'iqcIdPdfLtoG wrong isetin = ',isetin
        stop 'iqcIdPdfLtoG wrong isetin'
      endif

      if(idin.ge.0) then
        if(idin.lt.ifrst7(isetin) .or. idin.gt.ilast7(isetin)) then
         write(6,*) 'iqcIdPdfLtoG wrong idin = ',idin
         stop
        endif
        iqcIdPdfLtoG = 1000*isetf7(isetin) + idin - ifrst7(isetin) + 501
      else
        if(-idin.lt.ifrst7(-1) .or. -idin.gt.ilast7(-1)) then
         write(6,*) 'iqcIdPdfLtoG wrong idin = ',idin
         stop
        endif
        iqcIdPdfLtoG = 1000*isetf7(-1) - idin - ifrst7(-1) + 501
      endif

      return
      end

C     ============================================
      subroutine sqcIdPdfGtoL(idgin,isetout,idout)
C     ============================================

C--   Convert global pdf id to iset [-1,mset0] and id [ifrst7,ilast7]
C--   Returns -2 if isetout not found

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      jset = abs(idgin)/1000
      if(jset.lt.1 .or. jset.gt.mst0) stop 'sqcIdPdfGtoL wrong jset'
      do i = -1,mset0
        if(jset.eq.isetf7(i)) then
          isetout = i
          idout   = abs(idgin) - 1000*isetf7(i) + ifrst7(i) - 501
          if(isetout.lt.-1 .or. isetout.gt.mset0)
     +    stop 'sqcIdPdfGtoL wrong isetout'
          if(idout.lt.ifrst7(i) .or. idout.gt.ilast7(i))
     +    stop 'sqcIdPdfGtoL wrong idout'
          return
        endif
      enddo

      stop 'sqcIdPdfGtoL isetout not found'
      isetout = -2
      idout   = -2

      return
      end

C     ===============================================
      subroutine sqcPdfLims(idg,iy1,iy2,it1,it2,jmax)
C     ===============================================

C--   Return stor7 pdf table limits

C--   idg     (in)  stor7 pdf identifier in global format
C--   iy1    (out)  lower y index
C--   iy2    (out)  upper y index
C--   it1    (out)  lower t index
C--   it2    (out)  upper t index
C--   jmax   (out)  upper j index in satellite table(j,id)

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      dimension imin(6),imax(6)

      call sqcGetLimits(stor7,idg,imin,imax,jmax)
      iy1 = imin(1)            !iymin
      iy2 = imax(1)            !iymax
      it1 = imin(2)            !itmin
      it2 = imax(2)            !itmax

      return
      end

C=======================================================================
C==  Fast buffer addressing ============================================
C=======================================================================

C     ====================================
      integer function iqcIbufGlobal(ibuf)
C     ====================================

C--   Get the global identifier of ibuf in internal memory stor7
C--
C--   ibuf (in) : Buffer index [1,mbf0]
C--
C--   Note that scratch buffers have iset = 0

      implicit double precision (a-h,o-z)

      iqcIbufGlobal = iqcIdPdfLtoG(0,ibuf)

      return
      end

C     =================================
      subroutine sqcInvalidateBuf(ibuf)
C     =================================

C--   Invalidate scratch buffer ibuf
C--
C--   ibuf (in) : buffer index in local format [1,mbf0]
C--
C--   Remark: ibuf = 0 invalidates all scratch buffers

      implicit double precision (a-h,o-z)
      
      include 'qcdnum.inc'
      include 'qstor7.inc'

      if(ibuf.eq.0) then
        ibmi = ibuf
        ibma = ibuf
      else
        ibmi = 1
        ibma = mbf0
      endif

C--   Global identifiers
      igmi = iqcIbufGlobal(ibmi)
      igma = iqcIbufGlobal(ibma)

C--   Invalidate tables
      do idg = igmi,igma
        call sqcInvalidate(stor7,idg)
      enddo

      return
      end


