
C--   This is the file usrpdf.f with user pdf output routines

C--   integer function iqcChkLmxq(subnam,w,id,xx,qq,ichk)
C--   integer function iqcChkLmij(subnam,w,id,ix,iq,ichk)
C--   subroutine fpdfxq(jset,xx,qq,pdf,jchk)
C--   subroutine fpdfij(jset,ix,iq,pdf,jchk)
C--   double precision function fsnsxq(jset,id,xx,qq,jchk) 
C--   double precision function fsnsij(jset,id,ix,iq,jchk)
C--   double precision function fvalxq(jset,xx,qq,jchk)
C--   double precision function fvalij(jset,id,ix,iq,jchk)
C--   double precision function fsumxq(jset,def,xx,qq,jchk)
C--   double precision function fsumij(jset,def,ix,iq,jchk)
C--   double precision function fsplne(jset,id,xx,iq)
C--   double precision function splchk(jset,id,iq)
C--   subroutine PdfLst(jset,def,x,q,f,n,jchk)
C--   subroutine SnsLst(jset,id,x,q,f,n,jchk)            <--tobedone
C--   subroutine PdfTab(jset,def,xx,nx,qq,nq,pdf,jchk)
C--   subroutine SnsTab(jset,def,xx,nx,qq,nq,pdf,jchk)   <--tobedone
C--   subroutine PdfCpy(jset1,jset2)
C--   subroutine PdfExt(subr,jset,n,offset,epsi)
C--   integer function NpTabs(jset)

C==   ===============================================================
C==   Check cuts ====================================================
C==   ===============================================================

C     ===================================================
      integer function iqcChkLmxq(subnam,w,id,xx,qq,ichk)
C     ===================================================

C--   Check if pdf(x,q) in internal memory is within limits
C--
C--   subnam (in)  : subroutine name for error message
C--   w      (in)  : store with pdf tables
C--   id     (in)  : pdf identifier in global format
C--   xx     (in)  : value of x
C--   qq     (in)  : value of mu2
C--   ichk   (in)  : 1=fatal error when xx,qq, out of range
C--
C--   iqcChkLmxq = -1 : id does not exist (fatal error)
C--                 0 : OK
C--                 1 : xx > xmax (fatal error when ichk = 1)
C--                 2 : xx < xmim (fatal error when ichk = 1)
C--                 3 : qq < qmin (fatal error when ichk = 1)
C--                 4 : qq > qmax (fatal error when ichk = 1)

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'

      dimension w(*)

      character*(*) subnam
      character*20  etxt
      character*80  emsg

C--   Default
      iqcChkLmxq = -1
C--   Check if xx,qq are inside cuts
      yy         = -log(max(xx,1.D-50))
      tt         =  log(max(qq,1.D-50))
      iqcChkLmxq =  iqcChkCut5(w,id,yy,tt)
      if(iqcChkLmxq.eq.-1) then
C--     Pdf does not exist
        call smb_itoch(id,etxt,ltxt)
        write(emsg,'(''Pdf id = '',A,'' not found'')') etxt(1:ltxt)
        call sqcErrMsg(subnam,emsg)
      elseif(iqcChkLmxq.eq.0 .or. ichk.eq.0) then
C--     Just pass error code
        continue
      else
C--     Fatal error xx,qq outside cuts
        call sqcGetCut5(w,id,yma,tmi,tma)
        xmi = exp(-yma)
        qmi = exp( tmi)
        qma = exp( tma)
        if    (iqcChkLmxq.eq.1) then
          call sqcCutMsgD(subnam,'X   ','XMAX',xx,1.D0,0)
        elseif(iqcChkLmxq.eq.2) then
          call sqcCutMsgD(subnam,'X   ','XMIN',xx,xmi ,0)
        elseif(iqcChkLmxq.eq.3) then
          call sqcCutMsgD(subnam,'QMU2','QMIN',qq,qmi ,0)
        elseif(iqcChkLmxq.eq.4) then
          call sqcCutMsgD(subnam,'QMU2','QMAX',qq,qma ,0)
        else
          stop 'iqcChkLmxq: unknown ifail code --> STOP'
        endif
      endif

      return
      end

C     ===================================================
      integer function iqcChkLmij(subnam,w,id,ix,iq,ichk)
C     ===================================================

C--   Check if pdf(i,j) in internal memory is within limits
C--
C--   subnam (in)  : subroutine name for error message
C--   w      (in)  : store with pdf tables
C--   id     (in)  : pdf identifier in global format
C--   ix     (in)  : value of ix
C--   iq     (in)  : value of iq
C--   ichk   (in)  : 1=fatal error when ix,iq, out of range
C--
C--   iqcChkLmij = -1 : id does not exist (fatal error)
C--                 0 : OK
C--                 1 : xx > xmax (fatal error when ichk = 1)
C--                 2 : xx < xmim (fatal error when ichk = 1)
C--                 3 : qq < qmin (fatal error when ichk = 1)
C--                 4 : qq > qmax (fatal error when ichk = 1)

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'

      dimension w(*)

      character*(*) subnam
      character*20  etxt
      character*80  emsg

C--   Default
      iqcChkLmij = -1
C--   Get xx and yy value
      if(ix.lt.1) then
        xx =  1.D-50
        yy =  50.D0
      elseif(ix.gt.nyy2(0)) then
        xx =  1.D50
        yy = -50.D0
      else
        iy = nyy2(0)+1-ix
        yy = ygrid2(iy)
        xx = exp(-yy)
      endif
C--   Get qq and tt value
      if(iq.lt.1) then
        qq = 0.9*qlimd6
        tt = log(qq)
      elseif(iq.gt.ntt2) then
        qq = 1.1*qlimu6
        tt = log(qq)
      else
        tt = tgrid2(iq)
        qq = exp(tt)
      endif

C--   Check if yy,tt are inside cuts
      iqcChkLmij =  iqcChkCut5(w,id,yy,tt)
      if(iqcChkLmij.eq.-1) then
C--     Pdf does not exist
        call smb_itoch(id,etxt,ltxt)
        write(emsg,'(''Pdf id = '',A,'' not found'')') etxt(1:ltxt)
        call sqcErrMsg(subnam,emsg)
      elseif(iqcChkLmij.eq.0 .or. ichk.eq.0) then
C--     Just pass error code
        continue
      else
C--     Fatal xx,qq outside cuts
        call sqcGetCut5(w,id,yma,tmi,tma)
        xmi = exp(-yma)
        qmi = exp( tmi)
        qma = exp( tma)
        if    (iqcChkLmij.eq.1) then
          call sqcCutMsgI(subnam,'IX','XMAX',ix,1.D0,0)
        elseif(iqcChkLmij.eq.2) then
          call sqcCutMsgI(subnam,'IX','XMIN',ix,xmi ,0)
        elseif(iqcChkLmij.eq.3) then
          call sqcCutMsgI(subnam,'IQ','QMIN',iq,qmi ,0)
        elseif(iqcChkLmij.eq.4) then
          call sqcCutMsgI(subnam,'IQ','QMAX',iq,qma ,0)
        else
          stop 'iqcChkLmij: unknown ifail code --> STOP'
        endif
      endif

      return
      end

C==   ===============================================================
C==   Pdf interpolation  ============================================
C==   ===============================================================

C     ======================================
      subroutine fpdfxq(jset,xx,qq,pdf,jchk)
C     ======================================

C--   Get all g,q,qbar pdfs of iset
C--
C--   jset      (in) : pdf set [1,mset0]
C--   jchk      (in) : # 0 return error if xx,qq out of range
C--                    = 0 return null  if xx,qq out of range
C--   pdf(-6:6) (in) : the whole lot...

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension pdf(-6:6),val(0:12)

      character*80 subnam
      data subnam /'FPDFXQ ( ISET, X, QMU2, PDFS, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits (also that pdf set is filled)
      call sqcChkflg(jset,ichk,subnam)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)
C--   Check cuts on gluon pdf
      idg   = iqcIdPdfLtoG(jset,0)
      ifail = iqcChkLmxq(subnam,stor7,idg,xx,qq,jchk)

      if(ifail.eq.0) then
C--     xx,qq inside cuts
C--          0  1  2  3  4  5  6  7  8  9 10 11 12
C--     val  g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-      
        call sqcAllQpQmyt(idg,-log(xx),log(qq),val)
C--     Copy val to pdf
C--          -6 -5 -4 -3 -2 -1 0  1  2  3  4  5  6
C--     pdf  tb bb cb sb ub db g  d  u  s  c  b  t
        pdf(0) = val(0)
        do i = 1,6
          pdf( i) = 0.5*(val(i)+val(6+i))
          pdf(-i) = 0.5*(val(i)-val(6+i))
        enddo
      else
C--     xx,qq outside cuts set pdfs to null
        do i = -6,6
          pdf(i) = qnull6
        enddo
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')
      
      return
      end
        
C     ======================================
      subroutine fpdfij(jset,ix,jq,pdf,jchk)
C     ======================================

C--   Get all g,q,qbar pdfs of iset
C--
C--   jset      (in) : pdf set [1,mset0]
C--   jchk      (in) : # 0 return error if xx,qq out of range
C--                    = 0 return null  if xx,qq out of range
C--   pdf(-6:6) (in) : the whole lot...

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      dimension pdf(-6:6),val(0:12)

      character*80 subnam
      data subnam /'FPDFIJ ( ISET, IX, IQ, PDFS, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits (also if pdf set is filled)
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)
C--   Check cuts on gluon pdf
      idg   = iqcIdPdfLtoG(jset,0)
      ifail = iqcChkLmij(subnam,stor7,idg,ix,abs(jq),jchk)

      if(ifail.eq.0) then
C--     xx,qq inside cuts
C--          0  1  2  3  4  5  6  7  8  9 10 11 12
C--     val  g  d+ u+ s+ c+ b+ t+ d- u- s- c- b- t-
        iy = nyy2(0)+1-ix
        call sqcAllQpQmij(idg,iy,jq,val)
C--     Copy val to pdf
C--          -6 -5 -4 -3 -2 -1 0  1  2  3  4  5  6
C--     pdf  tb bb cb sb ub db g  d  u  s  c  b  t
        pdf(0) = val(0)
        do i = 1,6
          pdf( i) = 0.5*(val(i)+val(6+i))
          pdf(-i) = 0.5*(val(i)-val(6+i))
        enddo
      else
C--     xx,qq outside cuts set pdfs to null
        do i = -6,6
          pdf(i) = qnull6
        enddo
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end 
      
C     ====================================================
      double precision function fsnsxq(jset,id,xx,qq,jchk)
C     ====================================================

C--   Get pdf stored in jset versus x,q2
C--
C--   jset  (in) : pdf set [1,mset0]
C--   id    (in) : g, si, ns identifier [0,12+n]
C--   jchk  (in) : # 0 return error if xx,qq out of range
C--                = 0 return null  if xx,qq out of range

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'FSNSXQ ( ISET, ID, X, QMU2, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check jset is in range
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits (also check if pdf set is filled)
      call sqcChkflg(jset,ichk,subnam)
C--   Check id is in range
      call sqcIlele(subnam,'ID',0,id,ilast7(jset),' ')

C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)

      fsnsxq = qnull6   !null by default

C--   Global pdf identifier
      idg   = iqcIdPdfLtoG(jset,id)
C--   Check cuts on pdf
      ifail = iqcChkLmxq(subnam,stor7,idg,xx,qq,jchk)
C--   xx,qq inside cuts
      if(ifail.eq.0) then
        fsnsxq = dqcEpmyt(idg,-log(xx),log(qq))
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C     ====================================================
      double precision function fsnsij(jset,id,ix,jq,jchk)
C     ====================================================

C--   Get pdf stored in jset versus ix,iq
C--
C--   jset  (in) : pdf set [1-9]
C--   id    (in) : g, si, ns identifier [0,12+n]
C--   jchk  (in) : # 0 return error if ix, iq out of range
C--                = 0 return null  if ix, iq out of range
C--
C--   Remark: jq > 0 then follow the 4,5,6 convention at iqc,b,t
C--              < 0 then follow the 3,4,5 convention at iqc,b,t

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'FSNSIJ ( ISET, ID, IX, IQ, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check user input
      call sqcIlele(subnam,'ID',0,id,ilast7(jset),' ')
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)

      fsnsij = qnull6   !null by default

C--   Check cuts on pdf
      idg   = iqcIdPdfLtoG(jset,id)
      ifail = iqcChkLmij(subnam,stor7,idg,ix,abs(jq),jchk)
C--   xx,qq inside cuts
      if(ifail.eq.0) then
        iy     = nyy2(0)+1-ix
        fsnsij = dqcEpmij(idg,iy,jq)
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C     ====================================================
      double precision function fvalxq(jset,id,xx,qq,jchk)
C     ====================================================

C--   Get gluon, quark, antiquark pdf
C--
C--   jset  (in) : pdf set [1,mset0]
C--   id    (in) : gluon, q, qbar [-6,6]
C--   jchk  (in) : # 0 return error if xx,qq out of range
C--                = 0 return null  if xx,qq out of range

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
 
      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'FVALXQ ( ISET, ID, X, QMU2, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)
C--   Check user input
      call sqcIlele(subnam,'ID',-6,id,6,' ')

      fvalxq = qnull6   !null by default

C--   Check cuts on gluon
      idg   = iqcIdPdfLtoG(jset,0)
      ifail = iqcChkLmxq(subnam,stor7,idg,xx,qq,jchk)
C--   xx,qq inside cuts
      if(ifail.eq.0) then
        call sqcQpQmyt(idg,abs(id),-log(xx),log(qq),qplus,qminu)
        if(id.ge.0)     then
          fvalxq = 0.5D0 * (qplus+qminu)
        else
          fvalxq = 0.5D0 * (qplus-qminu)
        endif
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C     ====================================================
      double precision function fvalij(jset,id,ix,jq,jchk)
C     ====================================================

C--   Get gluon, quarks or antiquarks
C--
C--   Input    jset    : pdf set [1,mset0]
C--            id      : gluon, q qbar identifier [-6,6]
C--            jchk # 0: error if xx,qq out of range
C--                 = 0: return null if xx,qq out of range

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'FVALIJ ( ISET, ID, IX, IQ, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)
C--   Check user input
      call sqcIlele(subnam,'ID',-6,id,6,' ')

      fvalij = qnull6   !null by default

C--   Check cuts on gluon pdf
      idg   = iqcIdPdfLtoG(jset,0)
      ifail = iqcChkLmij(subnam,stor7,idg,ix,abs(jq),jchk)
C--   xx,qq inside cuts
      if(ifail.eq.0) then
        iy = nyy2(0)+1-ix
        call sqcQpQmij(idg,abs(id),iy,jq,qplus,qminu)
        if(id.ge.0)     then
          fvalij = 0.5D0 * (qplus+qminu)
        else
          fvalij = 0.5D0 * (qplus-qminu)
        endif
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C     =====================================================
      double precision function fsumxq(jset,def,xx,qq,jchk)
C     =====================================================

C--   Weighted sum of quarks and antiquarks
C--
C--   Input    jset    : pdf set [1,mset0]
C--            jchk # 0: error if xx,qq out of range
C--                 = 0: return null if xx,qq out of range

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      dimension def(-6:6)
 
      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'FSUMXQ ( ISET, DEF, X, QMU2, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)

      fsumxq = qnull6   !null by default

C--   Check cuts on gluon pdf
      idg   = iqcIdPdfLtoG(jset,0)
      ifail = iqcChkLmxq(subnam,stor7,idg,xx,qq,jchk)
C--   xx,qq inside cuts
      if(ifail.eq.0) then
        fsumxq = dqcSumQQByt(idg,def,-log(xx),log(qq))
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C     =====================================================
      double precision function fsumij(jset,def,ix,jq,jchk)
C     =====================================================

C--   Weighted sum of quarks and antiquarks
C--
C--   Input    jset    : pdf set [1,mset0]
C--            jchk # 0: error if xx,qq out of range
C--                 = 0: return null if xx,qq out of range

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      dimension def(-6:6)
 
      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'FSUMIJ ( ISET, DEF, IX, IQ, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)

      fsumij = qnull6   !null by default

C--   Check cuts on gluon pdf
      idg   = iqcIdPdfLtoG(jset,0)
      ifail = iqcChkLmij(subnam,stor7,idg,ix,abs(jq),jchk)
C--   xx,qq inside cuts
      if(ifail.eq.0) then
        iy = nyy2(0)+1-ix
        fsumij = dqcSumQQBij(idg,def,iy,jq)
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C     ===============================================
      double precision function fsplne(jset,id,xx,iq)
C     ===============================================

C--   Spline interpolation in x
C--
C--   jset   (in) : pdf set [1,mset0]
C--   id     (in) : pdf identifier [ifrst7,ilast7]
C--   xx     (in) : value of x
C--   iq     (in) : iq grid point

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'FSPLNE ( ISET, ID, X, IQ )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)
C--   Check id
      call sqcIlele(subnam,'ID',ifrst7(jset),id,ilast7(jset),' ')

C--   Get qq and tt value
      if(iq.lt.1) then
        qq = 0.9*qlimd6
        tt = log(qq)
      elseif(iq.gt.ntt2) then
        qq = 1.1*qlimu6
        tt = log(qq)
      else
        tt = tgrid2(iq)
        qq = exp(tt)
      endif

      fsplne = qnull6   !null by default

C--   Global identifier
      idg   = iqcIdPdfLtoG(jset,id)
C--   Check cuts on pdf
      ifail = iqcChkLmxq(subnam,stor7,idg,xx,qq,1)
C--   xx,qq inside cuts
      if(ifail.eq.0) then
        yy      = -log(xx)
        fsplne  =  dqcXSplne(idg,yy,iq)
      endif
C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C     ============================================
      double precision function splchk(jset,id,jq)
C     ============================================

C--   Check for spline oscillations
C--
C--   jset  (in)  Pdf set [1,mset0]
C--   id    (in)  Si/ns pdf identifier [0-12]
C--   iq    (in)  mu2 grid point

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qpdfs7.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'SPLCHK ( ISET, ID, IQ )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)
C--   Check user input
      call sqcIlele(subnam,'ID',ifrst7(jset),id,ilast7(jset),' ')
      iq = abs(jq)
      call sqcIlele(subnam,'IQ', 1, iq, ntt2,' ')

C--   Go ...
      iqmi = itfiz2(izmic2)
      iqma = itfiz2(izmac2)
      if(iq.lt.iqmi .or. iq.gt.iqma) then
         splchk = 0.D0
      else
         idg    = iqcIdPdfLtoG(jset,id)
         splchk = dqcSplChk(idg,jq)
      endif

C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C==   ===============================================================
C==   Fast pdf interpolation  =======================================
C==   ===============================================================
            
C     ========================================      
      subroutine PdfLst(jset,def,x,q,f,n,jchk)
C     ========================================

C--   Process list of interpolation points.
C--   The number of interpolations can be larger than mpt0 since
C--   this routine autmatically buffers in chunks of mpt0 words.
C--
C--   jset       (in)   pdf set [1,mset0]
C--   def(-6:6)  (in)   coefficients, ..., sb, ub, db, g, d, u, s, ...
C--                     for gluon  set def(0) = non-zero
C--                     for quarks set def(0) = 0.D0   
C--   x          (in)   list of x-points
C--   q          (in)   list of mu2 points
C--   f          (out)  list of interpolated pdfs
C--   n          (in)   number of items in the list
C--   jchk       (in)   0/1  no/yes check grid boundary  
      
      implicit double precision (a-h,o-z)
      
      include 'qcdnum.inc'
      include 'qstor7.inc'
      
      dimension xx(mpt0),qq(mpt0)
      dimension def(-6:6), coef(0:12,3:6)
      dimension x(*), q(*), f(*)
      
      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'PDFLST ( ISET, DEF, X, Q, F, N, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif
C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')      
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)

C--   Check user input
      if(n.le.0) call sqcErrMsg(subnam,'N should be larger than zero')
C--   Book scratch buffers
      call sqcUFBook(subnam)
C--   ibg is a temporary buffer
      ibg = iqcIdPdfLtoG(-1,1)
C--   idg is the gluon identifier of jset
      idg = iqcIdPdfLtoG(jset,0)

C--   Translate the coefficients def(-6:6) from flavour space to 
C--   singlet/non-singlet space
      if(def(0).eq.0.D0) then                !quarks
        do nf = 3,6
          coef(0,nf) = 0.D0                  
          call efromqq(def, coef(1,nf), nf) 
        enddo
      else                                   !gluon
        do nf = 3,6
          coef(0,nf) = def(0)
          do i = 1,12
            coef(i,nf) = 0.D0 
          enddo  
        enddo
      endif  
      
C--   Fill output array f in batches of mpt0 words
      ipt = 0
      jj  = 0      
      do i = 1,n
        ipt     = ipt+1
        xx(ipt) = x(i)
        qq(ipt) = q(i)
        if(ipt.eq.mpt0) then
          call sqcUFIni(subnam,xx,qq,mpt0,jchk)        !interpolation points
          call sqcFastPdf(idg,coef,ibg,0)              !fill sparse buffer
          call sqcFastFxq(stor7,ibg,f(jj*mpt0+1),mpt0) !interpolate
          ipt = 0
          jj  = jj+1
        endif
      enddo
C--   Flush remaining ipt points
      if(ipt.ne.0) then
        call sqcUFIni(subnam,xx,qq,ipt,jchk)        !interpolation points 
        call sqcFastPdf(idg,coef,ibg,0)             !fill sparse buffer
        call sqcFastFxq(stor7,ibg,f(jj*mpt0+1),ipt) !interpolate
      endif

C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C     ================================================
      subroutine PdfTab(jset,def,xx,nx,qq,nq,pdf,jchk)
C     ================================================

C--   Pdf linear combination interpolated on a grid
C--
C--   jset   (in)   pdf set [1,mset0]
C--   def    (in)   def(-6:6) coefficients of linear combination
C--                     for gluon  set def(0) = non-zero
C--                     for quarks set def(0) = 0.D0 
C--   xx     (in)   input array of x-values
C--   nx     (in)   dimension of xx in the calling routine
C--   qq     (in)   input array of mu2-values
C--   nq     (in)   dimension of qq in the calling routine
C--   pdf    (out)  pdf(nx,nq) 2-dim array of interpolated pdfs
C--   jchk   (in)   # 0: error if xx,qq out of range
C--                 = 0: return null if xx,qq out of range

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qgrid2.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'

      dimension def(-6:6)
 
      logical first
      save    first
      data    first /.true./

      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)
      
      dimension xx(nx), qq(nq), pdf(nx,nq), fff(nx*nq)
      
      character*80 subnam
      data subnam /'PDFTAB ( ISET, DEF, XX, NX, QQ, NQ, PDF, ICHK )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif
C--   Check pdf set
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')      
C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
C--   Check pdf set is filled
      if(.not.Lfill7(jset)) call sqcSetMsg(subnam,'ISET',jset)
C--   Save current evolution parameters
      call sqcBufPar6(1,ierr)
      if(ierr.eq.1) call sqcErrMsg(subnam,
     + 'Parameter buffer full: please call PULLPAR before')
C--   Set jset evolution parameters
      ksetj = isetf7(jset)
      call sqcSetPar6(stor7,ksetj)
C--   Check user input
      if(nx.le.0) call sqcErrMsg(subnam,'NX .le. 0 not allowed')
      if(nq.le.0) call sqcErrMsg(subnam,'NQ .le. 0 not allowed')
C--   Findout x-range
      xmi = xminc2
      xma = xmaxc2    !exclude x = 1
      call sqcRange(xx,nx,xmi,xma,aepsi6,ixmi,ixma,ierrx)
      if(ierrx.eq.2) 
     +    call sqcErrMsg(subnam,'XX not in strictly ascending order')
      if(jchk.ne.0 .and. (ixmi.ne.1 .or. ixma.ne.nx))
     +    call sqcErrMsg(subnam,'At least one XX(i) out of range') 
C--   Findout mu2-range
      qmi = qminc2
      qma = qmaxc2
      call sqcRange(qq,nq,qmi,qma,aepsi6,iqmi,iqma,ierrq)
      if(ierrq.eq.2) 
     +    call sqcErrMsg(subnam,'QQ not in strictly ascending order')
      if(jchk.ne.0 .and. (iqmi.ne.1 .or. iqma.ne.nq))
     +    call sqcErrMsg(subnam,'At least one QQ(i) out of range')
C--   Preset output table
      do iq = 1,nq
        do ix = 1,nx
          pdf(ix,iq) = qnull6
        enddo            
      enddo
C--   x or qmu2 completely out of range
      if(ierrx.ne.0 .or. ierrq.ne.0) return

C--   Scratch buffer
      itmp  = iqcIdPdfLtoG(-1,1)
C--   Gluon table
      idgl  = iqcIdPdfLtoG(jset,0)
C--   Now do the work
      nxx = ixma-ixmi+1
      nqq = iqma-iqmi+1
      call sqcPdfTab(idgl,def,itmp,xx(ixmi),nxx,qq(iqmi),nqq,fff)
      
C--   Now copy linear store fff(i) to pdf(ix,iq)
      i = 0
      do iq = iqmi,iqma
        do ix = ixmi,ixma
          i = i+1
          pdf(ix,iq) = fff(i)
        enddo
      enddo

C--   Restore current evolution parameters
      call sqcBufPar6(-1,ierr)
      if(ierr.eq.-1 .or. ierr.eq.2) call sqcErrMsg(subnam,
     + 'Parameter buffer empty: please call PUSHPAR before')

      return
      end
      
C==   ===============================================================
C==   External pdf input ============================================
C==   ===============================================================

C     ==============================
      subroutine PdfCpy(jset1,jset2)
C     ==============================

C--   Copy jset1 [1,3] to jset2 [5,mset0]

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qsflg4.inc'
      include 'qibit4.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./
      
      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'PDFCPY ( ISET1, ISET2 )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check jset1,2
      call sqcIlele(subnam,'ISET1',1,jset1,3,' ')
      call sqcIlele(subnam,'ISET2',5,jset2,mset0,' ')

C--   Check status bits
      call sqcChkflg(jset1,ichk,subnam)

C--   Check jset1 is filled
      if(.not.Lfill7(jset1)) call sqcSetMsg(subnam,'ISET1',jset1)

C--   Now book jset2 if it does not already exist
      ntab  = 13
      ifrst = 0
      noalf = 0
      call sqcPdfBook(jset2,ntab,ifrst,noalf,nwlast,ierr)

C--   Handle error code from sqcPdfBook
      if(ierr.ge.-3) then
        call sqcMemMsg(subnam,nwlast,ierr)
      elseif(ierr.eq.-4) then
        call sqcNtbMsg(subnam,'ISET',jset2)
      else
        stop 'PdfCpy unkown error code from sqcPdfBook'
      endif

C--   Now copy pdfs, parameter list and alfas tables
      kset1 = isetf7(jset1)
      kset2 = isetf7(jset2)
      call sqcPdfCpy(kset1,kset2)

C--   Set filled
      Lfill7(jset2) = .true.

C--   Update status bits
      call sqcSetflg(iset,idel,jset2)

      return
      end

C     ==========================================
      subroutine PdfExt(subr,jset,n,offset,epsi)
C     ==========================================

C--   Read pdfs into memory
C--
C--   subr    (external) user supplied subroutine
C--   jset    (in)       pdf set [5,mset0]
C--   n       (in)       number of pdfs beyond gluon,q,qbar
C--   offset  (in)       threshold offset mu2h*(1+-delta)
C--   epsi    (out)      estimate of spline accuracy
C--
C--   subr(x,mu2,qqbar) should return qqbar(-6:6+n)= ...,ub,db,gl,d,u,..

      implicit double precision (a-h,o-z)

      external subr

      include 'qcdnum.inc'
      include 'qluns1.inc'
      include 'qgrid2.inc'
      include 'qsflg4.inc'
      include 'qibit4.inc'
      include 'qpars6.inc'
      include 'qstor7.inc'
      include 'qpdfs7.inc'
      include 'qmaps8.inc'

      logical first
      save    first
      data    first /.true./
      
      save      ichk,       iset,       idel
      dimension ichk(mbp0), iset(mbp0), idel(mbp0)

      character*80 subnam
      data subnam /'PDFEXT ( SUBR, ISET, N, DELTA, EPSI )'/

C--   Initialize flagbits
      if(first) then
        call sqcMakeFl(subnam,ichk,iset,idel)
        first = .false.
      endif

C--   Check pdf set in range
      call sqcIlele(subnam,'ISET',5,jset,mset0,' ')
C--   Check extra tables are in range
      call sqcIlele(subnam,'N',0,n,mpdf0-13,' ')

C--   Check status bits
      call sqcChkflg(jset,ichk,subnam)
      
C--   Invalidate pdfs in jset
C--   This blocks access to jset via subr
      call sqcDelbit(ibPdfs4,istat4(1,jset),mbp0)

C--   Setup flavor map
      if(.not.Lnfmap8) call sqcNfTab(0)
C--   Make sure astable exists
      if(.not.Lastab8) call sqcAlfTab(iord6)
C--   Initialize store
      if(.not.Lwtini8) call sqcIniWt
C--   Define kinematic plane
      if(.not.Levcut8) call sqcEvCuts(0)

C--   Now book jset if it does not already exist
      ntab  = 13+n
      ifrst = 0
      noalf = 0
      call sqcPdfBook(jset,ntab,ifrst,noalf,nwlast,ierr)

C--   Handle error code from sqcPdfBook
      if(ierr.ge.-3) then
        call sqcMemMsg(subnam,nwlast,ierr)
      elseif(ierr.eq.-4) then
        call sqcNtbMsg(subnam,'ISET',jset)
      else
        stop 'PdfExt unkown error code from sqcPdfBook'
      endif

C--   Global id of gluon table
      ig0  = iqcIdPdfLtoG(jset, 0)

C--   Now import pdf set as defined by subr
      call sqcPdfExt(subr,ig0,n,offset)

C--   Check for spline oscillations
      epsi = 0.D0 
      do id = ifrst7(jset),ilast7(jset)
        idg = iqcIdPdfLtoG(jset,id)
        iq1 = itfiz2(izmic2)
        iq2 = itfiz2(izmac2)
        do iq = iq1,iq2
          eps  = dqcSplChk(idg,iq)
          epsi = max(epsi,eps)
        enddo  
      enddo

C--   Check max deviation
      if(dflim6.gt.0.D0 .and. epsi.gt.dflim6) call sqcErrMsg(subnam, 
     +          'Possible spline oscillation detected')

C--   Global identifiers
      kset   = isetf7(jset)
      igfrst = iqcIdPdfLtoG(jset,ifrst7(jset))
      iglast = iqcIdPdfLtoG(jset,ilast7(jset))
      idmax  = iqcGetNumberOfTables(stor7,kset,5)+ifrst7(jset)-1
      igmax  = iqcIdPdfLtoG(jset,idmax)

C--   Validate the pdfs, set iset pointer and store the cuts
      do idg = igfrst,iglast
        call sqcValidate(stor7,idg)
        call sqcSetCut5(stor7,idg,ymaxc2,tminc2,tmaxc2)
      enddo
C--   Invalidate pdfs that were not filled by sqcPdfExt (if any)
      do idg = iglast+1,igmax
        call sqcInvalidate(stor7,idg)
      enddo
C--   Store evolution parameters and alphas tables
      call sqcCpyParI( 0,jset)  !  0 = copy parms from qpars6
      call sqcCpyAlfI(-1,jset)  ! -1 = copy atabs from set -1
C--   Store the type of evolution
      call sqcSetPar(stor7,isetf7(jset),idievtyp6,5)
C--   Set filled
      Lfill7(jset) = .true.

C--   Update status bits
      call sqcSetflg(iset,idel,jset)
      
      return
      end

C     =============================
      integer function NpTabs(jset)
C     =============================

C--   Return the number of pdf tables in jset

      implicit double precision (a-h,o-z)

      include 'qcdnum.inc'
      include 'qstor7.inc'

      logical first
      save    first
      data    first /.true./

      character*80 subnam
      data subnam /'NPTABS ( ISET )'/

C--   Check if QCDNUM is initialized
      if(first) then
        call sqcChkIni(subnam)
        first = .false.
      endif
C--   Check jset in range
      call sqcIlele(subnam,'ISET',1,jset,mset0,' ')
C--   Go ...
      if(Lfill7(jset)) then
        NpTabs = ilast7(jset)-ifrst7(jset)+1
      else
        NpTabs  = 0
      endif

      return
      end


      
