C     ------------------------------------------------------------------
      program Toolbox04
C     ------------------------------------------------------------------
C--   QCDNUM toolbox example #4
C--   Add the routines SetStart and EvolSGNS.
C--   Evolution in the FFNS and partially in the VFNS (no heavy quarks).
C--   The routine CompareU compares unpolarised pdfs with QCDNUM.
C--
C--   QCDNUM  routines are in capitals
C--   Package routines are in mixed case (except the splitting functions)
C     ------------------------------------------------------------------
      implicit double precision (a-h,o-z)

C--   ------------------------------------------------------------------
C--   QCDNUM declarations and parameter settings
C--   ------------------------------------------------------------------
C--   x-grid
      dimension xmin(5), iwt(5)
      data nxmi/5/, nxin/100/, iosp/3/
      data xmin/1.D-5,0.2D0,0.4D0,0.6D0,0.75D0/
      data iwt / 1, 2, 4, 8, 16 /
C--   mu2-grid
      dimension qq(2),wt(2)
      data qq/2.D0,1.D4/, wt/1.D0,1.D0/, nqin/60/
C--   FFNS or VFNS and flavour thresholds
      data nfin/3/, q2c/3.D0/, q2b/25.D0/, q2t/1.D11/
C--   Input alphas
      data as0/0.364/, r20/2.D0/
C--   LO/NLO/NNLO,  unpol/pol/frag and starting scale
      data iord/1/, q0/2.D0/
C--   Renormalisation scale and factorisation scale
      data aar/1.D0/, bbr/0.D0/, aaq/1.D0/, bbq/0.D0/
C--   Input parton distributions and flavour composition of these
      external func
      dimension def(-6:6,12)
      data def  /
C--   tb  bb  cb  sb  ub  db   g   d   u   s   c   b   t
C--   -6  -5  -4  -3  -2  -1   0   1   2   3   4   5   6  
     + 0., 0., 0., 0., 0.,-1., 0., 1., 0., 0., 0., 0., 0.,   !dval
     + 0., 0., 0., 0.,-1., 0., 0., 0., 1., 0., 0., 0., 0.,   !uval
     + 0., 0., 0.,-1., 0., 0., 0., 0., 0., 1., 0., 0., 0.,   !sval
     + 0., 0., 0., 0., 0., 1., 0., 0., 0., 0., 0., 0., 0.,   !dbar
     + 0., 0., 0., 0., 1., 0., 0., 0., 0., 0., 0., 0., 0.,   !ubar
     + 0., 0., 0., 1., 0., 0., 0., 0., 0., 0., 0., 0., 0.,   !sbar
     + 0., 0.,-1., 0., 0., 0., 0., 0., 0., 0., 1., 0., 0.,   !cval
     + 0., 0., 1., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,   !cbar
     + 0.,-1., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1., 0.,   !bval
     + 0., 1., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,   !bbar
     +-1., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1.,   !tval
     + 1., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0. /  !tbar
C--   Weight files
      character*24 fnamq(2)
C--               123456789012345678901234
      data fnamq /'../weights/QcdNumUnp.wgt',
     +            '../weights/QcdNumPol.wgt' /

C--   ------------------------------------------------------------------
C--   Put here your own declarations and data statements
C--   ------------------------------------------------------------------
      parameter( nw = 231525 )
      dimension w(nw)
      dimension itypes_alf(6),itypes_pdf(6)
      data itypes_alf / 0, 0, 0, 0, 0, 1/
      data itypes_pdf / 0, 0, 0, 0,13, 0/
      character*12 key
      data key /'MyAddon v2.0'/
      character*24 fnamu(2)
      data fnamu /'../weights/QcdUsrUnp.wgt',
     +            '../weights/QcdUsrPol.wgt' /
      parameter (nxmax = 200) !change this when you use a larger xgrid
      dimension start(2,nxmax)

C--   ------------------------------------------------------------------
C--   QCDNUM set-up and pdf evolution
C--   ------------------------------------------------------------------

      lun    = 6            !lun = 6 stdout, -6 stdout w/out banner page
      lunout = abs(lun)      
      call QCINIT(lun,' ')
      call GXMAKE(xmin,iwt,nxmi,nxin,nx,iosp)
      call GQMAKE(qq,wt,2,nqin,nq)
      lun = NXTLUN(0)
      do iset = 1,2
        call READWT(lun,fnamq(iset),idmin,idmax,nwpdf,ierr)
        if(ierr.ne.0) then
          call FILLWT(iset,idmin,idmax,nwpdf)
          call DMPWGT(iset,lun,fnamq(iset))
        endif
      enddo
      call SETORD(iord)                         !LO, NLO, NNLO
      call SETALF(as0,r20)                      !input alphas
      iqc  = IQFRMQ(q2c)                        !charm threshold
      iqb  = IQFRMQ(q2b)                        !bottom threshold
      call SETCBT(nfin,iqc,iqb,999)             !thesholds in the vfns
      call SETABR(aar,bbr)                      !renormalisation scale
      iq0  = IQFRMQ(q0)                         !starting scale
      do iset = 1,2
        call EVOLFG(iset,func,def,iq0,eps)      !evolve all pdf's
      enddo

      write(lunout,'('' xmin ='',1PE11.2,3X,''subgrids  ='',I4)')
     &  xmin(1), nxmi
      write(lunout,'('' qmin ='',1PE11.2,3X,''qmax      ='',1PE11.2)')
     &  qq(1), qq(2)
      write(lunout,'('' nfin ='',I4,10X,''q2c, b, t ='',3(1PE11.2))')
     &  nfin, q2c, q2b, q2t
      write(lunout,'('' alfa ='',1PE11.2,3X,''r20       ='',1PE11.2)')
     &  as0, r20
      write(lunout,'('' aar  ='',1PE11.2,3X,''bbr       ='',1PE11.2)')
     &  aar, bbr
      write(lunout,'('' q20  ='',1PE11.2)') q0
      write(lunout,'(/)')

C--   ------------------------------------------------------------------
C--   User code
C--   ------------------------------------------------------------------

C--   ------------------------------------------------------------------
C--   Weights
C--   ------------------------------------------------------------------

      lun = nxtlun(0)
      call READTAB(w,nw,lun,fnamu(1),key,0,iset_piju,nwu,ie)
      if(ie.ne.0) then
        call FillWtU(w,nw,iset_piju)
        call DUMPTAB(w,iset_piju,lun,fnamu(1),key)
      endif  

      call READTAB(w,nw,lun,fnamu(2),key,0,iset_pijp,nwu,ie)
      if(ie.ne.0) then
        call FillWtP(w,nw,iset_pijp)
        call DUMPTAB(w,iset_pijp,lun,fnamu(2),key)
      endif
        
      call MAKETAB(w,nw,itypes_alf,0,0,iset_alfa, nwu)
      call MAKETAB(w,nw,itypes_pdf,0,0,iset_pdfu, nwu)
      call MAKETAB(w,nw,itypes_pdf,0,0,iset_pdfp, nwu)

C--   ------------------------------------------------------------------
C--   Evolution
C--   ------------------------------------------------------------------

      if(nx.gt.nxmax) stop 'ToolBox04: too many x-grid points'

      iqq     = nq             !comparison gridpoint
      iqstart = iq0
      iset    = 1              !1= unpolarised   2 = polarised
      isetp   = iset_piju
      isetf   = iset_pdfu
      if(iset.eq.2) then
        isetp = iset_pijp
        isetf = iset_pdfp
      endif

C--   id    0   1   2   3   4   5   6   7   8   9  10  11  12
C--   pdf  gl  si  e2+ e3+ e4+ e5+ e6+ va  e2- e3- e4- e5- e6-
C--   nf        3   3   3   4   5   6   3   3   3   4   5   6

C--   Singlet/gluon
      call SetStart(iset, 1, iqstart, start)       !singlet
      call SetStart(iset, 0, iqstart, start)       !gluon
      call EvolSGNS(w, iset_alfa, isetp, isetf, 1, iqstart, start)

      call CompareU(w, iset_pdfu, 0, iqq, 0, dif, lunout)
      call CompareU(w, iset_pdfu, 1, iqq, 0, dif, lunout)

      if(nfin .eq. 0 ) then
        idmax1 = 3                             !VFNS light quarks only
        idmax2 = 9
      else
        idmax1 = nfin                          !FFNS
        idmax2 = nfin+6
      endif

C--   Nonsinglets
      call setval('elim',-1.D0)
      do idf = 2,idmax1
        call SetStart(iset, idf, iqstart, start)   !nonsinglet e^+

        call EvolSGNS(w, iset_alfa, isetp, isetf, idf, iqstart, start)
        call CompareU(w, iset_pdfu, idf, iqq, 0, dif, lunout)
      enddo
      do idf = 7,idmax2
        call SetStart(iset, idf, iqstart, start)   !nonsinglet e^-
        call EvolSGNS(w, iset_alfa, isetp, isetf, idf, iqstart, start)
        call CompareU(w, iset_pdfu, idf, iqq, 0, dif, lunout)
      enddo

      end

C--   ------------------------------------------------------------------
C--   User routines
C--   ------------------------------------------------------------------

C     =======================================================
      subroutine CompareU(ww,iset_pdfu,idf,iq,nprint,dif,lun)
C     =======================================================

C--   Compare unpolarised pdfs
C--   ww        (in)      workspace
C--   iset_pdfu (in)      table set identifier unpolarised pdfs
C--   idf       (in)      pdf identifier [0-12]
C--   iq        (in)      q2 grid-point
C--   nprint    (in)      print first n comparisons
C--   dif       (out)     max absolute difference encountered
C--   lun       (in)      lun to print dif (0 = no print)

      implicit double precision (a-h,o-z)

      dimension ww(*)

      idw = 1000*iset_pdfu + 501 + idf
      dif = 0.D0
      call grpars(nx,xmi,xma,nq,qmi,qma,iosp)
      do ix = 1,nx
        fff = fsnsij(1,idf,ix,iq,1)
        uuu = EvPdfij(ww,idw,ix,iq,1)
        ddd = uuu-fff
        dif = max(dif,abs(ddd))
        if(ix.le.nprint)
     +  write(lun,'(I4,2E13.5,F10.6,1PE13.3)') ix,uuu,fff,ddd,ddd/fff
      enddo
      if(lun.ne.0) then
        write(lun,'('' pdf'',I3,'' dif ='',(1PE10.3))') idf, dif
      endif

      return
      end

C--   ------------------------------------------------------------------
C--   Package routines
C--   ------------------------------------------------------------------

C     ============================================
      subroutine SetStart( iset, idf, iq0, start )
C     ============================================
      implicit double precision (a-h,o-z)
      dimension start(2,*)

      i = 1                    !quarks
      if( idf.eq.0 ) i = 2     !gluon
      call GRPARS(nx, xmi, xma, nq, qmi, qma, iord)
      do ix = 1,nx
        start(i,ix) = FSNSIJ(iset, idf, ix, iq0, 1)
      enddo

      return
      end

C     ================================================================
      subroutine EvolSGNS( w, iseta, isetp, isetf, idfin, iq0, start )
C     ================================================================
      implicit double precision (a-h,o-z)
      logical   AtabInvalid
      external  AlfasFun
      dimension w(*), start(2,*)
      parameter (nxmax = 200) !change this when you use a larger xgrid
      dimension sbuf(2,nxmax)
      dimension idw(2,2,1), ida(2,2,1), idf(2), iqlim(2)

      call GRPARS( nx, xmi, xma, nq, qmi, qma, iosp )
      if( nx.gt.nxmax) stop 'EvolSGNS: too many x-grid points'
      call getord(io)
      if(io.ne.1) stop 'EvolSGNS: can only handle LO'
      
      idalf      = 1000*iseta + 601
      idw(1,1,1) = 1000*isetp + 101           !PQQ
      idw(1,2,1) = 1000*isetp + 201           !PQG
      idw(2,1,1) = 1000*isetp + 102           !PGQ
      idw(2,2,1) = 1000*isetp + 202           !PGG
      ida(1,1,1) = idalf
      ida(1,2,1) = idalf
      ida(2,1,1) = idalf
      ida(2,2,1) = idalf
      idf(1)     = 1000*isetf + 501 + idfin   !quark table
      idf(2)     = 1000*isetf + 501           !gluon table 

      if(AtabInvalid()) call EVFILLA( w, idalf, AlfasFun )      

      n = 1                          !non-singlet evolution      
      if( idfin.eq.1 ) n = 2         !singlet/gluon evolution

      do i = 1,n
        do j = 1,nx
           sbuf(i,j) = start(i,j)    !copy start array
        enddo   
      enddo
      iqlim(2) = iq0
      nf       = 1
      do while( nf.gt.0 )            !upward evolution
        iqlim(1) = iqlim(2)
        iqlim(2) = 99999
        call EVDGLAP( w, idw, ida, idf, sbuf, 2, n, iqlim, nf, eps )
      enddo
      
      do i = 1,n                       
        do j = 1,nx
           sbuf(i,j) = start(i,j)    !copy start array
        enddo   
      enddo
      iqlim(2) = iq0
      nf       = 1
      do while( nf.gt.0 )            !downward evolution
        iqlim(1) = iqlim(2)
        iqlim(2) = -99999
        call EVDGLAP( w, idw, ida, idf, sbuf, 2, n, iqlim, nf, eps )
      enddo
          
      return
      end

C     =====================================================
      double precision function AlfasFun( iq, nf, ithresh )
C     =====================================================
      implicit double precision (a-h,o-z)

      idum = nf                    !avoid compiler warning
      if( ithresh .eq. -1 ) then
        AlfasFun = ALTABN( 0, -iq, 1, ierr )  !alfas/2pi
      else
        AlfasFun = ALTABN( 0,  iq, 1, ierr )  !alfas/2pi
      endif    

      return
      end

C     ==============================
      logical function AtabInvalid()
C     ==============================
      implicit double precision (a-h,o-z)
      save ioL, asL, r2L, nfL, qcL, qbL, qtL, arL, brL
      data ioL/0/, asL/-1.D0/, r2L/-1.D0/, nfL/9/, qcL/-1.D0/
      data qbL/-1.D0/, qtL/-1.D0/, arL/-1.D0/, brL/-1.D0/

      call GETORD( io )
      call GETALF( as, r2 )
      call GETCBT( nf, qc, qb, qt )
      call GETABR( ar, br)

      if(io.eq.ioL .and. as.eq.asL .and. r2.eq.r2L .and.
     &   nf.eq.nfL .and. qc.eq.qcL .and. qb.eq.qbL .and. qt.eq.qtL .and.
     &   ar.eq.arL .and. br.eq.brL) then
        AtabInvalid = .false.
      else
        AtabInvalid = .true.
        ioL = io
        asL = as
        r2L = r2
        nfL = nf
        qcL = qc
        qbL = qb
        qtL = qt
        arL = ar
        brL = br
      endif

      return
      end

C     ======================================
      subroutine FillWtU( w, nw, iset_piju )
C     ======================================
      implicit double precision (a-h,o-z)
      dimension w(nw)
      dimension itypes(6)
      data itypes / 2, 2, 0, 0, 0, 0 /

      external ACHI,PQQR,PQQS,PQQD,PQGA,PGQA,PGGR,PGGS,PGGA,PGGD
      
      call MAKETAB( w, nw, itypes, 0, 0, iset_piju, nwused ) 

C--   Fill unpolarised weight tables

      call getint('lunq',lun)
      write(lun,'(/'' FILLWTU: start unpolarised weight calculation'')')

      idPQQ = 1000*iset_piju + 101
      idPQG = 1000*iset_piju + 201
      idPGQ = 1000*iset_piju + 102
      idPGG = 1000*iset_piju + 202
      
      call MAKEWRS( w, idPQQ, PQQR, PQQS, ACHI, 0 )
      call MAKEWTD( w, idPQQ, PQQD, ACHI )
      
      call MAKEWTA( w, idPQG, PQGA, ACHI )
      
      call MAKEWTA( w, idPGQ, PGQA, ACHI )
      
      call MAKEWRS( w, idPGG, PGGR, PGGS, ACHI, 0 )
      call MAKEWTA( w, idPGG, PGGA, ACHI )
      call MAKEWTD( w, idPGG, PGGD, ACHI )

      write(lun,'( '' FILLWTU: end   unpolarised weight calculation'')')
      
      return
      end

C     ======================================
      subroutine FillWtP( w, nw, iset_pijp )
C     ======================================
      implicit double precision (a-h,o-z)
      dimension w(nw)
      dimension itypes(6)
      data itypes / 2, 2, 0, 0, 0, 0 / 
      
      call MAKETAB(w, nw, itypes, 0, 0, iset_pijp, nwused)

C--   Put here the code to fill polarised weight tables

      return
      end

C--   ------------------------------------------------------------------
C--   Unpolarised splitting functions
C--   ------------------------------------------------------------------

C     ====================================
      double precision function ACHI(qmu2)
C     ====================================
      implicit double precision (a-h,o-z)
      ddum = qmu2          !avoid compiler warning
      ACHI = 1.D0
      return
      end

C     =========================================
      double precision function PQQR(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = qmu2          !avoid compiler warning
      idum = nf            !avoid compiler warning
      PQQR = (4.D0/3.D0)*(1.D0+x*x)
      return
      end

C     =========================================
      double precision function PQQS(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = qmu2          !avoid compiler warning
      idum = nf            !avoid compiler warning
      PQQS = 1.D0/(1.D0-x)
      return
      end

C     =========================================
      double precision function PQQD(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = x             !avoid compiler warning
      ddum = qmu2          !avoid compiler warning
      idum = nf            !avoid compiler warning
      PQQD = 2.D0
      return
      end

C     =========================================
      double precision function PQGA(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = qmu2          !avoid compiler warning
      PQGA = nf * ( x**2 + (1.D0-x)**2 )
      return
      end

C     =========================================
      double precision function PGQA(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = qmu2          !avoid compiler warning
      idum = nf            !avoid compiler warning
      PGQA = (4.D0/3.D0) * ( 1.D0 + (1.D0-x)**2 ) / x
      return
      end

C     =========================================
      double precision function PGGR(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = qmu2          !avoid compiler warning
      idum = nf            !avoid compiler warning
      PGGR = 6.D0 * x
      return
      end

C     =========================================
      double precision function PGGS(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = qmu2          !avoid compiler warning
      idum = nf            !avoid compiler warning
      PGGS = 1.D0/(1.D0-x)
      return
      end

C     =========================================
      double precision function PGGA(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = qmu2          !avoid compiler warning
      idum = nf            !avoid compiler warning
      PGGA = 6.D0 * ( (1.D0-x)/x + x*(1.D0-x) )
      return
      end

C     =========================================
      double precision function PGGD(x,qmu2,nf)
C     =========================================
      implicit double precision (a-h,o-z)
      ddum = x             !avoid compiler warning
      ddum = qmu2          !avoid compiler warning
      PGGD = 11.D0/2.D0 - nf/3.D0
      return
      end

C--   ------------------------------------------------------------------
C--   Pdf start functions
C--   ------------------------------------------------------------------

C     ======================================
      double precision function func(ipdf,x)
C     ======================================

      implicit double precision (a-h,o-z)

                     func = 0.D0
      if(ipdf.eq. 0) func = xglu(x)
      if(ipdf.eq. 1) func = xdnv(x)
      if(ipdf.eq. 2) func = xupv(x)
      if(ipdf.eq. 3) func = 0.D0
      if(ipdf.eq. 4) func = xdbar(x)
      if(ipdf.eq. 5) func = xubar(x)
      if(ipdf.eq. 6) func = xsbar(x)
      if(ipdf.eq. 7) func = 0.D0
      if(ipdf.eq. 8) func = 0.D0
      if(ipdf.eq. 9) func = 0.D0
      if(ipdf.eq.10) func = 0.D0
      if(ipdf.eq.11) func = 0.D0
      if(ipdf.eq.12) func = 0.D0

      return
      end
 
C     =================================
      double precision function xupv(x)
C     =================================

      implicit double precision (a-h,o-z)

      data au /5.107200D0/

      xupv = au * x**0.8D0 * (1.D0-x)**3.D0

      return
      end

C     =================================
      double precision function xdnv(x)
C     =================================

      implicit double precision (a-h,o-z)

      data ad /3.064320D0/

      xdnv = ad * x**0.8D0 * (1.D0-x)**4.D0

      return
      end

C     =================================
      double precision function xglu(x)
C     =================================

      implicit double precision (a-h,o-z)

      data ag    /1.7D0 /

      common /msum/ glsum, uvsum, dvsum

      xglu = ag * x**(-0.1D0) * (1.D0-x)**5.D0

      return
      end

C     ==================================
      double precision function xdbar(x)
C     ==================================

      implicit double precision (a-h,o-z)

      data adbar /0.1939875D0/

      xdbar = adbar * x**(-0.1D0) * (1.D0-x)**6.D0

      return
      end

C     ==================================
      double precision function xubar(x)
C     ==================================

      implicit double precision (a-h,o-z)

      xubar = xdbar(x) * (1.D0-x)

      return
      end

C     ==================================
      double precision function xsbar(x)
C     ==================================

      implicit double precision (a-h,o-z)

      xsbar = 0.2D0 * (xdbar(x)+xubar(x))

      return
      end







